/*
Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
This file is part of Pyrame.

Pyrame is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Pyrame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
*/

#include "libacq.h"


void open_replay_file(struct acqunit_workspace *auws) {
  char name[4096];
  if (auws->replay_enabled) {
    sprintf(name,"%s/running_data_au_%s.replay",auws->datadir,auws->name);
    auws->replay_file=open(name,O_WRONLY|O_CREAT|O_TRUNC,0666);
    if (auws->replay_file==-1) {
      printf("warning : acq : cant open replay file %s",name);
    }
  } else {
    auws->replay_file=-1;
  }
}

void close_replay_file(struct acqunit_workspace *auws) {
  if (auws->replay_file!=-1)
    close(auws->replay_file);
  auws->replay_file=-1;
}


int opennew_files(struct acqunit_workspace *auws) {
  int i;
  char name[4096];

  open_replay_file(auws);

  printf("opennewfiles for nb_streams=%d\n",auws->nb_streams);
  for (i=0;i<auws->nb_streams;i++) {
    sprintf(name,"%s/running_data_%s.raw",auws->datadir,auws->streams[i].name);
    debug("opening file %s\n",name);
    auws->streams[i].output=open(name,O_WRONLY|O_CREAT|O_TRUNC,0666);
    if (auws->streams[i].output==-1) {
      printf("cant open output %s : \n",name);
      perror("open");
      return -1;
    }
  }
  return 0;
}

void close_files(struct acqunit_workspace *auws) {
  int i;

  close_replay_file(auws);
  
  for (i=0;i<auws->nb_streams;i++) {
    if (auws->streams[i].output!=-1)
      close(auws->streams[i].output); 
    auws->streams[i].output=-1;
  }
}

void move_files(char *prefix,struct acqunit_workspace *auws) {
  int i;
  int res;
  char src[4096];
  char dst[4096];

  //moving replay file
  if (auws->replay_enabled) {
    sprintf(src,"%s/running_data_au_%s.replay",auws->datadir,auws->name);
    sprintf(dst,"%s/%s_au_%s.replay",auws->datadir,prefix,auws->name);
    res=rename(src,dst);
    if (res<0) {
      printf("warning : acq : cant rename %s to %s\n",src,dst);
      perror("rename");
    }
  }
  
  //moving stream files
  for (i=0;i<auws->nb_streams;i++) {
    sprintf(src,"%s/running_data_%s.raw",auws->datadir,auws->streams[i].name);
    sprintf(dst,"%s/%s_%s.raw",auws->datadir,prefix,auws->streams[i].name);
    res=rename(src,dst);
    if (res<0) {
      printf("warning : acq : cant rename %s to %s\n",src,dst);
      perror("rename");
    }
  }
}

void convert_files(char *acq_name, char *param1, char *param2, char *param3, char *param4,char *param5,char *param6,struct acqunit_workspace *auws) {
  int i;
  int res;
  struct incr_string *scommand=new_is();

  if (!strcmp(auws->convert_script,"none")) {
    printf("no convert script: skipping conversion\n");
    return;
  }

  //build convert script command:
  //prototype : convert_script acq_name param1 param2 param3 param4 param5 suffix1 suffix2 ... suffixN

  //replay file
  fsuffix_is(scommand,"%s %s %s",auws->convert_script,auws->datadir,acq_name);
  
  //params
  fsuffix_is(scommand," %s %s %s %s %s %s",param1,param2,param3,param4,param5,param6);
  
  if (auws->replay_enabled) {
    fsuffix_is(scommand," _au_%s.replay",auws->name);
  }

  //stream files
  for (i=0;i<auws->nb_streams;i++) {
    fsuffix_is(scommand," _%s.raw",auws->streams[i].name);
  }
  fsuffix_is(scommand," &");
  printf("launching conversion script: %s\n",scommand->str);
  res=system(scommand->str);
  if (res<0) {
    perror("system");
  }
  free_is(scommand);
}
  

void flush_files(struct acqunit_workspace *auws,char *acq_name, char *param1, char *param2, char *param3, char *param4,char *param5,char *param6) {
  printf("flushing and renaming cache file\n");
  pthread_mutex_lock(auws->file_mutex);
  close_files(auws);
  move_files(acq_name,auws);
  system("sync");
  convert_files(acq_name,param1,param2,param3,param4,param5,param6,auws);
  opennew_files(auws);
  pthread_mutex_unlock(auws->file_mutex);
  printf("end of flush files\n");
}

void cache_init(struct consumer_workspace *ws) {
  //struct acq_workspace *gws=(struct acq_workspace *)ws->gws;
}

void cache_deinit(struct consumer_workspace *ws) {
  //struct acq_workspace *gws=(struct acq_workspace *)ws->gws;
}

//This is the treatment done on every data packet to store data in files
int cache_treatment(struct data_descriptor *ad,struct consumer_workspace *ws) {
  struct acq_workspace *gws=(struct acq_workspace *)ws->gws;
  int stream=ad->streamid;
  int res;
  int output;

  //printf("cache treatment for stream %d with size %d\n",stream,ad->data_size);

  if (ad->auws==NULL)
    return 1;
  
  //take the mutex to avoid write while flushing
  pthread_mutex_lock(ad->auws->file_mutex);

  //if necessary save the replay data
  if (ad->auws->replay_enabled) {
    //write the size of the packet in the replay   
    res=write(ad->auws->replay_file,&ad->generic_buffer->size,sizeof(int));
    if (res==0)
      printf("error : nothing written\n");
    if (res==-1) {
      printf("error : cant write on replay file\n");
      perror("write");
      pthread_mutex_unlock(ad->auws->file_mutex);
      return -1;
    }
    //write the packet in replay
    res=write(ad->auws->replay_file,ad->generic_buffer->data,ad->generic_buffer->size);
    if (res==0)
      printf("error : nothing written\n");
    if (res==-1) {
      printf("error : cant write on replay file\n");
      perror("write");
      pthread_mutex_unlock(ad->auws->file_mutex);
      return -1;
    }
  }
  
  if (stream==-1) { //goto trash
    printf("error : data going to trash %d\n",ad->generic_buffer->size);
  } else { //normal file writing
    output=ad->auws->streams[stream-ad->auws->fid].output;
    if (output>0) { //write data in specific bystream file
      res=write(output,ad->data,ad->data_size);
      if (res==0)
        printf("error : nothing written\n");
      if (res==-1) {
	printf("error : cant write on streamfile %d\n",stream-ad->auws->fid);
        perror("write");
        pthread_mutex_unlock(ad->auws->file_mutex);
        return -1;
      }
      if (res>0) {
        set_stats(gws->stats,"bytes_on_dsk",SOP_PLUS,ad->data_size);
      }
    } else {
        printf("error : Data should be written but file is closed\n");
    }
  }
  
  pthread_mutex_unlock(ad->auws->file_mutex);

  //autoflush
  if ((gws->autoflush_active) && (gws->rawsize>gws->autoflush_limit*1024*1024)) {
    printf("autoflush\n");
    flush_files(ad->auws,"auto_flush",NULL,NULL,NULL,NULL,NULL,NULL);
    set_stats(gws->stats,"bytes_on_dsk",SOP_SET,0);
  }
  return 1;

}

