/*
Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
This file is part of Pyrame.

Pyrame is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Pyrame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
*/

#include "libacq.h"

void data_socket_init(struct consumer_workspace *ws) {
  sigset_t sigpipe_mask;
  sigemptyset(&sigpipe_mask);
  sigaddset(&sigpipe_mask, SIGPIPE);
  sigset_t saved_mask;
  if (pthread_sigmask(SIG_BLOCK, &sigpipe_mask, &saved_mask) == -1) {
    perror("pthread_sigmask");
    exit(1);
  }
}

void opennew_socket(struct acqunit_workspace * auws) {
  int i;
  int port;
  for (i=0;i<auws->nb_streams;i++) {
    if (auws->streams[i].socket==NULL) {
      port=submod_getport("DATASTREAM_BASE_PORT")+i+auws->fid;
      printf("adding socket for stream %d on au %s on port %d\n",i,auws->name,port);
      auws->streams[i].socket=netserv_add_new_port(auws->globalws->nw,port);
    }
    if (auws->streams[i].socket==NULL) {
      printf("cant add new port to netserv\n");
      exit(1);
    }
  }
}

void close_socket(struct acqunit_workspace * auws) {
  int i;
  int port;
  for (i=0;i<auws->nb_streams;i++) {
    port=submod_getport("DATASTREAM_BASE_PORT")+i+auws->fid;
    debug("closing port %d\n",port);
    netserv_remove_port(auws->globalws->nw,port);
  }
}



void data_socket_deinit(struct consumer_workspace *ws) {
}

int data_socket_treatment(struct data_descriptor *ad,struct consumer_workspace *ws) {
  int nb_send;
  struct acq_workspace *gws=(struct acq_workspace *)ws->gws;
  struct netserv_client *socket;

  if (ad->auws==NULL)
    return 1;

  //printf("data_socket treatment\n");
  if (ad->streamid!=-1) {
    socket=ad->auws->streams[ad->streamid-ad->auws->fid].socket;
    if (socket==NULL) {
      printf("error : no socket found for streamid %d in au %s with fid %d\n",ad->streamid,ad->auws->name,ad->auws->fid);
      return -1;
    }
    nb_send=send2client(socket,ad->data,ad->data_size);
    if (nb_send<0) {
      return -1;
    }
    //printf("nb_send=%d\n",nb_send);
    set_stats(gws->stats,"bytes_on_skt",SOP_PLUS,nb_send*ad->data_size);
  } else {
    printf("warning : stream id is -1 for this packet\n");
  }
  return 1;
}

