/*
Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
This file is part of Pyrame.

Pyrame is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Pyrame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
*/


#include "libacq.h"


void shmem_init(struct consumer_workspace *ws) {
  
}


void flush_shmem(struct acqunit_workspace * auws) {
  int i;
  for (i=0;i<auws->nb_streams;i++) {
    if (auws->streams[i].shdata==NULL) {
      debug("flushing  shbuffer for stream %d in acqunit %s\n",i,auws->name);
      flush_shbuffer(auws->streams[i].shdata);
    }
  }
}


void opennew_shmem(struct acqunit_workspace * auws,int timeout) {
  int i;
  int shbuf_index;
  for (i=0;i<auws->nb_streams;i++) {
    shbuf_index=i+auws->fid;
    if (auws->streams[i].shdata==NULL) {
      debug("creating new shbuffer %d for stream %d in acqunit %s\n",shbuf_index,i,auws->name);
      auws->streams[i].shdata=open_shbuffer(shbuf_index,CREATE,timeout);
    } else {
      debug("already existing shbuffer for stream %d in acqunit %s\n",shbuf_index,i,auws->name);
    }
    //the shmem should only be ready by the client
    //ready_shbuffer(auws->streams[i].shdata);
  }
}

void close_shmem(struct acqunit_workspace * auws) {
  int i;
  int shbuf_index;
  for (i=0;i<auws->nb_streams;i++) {
    shbuf_index=i+auws->fid;
    if (auws->streams[i].shdata==NULL) {
      debug("closing shbuffer %d for stream %d in acqunit %s\n",shbuf_index,i,auws->name);
      close_shbuffer(auws->streams[i].shdata);
    }
  }
}



 
void shmem_deinit(struct consumer_workspace *ws) {
}



int shmem_treatment(struct data_descriptor *ad,struct consumer_workspace *ws) {
 
  struct acq_workspace *gws=(struct acq_workspace *)ws->gws;
  int res;
  struct shbuffer *cshbuf;
  int shbuf_id;

  if (ad->auws==NULL)
    return 1;

  shbuf_id=ad->streamid-ad->auws->fid;
  cshbuf=ad->auws->streams[shbuf_id].shdata;
  if (cshbuf==NULL) {
    printf("error : null shbuffer %d for streamid %d in au %s with fid %d\n",shbuf_id,ad->streamid,ad->auws->name,ad->auws->fid);
    return -1;
  }
  if (ad->streamid!=-1) {
    if (isready_shbuffer(cshbuf)) {
      //debug("writing to shbuffer %d in au %s\n",cshbuf->portid,ad->auws->name);
      res=write_shbuffer(cshbuf,(char *)ad->data,ad->data_size);
      if (res==-1) {
	printf("error: cant write to shbuf due to mutex error\n");
	return -1;
      } else {
	set_stats(gws->stats,"bytes_on_shm",SOP_PLUS,ad->data_size);
      }
    }
  } else {
    printf("error: ad->streamid is -1\n");
  }
      
  return 1;
}

