/*
Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
This file is part of Pyrame.

Pyrame is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Pyrame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
*/

#include "bindpyrame_py.h"


static PyObject* open_socket_py(PyObject *self, PyObject *args) {
  int res;
  char *host;
  int port;
  int sockfd;
  PyThreadState *_save;
  res=PyArg_ParseTuple(args,"si",&host,&port);
  if (!res) {
    return NULL;
  }
  _save=PyEval_SaveThread();
  sockfd=open_socket(host,port);
  PyEval_RestoreThread(_save);
  return Py_BuildValue("i",sockfd);
}

static PyObject* get_port_py(PyObject *self, PyObject *args) {
  char *name;
  int res;
  int port;
  PyThreadState *_save;
  res=PyArg_ParseTuple(args,"s",&name);
  if (!res) {
    return NULL;
  }
  _save=PyEval_SaveThread();
  port=get_port(name);
  PyEval_RestoreThread(_save);
  return Py_BuildValue("i",port);
}

struct cmd *cmd_from_args(char *name,PyObject *args,int offset) {
  struct cmd *command=malloc(sizeof(struct cmd));
  int i;
  PyObject *pyres;
  command->name=strdup(name);
  command->nb_params=PyTuple_Size(args)-offset;
  command->params=malloc(sizeof(char*)*command->nb_params);
  for (i=0;i<command->nb_params;i++) {
    pyres=PyTuple_GetItem(args,i+offset);
    if (pyres==NULL) {
      printf("Error : cant get python parameter number %d\n",i);
      free(command->params);
      free(command);
      return NULL;
    }
    command->params[i]=strdup(PyString_AsString(pyres));
    if (command->params[i]==NULL) {
      printf("Error : cant get python parameter number %d\n",i);
      free(command->params);
      free(command);
      return NULL;
    }
  }
  return command;
}

static PyObject* execcmd_py(PyObject *self, PyObject *args) {
  char *name;
  int sock;
  int res;
  struct cmd_result *result;
  struct cmd *command;
  PyObject *py_result;
  int offset=2;
  PyThreadState *_save;
  res=PyArg_ParseTuple(PyTuple_GetSlice(args,0,offset),"is",&sock,&name);
  if (!res) {
    return NULL;
  }
  command=cmd_from_args(name,args,offset);
  if (command==NULL) {
    return NULL;
  }
  _save=PyEval_SaveThread();
  result=exec_cmd(sock,command);
  PyEval_RestoreThread(_save);
  free_cmd(command);
  py_result=Py_BuildValue("is",result->retcode,result->str);
  free_cmd_result(result);
  return py_result;
}

static PyObject* sendcmd_py(PyObject *self, PyObject *args) {
  char *host;
  int port;
  char *name;
  int res;
  struct cmd *command;
  struct cmd_result *result;
  int offset=3;
  PyThreadState *_save;
  PyObject *py_result;
  res=PyArg_ParseTuple(PyTuple_GetSlice(args,0,offset),"sis",&host,&port,&name);
  if (!res) {
    return NULL;
  }
  command=cmd_from_args(name,args,offset);
  if (command==NULL) {
    return NULL;
  }
  _save=PyEval_SaveThread();
  result=send_cmd(host,port,command);
  PyEval_RestoreThread(_save);
  free_cmd(command);
  py_result=Py_BuildValue("is",result->retcode,result->str);
  free_cmd_result(result);
  return py_result;
}

#ifndef PyMODINIT_FUNC
#define PyMODINIT_FUNC void
#endif

PyMODINIT_FUNC initbindpyrame(void) {
  PyObject * m;
  m=Py_InitModule3("bindpyrame",bindpyrame_py_methods,"Python binding to Pyrame");
  if (m == NULL)
    return;
}

