/*
Copyright 2012-2015 Frédéric Magniette, Miguel Rubio-Roy
This file is part of Pyrame.

Pyrame is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Pyrame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
*/

//version 3
#include "cmdmod.h"

//global variable workspace for external functions
struct cmdmod_ws *ws;

struct cmdmod_ws *cmdmod_getenv() {
  return ws;
}


//*************************** NETSERV FUNCTIONS *************************


int cmdmod_aeh(void **vws,struct netserv_client *client) {
  struct cmd_result *cres;
  if (*vws==NULL) {
    printf("probably some previous module use the port, try to kill it smoothly\n");
    cres=sendcmd("localhost",client->port,"exit_module","end");
    if (cres->retcode!=1) {
      printf("error in killing smoothly the module : %s\n",cres->str);
      free_cmd_result(cres);
      return 0;
    } else {
      printf("module successfully killed\n");
      *vws=(void *)1;
      return 1;
    }
  } else {
    printf("module has already been killed...waiting for him to release the socket\n");
    usleep(200000);
    return 1;
  }
  
  
}

void init_netserv(struct cmdmod_ws *ws) {
  char *str_port=env_getvar(ws->env,"system","listen_port");
  unsigned short int_port;
  char *converted_port=malloc(1024);

  //if no port declared use the module name
  if ((str_port==NULL) || (!strcmp(str_port,"none"))) {
    str_port=env_getvar(ws->env,"system","name");
  }
  int_port=get_port(str_port);
  sprintf(converted_port,"%d",int_port);
  if (int_port==65535) {
    printf("no port was found for description %s...exiting\n",str_port);
    cexit(1);
  }
  env_setvar(ws->env,"system","listen_port",converted_port,ENV_INT,0);
  //ws->clients=first_client(start_monoport_netserv(int_port,10,cmdmod_aeh));
  ws->clients=first_client(start_monoport_netserv(int_port,50));
  //debug("listening on port %d\n",int_port);
  free(converted_port);
}


//****************************** XML FUNCTIONS ****************************


void config_handler_start(void *data, const char *el, const char **attr) {
  /* this function is called when a begin tag is found.
     The corresponding field in the workspace is initialized.
     data is the cmdmod workspace
     el is the name of the tag
     attr is an array of the tag attributes
     We expect the first attribute to be the name.
  */
  
  struct cmdmod_ws *ws=(struct cmdmod_ws *)data;
  ws->next_data=NEXT_IGNORE;
  
  //new cmd start
  if (!strcmp(el,"cmd")) {
    ws->current_function=(struct function *)malloc(sizeof(struct function));
    memset(ws->current_function,0,sizeof(struct function));
    ws->current_function->type=SUBMOD_TYPE_NONE;
    //the first attribute should be the name of the function
    ws->current_function->name=strdup(attr[1]);
    if (!strcmp(attr[3],"host"))
      ws->current_function->type=SUBMOD_TYPE_HOST;
    if (!strcmp(attr[3],"script"))
      ws->current_function->type=SUBMOD_TYPE_SCRIPT;
  }
  
  //name start found
  if (!strcmp(el,"name")) { 
    ws->next_data=NEXT_NAME;
  }

  //host start found
  if (!strcmp(el,"host")) { 
    ws->next_data=NEXT_HOST;
    ws->current_function->host=malloc(1);
    *ws->current_function->host=0;
  }

   //host start found
  if (!strcmp(el,"language")) { 
    ws->next_data=NEXT_LANG;
  }

  //begin port start found
  if (!strcmp(el,"port")) {
    ws->next_data=NEXT_PORT;
    ws->current_function->portstring=malloc(1);
    *ws->current_function->portstring=0;
  }

  //begin of listen_port found
  if (!strcmp(el,"listen_port")) {
    ws->next_data=NEXT_LPORT;
  }
  
  //begin of port_base found: deprecated -> ignore corresponding data
  if (!strcmp(el,"port_base")) {
    ws->next_data=NEXT_IGNORE;
  }

  //begin of code file found
  if (!strcmp(el,"file")) {
    ws->next_data=NEXT_FILE;
  }
}

void config_handler_end(void *data, const char *el) {
  /* this function is called when an end tag is found
     data is the cmdmod workspace
     el is the name of the tag
  */
  
  struct cmdmod_ws *ws=(struct cmdmod_ws *)data;
  
  //next data has to be ignored until a new start is discovered
  ws->next_data=NEXT_IGNORE;

  //end of cmd found ->registering in db
  if (!strcmp(el,"cmd")) {
    if (ws->current_function->type==SUBMOD_TYPE_HOST) {
      ws->current_function->conn=get_conn(ws,ws->current_function->host,get_port(ws->current_function->portstring));
      ws->current_function->port=get_port(ws->current_function->portstring);
      add_func(ws->hfdb,ws->current_function);
    }
    if (ws->current_function->type==SUBMOD_TYPE_SCRIPT) {
      add_func(ws->sfdb,ws->current_function);
    }
  }

  /*//end of name found
  if (!strcmp(el,"name")) {
    output(ws,"module name: %s\n",env_getvar(ws->env,"system","name"));
  }

  //end of language found
  if (!strcmp(el,"language")) {
    output(ws,"found language: %s\n",env_getvar(ws->env,"system","language"));
  }

  //end of code file found
  if (!strcmp(el,"file")) {
    output(ws,"found file: %s\n",env_getvar(ws->env,"system","code_file_name"));
    }*/

  return;
}

void config_handler_data(void *data,const char *value,int len) {
  /* this function is called when a data between tags is found
     the ws->next_data variable store the nature of this data
     to store it in the good place.
     data is the cmdmod workspace
     value is the data string or a part of it (if total data > 4096 bytes)
     len is the size of value
  */
  
  struct cmdmod_ws *ws=(struct cmdmod_ws *)data;
  int len0=0;
  char *tmp;
  
  switch(ws->next_data) {
    
    //module name found
  case NEXT_NAME:
    tmp=strndup(value,len);
    env_setvar(ws->env,"system","name",tmp,ENV_STR,0);
    free(tmp);
    break;
    
    //host name found
  case NEXT_HOST:
    len0=strlen(ws->current_function->host);
    ws->current_function->host=realloc(ws->current_function->host,len0+len+1);
    strncat(ws->current_function->host,value,len);
    break;

    //language name found
  case NEXT_LANG:
    tmp=strndup(value,len);
    env_setvar(ws->env,"system","language",tmp,ENV_STR,0);
    free(tmp);
    break;
    
    //port number found
  case NEXT_PORT:
    len0=strlen(ws->current_function->portstring);
    ws->current_function->portstring=realloc(ws->current_function->portstring,len0+len+1);
    strncat(ws->current_function->portstring,value,len);
    break;
    
    //code file found
  case NEXT_FILE:
    tmp=strndup(value,len);
    env_setvar(ws->env,"system","code_file_name",tmp,ENV_STR,0);
    free(tmp);
    break;
    
    //listening port found
  case NEXT_LPORT:
    tmp=strndup(value,len);
    env_setvar(ws->env,"system","listen_port",tmp,ENV_INT,0);
    free(tmp);
    break;
  }
}

void parse_config(char * filename,struct cmdmod_ws *ws) {
  XML_Parser parser = XML_ParserCreate(NULL);
  FILE *f;
  char buf[4096]; 
  int len;
  int res;


  //preparing the parser
  XML_SetUserData(parser,ws);
  XML_SetElementHandler(parser,config_handler_start,config_handler_end);
  XML_SetCharacterDataHandler(parser,config_handler_data);

  //open the xml file to parse
  f=fopen(filename,"r");
  if (f==NULL) {
    printf("unable to open the xml file %s\n",filename);
    perror("fopen");
    cexit(1);
  }
  
  //parsing the file
  while(!feof(f)) {
    len=fread(buf,1,4095,f);
    buf[len]=0;
    if (feof(f))
      res=XML_Parse(parser,buf,len,1);
    else
      res=XML_Parse(parser,buf,len,0);
    if (!res) {
      printf("Error %d in XML_Parse\n",XML_GetErrorCode(parser));
      printf("Error msg: %s\n",XML_ErrorString(XML_GetErrorCode(parser)));
      printf("Error buffer: %s\n",buf);
    }
  }
  XML_Parse(parser,buf,0,1);

  //freeing the parser and the file
  XML_ParserFree(parser);
  fclose(f);
}

//************************ WORKSPACE WRAPPER FUNCTIONS ************************

//wrapper for treat_extcmd without the workspace
struct cmd_result * treat_cmd(struct cmd *command) {
  return treat_extcmd(ws,command);
}

//************************** OUTPUT FUNCTIONS *******************************

void output(struct cmdmod_ws *ws,const char *format,...) {
  char timebuf[100];
  //time_t now=time(0);
  struct timeval now;
  gettimeofday(&now,NULL);
  va_list args;
  va_start(args,format);
  strftime(timebuf,100,"%H:%M:%S",localtime(&now.tv_sec));
  dprintf(ws->old_out,"%s: %s : ",env_getvar(ws->env,"system","name"),timebuf);
  vdprintf(ws->old_out,format,args);
  va_end(args);
}

void debug(const char *format, ...) {
  va_list args;
  if (ws->debug) {
    va_start(args,format);
    vprintf(format,args);
    va_end(args);
  }
}

void *thread_output(void *workspace) {
  size_t tmp_s;
  char *tmp=NULL;
  struct cmdmod_ws *ws=(struct cmdmod_ws *)workspace;
  FILE *fp=fdopen(ws->pipe_out[0],"r");
  while(1) {
    while(getline(&tmp,&tmp_s,fp)!=-1) {
      output(ws,"%s",tmp);
    }
    if (tmp!=NULL) {
      free(tmp);
    }
  }
  return NULL;
}

void cexit(int nb) {
  sleep(1);
  exit(nb);
}

char *get_caller_ip() {
  struct sockaddr_in adr_inet;
  unsigned int len_inet;
  int ret;
  len_inet=sizeof(adr_inet);
  ret=getsockname(ws->current_socket,(struct sockaddr *)&adr_inet,&len_inet);
  if (ret<0) {
    return strdup("");
  }
  return strdup(inet_ntoa(adr_inet.sin_addr));
}

//************************** PYRAME FUNCTIONS *******************************

void wait_for_cmd(struct netserv_client * clients,struct cmd_result *(*treat_func)(struct cmdmod_ws *,struct cmd *)) {

  fd_set readfds;
  int highest;
  struct timeval tv;
  long unsigned int command_s=4096;
  char * command=malloc(command_s);
  int res;
  int size;
  int fres;
  int i;
  long unsigned int j;
  int end_found=0;
  int k;
  struct cmd *toexec;

  struct cmd_result *cmdres;
 

  FD_ZERO(&readfds);
  if (clients->control_socket != NULL) {
    FD_SET(clients->control_socket[0],&readfds);
    highest=clients->control_socket[0];
  } else { highest = 0; }
  for(i=0;i<clients->nb_sock_client;i++) {
    if (clients->sock_client[i]!=-1) {
      FD_SET(clients->sock_client[i],&readfds);
      if (highest<=clients->sock_client[i])
        highest=clients->sock_client[i];
    }
  }
  tv.tv_sec=0;
  tv.tv_usec=UCMD_DELAY;
  if (select(highest+1,&readfds, NULL, NULL, &tv)>=0 ) {
    if (clients->control_socket != NULL) {
      if (FD_ISSET(clients->control_socket[0],&readfds)) {
        fres=read(clients->control_socket[0],command,4096); // flush pipe
        if (fres<0) {
          printf("ERROR : cant flush pipe\n");
        }
      }
    }
    for (i=0;i<clients->nb_sock_client;i++) {
      if (clients->sock_client[i]!=-1) {
        if (FD_ISSET(clients->sock_client[i],&readfds)) {
          ws->current_socket=clients->sock_client[i];
          ws->current_index=i;
          j=0;
          end_found=0;
          do {
            if (j+1024>=command_s) {
              command_s+=4096;
              command=realloc(command,command_s);
            }
            size=read(clients->sock_client[i],command+j,1024);
            if (size<=0) {
              close(clients->sock_client[i]);
              clients->sock_client[i]=-1;
              break;
            }
            for (k=j;k<j+size;k++) {
              if (command[k]=='\n') {
                end_found=1;
                command[k+1]=0;
              }
            }
            j+=size;
          } while (end_found==0);
          command[j]=0;
          if (size>0) {
            toexec=parse_command(command);
            if (toexec==NULL) {
              cmdres=raw_build_result(0,"Malformed command");
            } else {
              cmdres=treat_func(ws,toexec);
              free_cmd(toexec);
            }

            //check if the result has to be ignored
            if (cmdres->retcode!=KEEPOPEN) {

              res=send_result(clients->sock_client[i],cmdres);
              if (res<=0) {
                printf("cmd socket lost (send) %d\n",i);
                perror("write");
                close(clients->sock_client[i]);
                clients->sock_client[i]=-1;
              }
              free_cmd_result(cmdres);
            } else {
              //printf("keepopen\n");
            }
          }
        }
      }
    }
    free(command);
  } else {
    perror("wait_for_cmd select");
    free(command);
    /*if (errno==EINTR) {
      return;
    }*/
    exit(1);
  }
}

//************************** MAIN FUNCTION **********************************

void usage(char *executable) {
  printf("usage: %s config_file [-a arg1 arg2 ...]\n",executable);
  cexit(4);
}

void load_api(struct cmdmod_ws *ws) {
  char *code=env_getvar(ws->env,"system","code_file_name");
  int i;
  int code_size;
  char *api;
  FILE *fapi;
  char *line=NULL;
  size_t zero=0;
  char *name;
  char *sapi;
  int dp;
  struct function *newfunc;
  
  if (code==NULL) {
    printf("Unable to find code file name...exiting\n");
    cexit(1);
  }
  code_size=strlen(code);
  api=(char *)malloc(strlen(code)+50);
  for(i=code_size-1;i>=0;i--) {
    if (code[i]=='.')
      break;
  }
  if (i==0) {
    printf("no . found in code name %s\n",code);
    printf("cant load api...exiting\n");
    cexit(1);
  }
  strncpy(api,code,i+1);
  api[i+1]='a';
  api[i+2]='p';
  api[i+3]='i';
  api[i+4]=0;
  //printf("api=%s\n",api);

  fapi=fopen(api,"r");
  if (fapi==NULL) {
    perror("fopen");
    printf("cant load api...exiting\n");
    free(api);
    return;
  }
  free(api);
  while(getline(&line,&zero,fapi)>0) {
    line[strlen(line)-1]=0;
    //printf("line=%s\n",line);
    i=0;
    dp=0;
    while(line[i]!=0) {
      if (line[i]==':') {
        dp=i;
      }
      i++;
    }
    //printf("dp=%d\n",dp);
    if (dp==0) {
      printf("no \':\' in %s : \n",line);
    }
    name=strndup(line,dp);
    sapi=strdup(line+dp+1);
    //printf("extracted func name=%s api=%s\n",name,sapi);
    newfunc=(struct function *)malloc(sizeof(struct function));
    newfunc->type=SUBMOD_TYPE_SCRIPT;
    newfunc->name=name;
    newfunc->api=sapi;
    add_func(ws->sfdb,newfunc);
    free(line);
    line=NULL;
    zero=0;
  }
  free(line);
}


int main(int argc,char **argv) {

  int index;
  char *language;
  char lde;
  char *oldenv=NULL;

  //creating workspace
  ws=(struct cmdmod_ws *)malloc(sizeof(struct cmdmod_ws));
  memset(ws,0,sizeof(struct cmdmod_ws));

  //creating environment
  ws->env=new_cenv();
  env_newns(ws->env,"system");
  env_setvar(ws->env,"system","name","unknown module",ENV_STR,0);

  if (argc<2) {
    usage(argv[0]);
  }
  index=2;
  while(index<argc) {
    if (!strcmp(argv[index],"-d")) {
      ws->debug=1;
      //printf("debug mode\n");
      index++;
      continue;
    }
    if (!strcmp(argv[index],"-e")) {
      printf("loading old environment %s\n",argv[index+1]);
      lde=1;
      oldenv=argv[index+1];
      index+=2;
      continue;
    }
    if (!strcmp(argv[index],"-r")) {
      printf("register with cmod at %s\n",argv[index+1]);
      ws->env->registered=1;
      ws->cmod_ip=strdup(argv[index+1]);
      ws->cmod_port=get_port("cmod");
      index+=2;
      continue;
    }
    if (!strcmp(argv[index],"-a")) {
      ws->init_nb_args=argc-index-1;
      if (ws->init_nb_args!=0)
        ws->init_args=argv+index+1;  
      break;
    }
    usage(argv[0]);
    cexit(1);
  }

  //create dbs for funcs and connections
  ws->cdb=NULL;
  ws->hfdb=init_func_db();
  ws->sfdb=init_func_db();
  ws->cfdb=init_func_db();

  //create wrappers
  ws->we=new_wrapenv();

  //init exec mutex
  ws->exec_mutex=malloc(sizeof(pthread_mutex_t));
  pthread_mutex_init(ws->exec_mutex,NULL);

  //cmod init
  ws->local=0;
  ws->local_next_id=1;
  if (ws->cmod_ip!=NULL) {
    ws->cmod_port=get_port("cmod");
  }

  //load configuration from the xml file
  parse_config(argv[1],ws);

  //populate the internal function database
  init_ifdb(ws->sfdb,env_getvar(ws->env,"system","name"));

  //initialize the output redirection
  if (pipe(ws->pipe_out)<0) {
    printf("cant open out pipe...exiting\n");
    cexit(7);
  }
  ws->old_out=dup(STDOUT_FILENO);
  dup2(ws->pipe_out[1],STDOUT_FILENO);
  setbuf(stdout, NULL);
  
  //launch the output thread

  ws->t_output=(pthread_t*)malloc(sizeof(pthread_t));
  if (pthread_create(ws->t_output,NULL,thread_output,ws)<0) {
    perror("pthread_create");
    cexit(8);
  }
  pthread_detach(*ws->t_output);
  
  //initialize the language (python if nothing is specified)
  language=env_getvar(ws->env,"system","language");
  if (language==NULL) {
    language="python";
  }
    
  if (!strcmp(language,"python")) {
    ws->lang_spec_ws=init_python(ws);
    ws->deinit_func=deinit_python;
    ws->execcmd_func=exec_command_python;
  }
  if (!strcmp(language,"lua")) {
    ws->lang_spec_ws=init_lua(ws);
    ws->deinit_func=deinit_lua;
    ws->execcmd_func=exec_command_lua;
  }
  if (!strcmp(language,"sh")) {
    ws->lang_spec_ws=init_sh(ws);
    ws->deinit_func=deinit_sh;
    ws->execcmd_func=exec_command_sh;
  }
  if (!strcmp(language,"R")) {
    ws->lang_spec_ws=init_R(ws);
    ws->deinit_func=deinit_R;
    ws->execcmd_func=exec_command_R;
  }
  if (!strcmp(language,"c")) {
    ws->lang_spec_ws=init_c(ws);
    ws->deinit_func=deinit_c;
    ws->execcmd_func=exec_command_c;
  }
  if (ws->lang_spec_ws==NULL) {
    printf("unknown language %s...exiting\n",language);
    cexit(9);
  }
  //one can add new languages here: octave, julia or autoit linux alternative could be candidates

  //load the api in func db
  load_api(ws);
  
  //debug func db
  /*debug("host func db\n");
  debug_db(ws->hfdb);
  debug("script func db\n");
  debug_db(ws->sfdb);*/
    
  //prepare netserv
  init_netserv(ws);

  //load the old environment if necessary
  if (lde && oldenv!=NULL)
    env_load(ws->env,oldenv);

  printf("module %s language=%s file=%s port=%s up and running\n",env_getvar(ws->env,"system","name"),env_getvar(ws->env,"system","language"),env_getvar(ws->env,"system","code_file_name"),env_getvar(ws->env,"system","listen_port"));

  //waiting for commands
  while(1){ 
    wait_for_cmd(ws->clients,treat_intcmd);
  }

  return 0;
}
