#include "cmdmod_R.h"

char *escape_quote_R(char *src) {
  int i;
  int j;
  char *res;
  int size=strlen(src)+10;
  
  for (i=0;i<strlen(src);i++) {
    if (src[i]=='"')
      size+=4;
  }
  res=malloc(size);

  j=0;
  for (i=0;i<strlen(src);i++) {
    if (src[i]=='"') {
      res[j]='\\';
      res[j+1]='\\';
      res[j+2]='\\';
      res[j+3]='"';
      j+=4;
    } else {
      res[j]=src[i];
      j++;
    }
  }
  res[j]=0;
  return res;
}

char *build_args_R(int argc,char **argv) {
  struct incr_string *ires=new_is();
  char *esc_param;
  int i;
  fsuffix_is(ires,"(");
  for(i=0;i<argc;i++) {
    esc_param=escape_quote_R(argv[i]);
    if (i==0) {
      fsuffix_is(ires,"\\\"%s\\\"",esc_param);
    } else {
      fsuffix_is(ires,",\\\"%s\\\"",esc_param);
    }
    free(esc_param);
  }
  fsuffix_is(ires,")");
  return reduce_is(ires);
}

void *init_R(struct cmdmod_ws *cws) {
  /* this function is called by cmdmod if language is R
     cws is the cmdmod workspace
  */

  char *args;
  //create the language specific workspace
  struct R_ws *rws=malloc(sizeof(struct R_ws));
  memset(rws,0,sizeof(struct R_ws));

  //initialize the R interpreter
  char *vp[]={"R","--no-save",NULL};
  rws->iws=init_inter(cws,"/usr/bin/R",vp);

  //import the helper
  send2inter(rws->iws,"source(\"/opt/pyrame/cmdmod_helper.r\")\n");

  //load the code file
  send2inter(rws->iws,"source(\"%s\")\n",submod_getvar("system","code_file_name"));

  //exec init
  args=build_args_R(cws->init_nb_args,cws->init_args);
  send2inter(rws->iws,"internal_execcmd(\"init%s\")\n",args);
  free(args);
 
  //treat module output
  treat_output(rws->iws);
  
  //return workspace
  return (void *)rws;
}

void deinit_R(void *ws) {
  struct R_ws *rws=(struct R_ws *)ws;
  //execute deinit function
  send2inter(rws->iws,"internal_execcmd(\"deinit()\")\n");
  //quit R
  send2inter(rws->iws,"quit()\n");
  //treat module output
  treat_output(rws->iws);
}


struct cmd_result *exec_command_R(struct cmd *command,struct function *func,void *ws) {
  /* This function is called any time a pyrame command is received by cmdmod.
     command contains the name of the pyrame command and the parameters
     func contains the description of the function including its func name.
  */
  struct R_ws *rws=(struct R_ws *)ws;
  struct cmd_result *cres;
  
  //int i;
  struct incr_string *ires=new_is();
  char *args=build_args_R(command->nb_params,command->params);
  fsuffix_is(ires,"internal_execcmd(\"%s%s\")\n",func->name,args);
  free(args);
  
  cres=exec_command_inter(ires->str,rws->iws);
  free_is(ires);
  return cres;
}


