
#include "cmdmod.h"
#include "cmdmod_c.h"

struct cmd_result *submod_execcmd(char * funcname,...) {
  
  struct cmd_result *cres;
  va_list pa;
  char *param;
  int nbparams=0;

  //fill the command
  struct cmd *command=malloc(sizeof(struct cmd));
  command->name=funcname;

  //get the number of params
  va_start(pa,funcname); 
  while(1) { 
    param=va_arg(pa,char *);
    if (param==NULL) {
      printf("error : param is null should be end\n");
      break;
    }
    if (!strcmp(param,"end")) {
      break;
    } else {  
      nbparams++;
    }
  }
  va_end(pa);

  command->nb_params=nbparams;
  command->params=(char **)malloc(nbparams*sizeof(char *));

  nbparams=0;
  //fill the command with params
  va_start(pa,funcname); 
  while(1) { 
    param=va_arg(pa,char *);
    if (!strcmp(param,"end")) {
      break;
    } else {  
      command->params[nbparams]=param;
      nbparams++;
    }
  }
  va_end(pa);
  
  cres=treat_cmd(command);
  free(command->params);
  free(command);
  return cres;
}

void *init_c(struct cmdmod_ws *cws) {
  /* this function is called by cmdmod if language is c
     cws is the cmdmod workspace
  */

  void * (*dlfunc)(int,char **);
  //create the language specific workspace
  struct c_ws *lcws=malloc(sizeof(struct c_ws));
  memset(lcws,0,sizeof(struct c_ws));

  //load the code file
  lcws->lib=dlopen(submod_getvar("system","code_file_name"),RTLD_NOW|RTLD_GLOBAL);
  if (!lcws->lib) {
    printf("Error with dlopen : %s, unable to open %s\n",dlerror(),submod_getvar("system","code_file_name"));
    cexit(21);
  }

  //calling the init function if present to initialize the module workspace
  dlfunc=dlsym(lcws->lib,"init");
  if (!dlfunc) {
    debug("No init function found\n");
    lcws->mod_ws=NULL; 
  } else {
    if (cws->init_nb_args!=0) {
      lcws->mod_ws=dlfunc(cws->init_nb_args,cws->init_args);
    } else {
      lcws->mod_ws=dlfunc(0,NULL);
    }
  }

  return (void *)lcws;
}

void deinit_c(void *ws) {
  struct c_ws *lcws=(struct c_ws *)ws;
  void (*dlfunc)(void *);
  dlfunc=dlsym(lcws->lib,"deinit");
  if (!dlfunc) {
    debug("no deinit symbol\n");
    return;
  }
  dlfunc((void *)lcws->mod_ws);
}

struct cmd_result *exec_command_c(struct cmd *command,struct function *func,void *ws) {
  /* This function is called any time a pyrame command is received by cmdmod.
     command contains the name of the pyrame command and the parameters
     func contains the description of the function including its func name.
  */
  struct c_ws *lcws=(struct c_ws *)ws;
  struct cmd_result *(*dlfunc)(struct cmd *,void *);
  dlfunc=dlsym(lcws->lib,func->name);
  if (!dlfunc) {
    debug("Error : unable to load symbol %s : %s\n",func->name,dlerror());
    return 0;
  }
  return dlfunc(command,lcws->mod_ws);
}

