#include "cmdmod.h"

void *serializer(void *workspace) {
  struct dispatch_ws *ws=(struct dispatch_ws*)workspace;
  struct circ_bl_elem *cbl=ws->cbl;
  while (1) {
    sem_wait(&(ws->sem_serializer));
    pthread_mutex_lock(ws->evt_mutex);
    if (cbl->block && cbl->block->finalized && cbl->block->bstr==NULL) {
      cbl->block->bstr=saf_str_block(cbl->block);
    }
    cbl=cbl->next;
    pthread_mutex_unlock(ws->evt_mutex);
  }
  return NULL;
}

//***************************** SUBMOD DISPATCHER FUNCTIONS ************

int init_dispatcher(struct cmdmod_ws *ws,int nb_blocks,char *name,char *schema_str,char *port) {  
  int i;
  struct dispatch_ws * dws;
  struct circ_bl_elem *tmp_cbl=NULL;
  struct circ_bl_elem *first_cbl;
  struct circ_bl_elem *prev_cbl;
  struct cmd_result *pres;
  int socket;
  int finished;
  int nbres;
  char **result;
  int nb_attempts;
  int varmod_port;

  if (ws->dws!=NULL)
    return 0;

  //allocate dispatcher structure
  dws=malloc(sizeof(struct dispatch_ws));
  memset(dws,0,sizeof(struct dispatch_ws));

  //allocate circular block list
  first_cbl=malloc(sizeof(struct circ_bl_elem));
  memset(first_cbl,0,sizeof(struct circ_bl_elem));
  prev_cbl=first_cbl;
  for(i=0;i<nb_blocks-1;i++){
    tmp_cbl=malloc(sizeof(struct circ_bl_elem));
    memset(tmp_cbl,0,sizeof(struct circ_bl_elem));
    tmp_cbl->block=NULL;
    prev_cbl->next=tmp_cbl;
    prev_cbl=tmp_cbl;
  }
  tmp_cbl->next=first_cbl;

  dws->evt_mutex=malloc(sizeof(pthread_mutex_t));
  dws->cbl=first_cbl;
  dws->cur_block=-1;
  pthread_mutex_init(dws->evt_mutex,NULL);
  
  dws->run_nb=random();

  finished=0;
  nb_attempts=0;
  varmod_port=submod_getport("varmod");
  //wait for varmod
  while(finished==0) {
    nb_attempts++;
    socket=open_socket("localhost",varmod_port);
    if (socket==-1)
      usleep(100000);
    else
      finished=1;
    if (nb_attempts>15) {
      return -ENOVARMOD;
    }
  }


  //TODO : add a real address
  pres=submod_execcmd("register_ds@varmod","id","default",name,"127.0.0.1",port,"undef",schema_str,"end");
  if (pres->retcode==0) {
    return -ECANTREGDS;
  }
  result=split(pres->str,',',&nbres);
  dws->dataport=atoi(result[0]);
  dws->datareadyport=atoi(result[1]);
  free_cmd_result(pres);
  dws->name=strdup(name);
  

  //add a data-ready socket to the netserv
  dws->netserv_ws=start_empty_netserv(10);
  dws->data_ready_sockets=netserv_add_new_port(dws->netserv_ws,dws->datareadyport);
  printf("listening on dataready socket port %d\n",dws->datareadyport);

  dws->schema_str=strdup(schema_str);
  dws->schema=saf_parse_chans(schema_str);
  if (dws->schema==NULL) {
    return -EBADSCHEMA;
  }

  //start serializer
  /*sem_init(&(dws->sem_serializer),0,0);
  if (pthread_create(&(dws->t_serializer),NULL,serializer,dws)<0) {
    perror("pthread_create");
    return raw_build_result(0,"error creating serializer thread");
  }
  pthread_detach(dws->t_serializer);*/

  ws->dws=dws;
  return dws->dataport;
}

int flush_dispatcher(struct dispatch_ws *ws) {
  struct circ_bl_elem*cbl;
  //aquire the mutex
  pthread_mutex_lock(ws->evt_mutex);

  ws->run_nb=random();

  //de-finalize all buffer
  if (ws->cbl->block) {
    free_block(ws->cbl->block);
    ws->cbl->block=NULL;
  }
  cbl=ws->cbl->next;
  while(cbl!=ws->cbl) {
    if (cbl->block) {
      free_block(cbl->block);
      cbl->block=NULL;
    }
    cbl=cbl->next;
  }

  ws->cur_block=-1;

  //release the mutex
  pthread_mutex_unlock(ws->evt_mutex);
  return 0;
}

int insert_bl(struct dispatch_ws *ws,struct block *bl) {

  if (bl->id<=ws->cur_block) {
    return -EMONOTONY;
  }

  if (ws->cbl->block!=NULL) {
    if (ws->mode==1) {
      //printf("offline mode\n");
      while(ws->cbl->nb_cs>0) {
        //printf("waiting for %p consumer count to be zero (now %d)\n",ws->cbl,ws->cbl->nb_cs);
        usleep(1000000);
      }
    }
  } else {
    // printf("filling a new block\n");
  }
  
  //aquire the mutex
  pthread_mutex_lock(ws->evt_mutex);

  if (ws->cbl->block!=NULL) {
    //printf("destroying old block\n");
    free_block(ws->cbl->block); 
  }
  ws->cbl->block=bl;
  //printf("the new block has %d as nb_cs\n",ws->init_nb_cs);
  //printf("workspace is %p %p\n",ws,&(ws->init_nb_cs));
  ws->cbl->nb_cs=ws->init_nb_cs;
  ws->cbl=ws->cbl->next;
  ws->cur_block=bl->id;

  //release the mutex
  pthread_mutex_unlock(ws->evt_mutex);
  return 0;
}


int finalize_bl(struct dispatch_ws *ws,struct block *bl) {
  //aquire the mutex
  pthread_mutex_lock(ws->evt_mutex);

  if (check_block(bl)==0) {
    //printf("finalizing block %d\n",bl->id);
    bl->bstr=saf_str_block(bl);
    bl->finalized=1;
    pthread_mutex_unlock(ws->evt_mutex);
    //increment number of unserialized blocks
    //sem_post(&(ws->sem_serializer));
    //warn user that a new block is available
    send2client(ws->data_ready_sockets,(unsigned char *)"r",1);
  } else {
    printf("error in block %d...destroying",bl->id);
    pthread_mutex_unlock(ws->evt_mutex);
    return -EBADBLOCK;
  }
  return 0;
}

int set_chans(struct dispatch_ws *ws,char *schema_str) {
  
  if (ws==NULL) {
    return -ENODISPATCH;
  }

  if (ws->schema_str!=NULL)
    free(ws->schema_str);
  ws->schema_str=strdup(schema_str);
  if (ws->schema!=NULL)
    free_evtstruct(ws->schema);
  ws->schema=saf_parse_chans(schema_str);
  if (ws->schema==NULL) {
    return -EBADSCHEMA;
  }
  return 0;
}

//*********************** EXTERNAL MODULE FUNCTIONS *************

struct cmd_result *treat_get_blocklist(struct cmdmod_ws *ws,struct cmd *command) {
  int blclim; //block limit arg 0
  struct circ_bl_elem *cbl; //iterator
  struct incr_string* blcres; //result
  int first=1;
  struct cmd_result *pres;

  if (ws->dws==NULL) {
    return raw_build_result(0,"dispatcher not initialized");
  }
  
  if (command->nb_params!=1) {
    return build_result(0,"get_blocklist_module, %d arguments given, 1 needed",command->nb_params);
  }

  blclim=atoi(command->params[0]);
  
  //aquire the mutex
  pthread_mutex_lock(ws->dws->evt_mutex);
  
  //contruct the block list
  cbl=ws->dws->cbl;
  blcres=new_is();
  do {
    if (cbl->block && cbl->block->finalized && cbl->block->id>blclim) {
      //printf("new block : %d \n",eptr->block);
      if (first==1) {
        fsuffix_is(blcres,"%d",cbl->block->id);
        first=0;
      } else {
        fsuffix_is(blcres,",%d",cbl->block->id);
      }
    }/* else {
      if (cbl->block)
        printf("block %d rejected\n",cbl->block->id);
    }*/
    cbl=cbl->next;
  } while(cbl!=ws->dws->cbl);
  
  //release the mutex
  pthread_mutex_unlock(ws->dws->evt_mutex);
  
  //return the result
  pres=raw_build_result(1,blcres->str);
  free_is(blcres);
  return pres;
}

int find_next_block(struct dispatch_ws *ws,int last_key) {
  struct circ_bl_elem*cbl;
  unsigned long int res=(unsigned long int)-1;
  
  //aquire the mutex
  pthread_mutex_lock(ws->evt_mutex);
  
  //find the lesser block nb up to blclim
  cbl=ws->cbl;
  do {
    if (cbl->block && cbl->block->finalized && cbl->block->id>last_key && cbl->block->id<res) {
      res=cbl->block->id;
    }
    cbl=cbl->next;
  } while(cbl!=ws->cbl);
  //release the mutex
  pthread_mutex_unlock(ws->evt_mutex);
  if (res==(unsigned long int)-1) {
    return -1;
  } else {
    return (int)res;
  }
}

int find_last_block(struct dispatch_ws *ws,int last_key) {
  struct circ_bl_elem*cbl;
  int res=-1;
  
  //aquire the mutex
  pthread_mutex_lock(ws->evt_mutex);
  
  //find the lesser block nb up to blclim
  cbl=ws->cbl;
  do {
    if (cbl->block && cbl->block->finalized && cbl->block->id>res && cbl->block->id>last_key) {
      res=cbl->block->id;
    }
    cbl=cbl->next;
  } while(cbl!=ws->cbl);
  //release the mutex
  pthread_mutex_unlock(ws->evt_mutex);
  return res;
}

struct cmd_result *treat_get_nextblock(struct cmdmod_ws *ws,struct cmd *command) {
  int blclim;
  int res;

  if (ws->dws==NULL) {
    return raw_build_result(0,"dispatcher not initialized");
  }
  
  if (command->nb_params!=1) {
    return build_result(0,"get_next_block_module, %d arguments given, 1 needed",command->nb_params);
  }

  blclim=atoi(command->params[0]);
  //printf("get_next_block blclim %d\n",blclim);
  res=find_next_block(ws->dws,blclim);

  
  if (res==-1) {
    return raw_build_result(0,"no such block found");
  } else {
    return build_result(1,"%d",res);
  }
}


struct cmd_result *treat_get_block(struct cmdmod_ws *ws,struct cmd *command) {
  int rblock; //number of requested block arg 0
  struct circ_bl_elem*cbl; //iterator
  struct cmd_result *pres;

  if (ws->dws==NULL) {
    return raw_build_result(0,"dispatcher not initialized");
  }
  
  if (command->nb_params!=1) {
    return build_result(0,"get_block_module, %d arguments given, 1 needed",command->nb_params);
  }

  //The argument is the requested block
  rblock=atoi(command->params[0]);
  //printf("get_block id %d\n",rblock);

  //aquire the mutex
  pthread_mutex_lock(ws->dws->evt_mutex);
  
  //fill the result
  cbl=ws->dws->cbl;
  do {
    if (cbl->block!=NULL) {
      if (cbl->block->id==rblock && cbl->block->finalized) {
        if (cbl->block->bstr==NULL) {
          cbl->block->bstr=saf_str_block(cbl->block);
        }
        pres=raw_build_result(1,cbl->block->bstr);
        cbl->nb_cs--;
        //printf("reducing consumer count on %p to %d\n",cbl,cbl->nb_cs);
        pthread_mutex_unlock(ws->dws->evt_mutex);
        return pres;
      }
    }
    cbl=cbl->next;
  } while(cbl!=ws->dws->cbl);
  
  //release the mutex
  pthread_mutex_unlock(ws->dws->evt_mutex);
  
  //return the result
  return raw_build_result(0,"converter : block not found");
}

struct cmd_result *find_block(struct dispatch_ws *ws,int block_id) {
  struct circ_bl_elem*cbl;
  struct cmd_result *pres;
  //aquire the mutex
  pthread_mutex_lock(ws->evt_mutex);
  
  //fill the result
  cbl=ws->cbl;
  do {
    if (cbl->block!=NULL) {
      if (cbl->block->id==block_id && cbl->block->finalized) {
        if (cbl->block->bstr==NULL) {
          cbl->block->bstr=saf_str_block(cbl->block);
        }
        pres=build_result(1,"%d,%s",block_id,cbl->block->bstr);
        cbl->nb_cs--;
        //printf("reducing consumer count on %p to %d\n",cbl,cbl->nb_cs);
        pthread_mutex_unlock(ws->evt_mutex);
        return pres;
      }
    }
    cbl=cbl->next;
  } while(cbl!=ws->cbl);
  
  //release the mutex
  pthread_mutex_unlock(ws->evt_mutex);
  return raw_build_result(-1,"no corresponding block");
}

struct cmd_result *treat_smart_get_block(struct cmdmod_ws *cws,struct cmd *command) {
  int run_nb;
  int res;
  int block_id;
  char policy;
  struct dispatch_ws *ws=cws->dws;
  struct cmd_result *pres=NULL;

  if (ws==NULL) {
    return raw_build_result(0,"dispatcher not initialized");
  }
  
  if (command->nb_params!=3) {
    return build_result(0,"smart_get_block_module, %d arguments given, 1 needed",command->nb_params);
  }

  //run number
  run_nb=atoi(command->params[0]);

  //policy
  policy=command->params[1][0];

  //requested block
  block_id=atoi(command->params[2]);

  //printf("smart_get_block run_nb=%d policy=%c block_id=%d\n",run_nb,policy,block_id);

  if (run_nb!=ws->run_nb) {
    return build_result(-2,"%d",ws->run_nb);
  }

  switch (policy) {
    case 'e': //exact
      pres=find_block(ws,block_id);
      break;
    case 'f': //first
      res=find_next_block(ws,block_id);
      pres=find_block(ws,res);
      break;
    case 'l': //last
      res=find_last_block(ws,block_id);
      pres=find_block(ws,res);
      break;
    default:
      pres=raw_build_result(0,"unknown policy");
      break;
  }
  //printf("smart_get_block retcode=%d res=%s\n",pres->retcode,pres->str);
  return pres;
}

struct cmd_result *treat_get_chans(struct cmdmod_ws *ws,struct cmd *command) {
  
  if (ws->dws==NULL) {
    return raw_build_result(0,"dispatcher not initialized");
  }
  
  return raw_build_result(1,ws->dws->schema_str);
}


struct cmd_result *treat_register_consumer(struct cmdmod_ws *ws,struct cmd *command) {
  if (ws->dws==NULL) {
    return raw_build_result(0,"dispatcher not initialized");
  }
  printf("new consumer registered\n");
  if (ws->dws->mode==0) {
    ws->dws->mode=1;
    printf("going to offline mode\n");
  }
  //printf("workspace in treat is %p %p\n",ws->dws,&(ws->dws->init_nb_cs));
  ws->dws->init_nb_cs++;
  //printf("init_nb_cs=%d\n",ws->dws->init_nb_cs);
  return raw_build_result(1,"ok");
}
