#include "cmdmod.h"

struct cenv *new_cenv() {
  struct cenv *res=malloc(sizeof(struct cenv));
  memset(res,0,sizeof(struct cenv));
  res->nbrecs=0;
  res->recs=NULL;
  res->registered=0;
  res->env_mutex=malloc(sizeof(pthread_mutex_t));
  pthread_mutex_init(res->env_mutex,NULL);
  res->t_synchro=(pthread_t*)malloc(sizeof(pthread_t));
  return res;
}

struct envrec *search_rec(struct cenv *env,const char *ns,const char *name) {
  struct envrec *search=env->recs;
  while(search!=NULL) {
    if ((!strcmp(search->param_name,name)) && (!strcmp(search->ns,ns))) {
      return search;
    }
    search=search->next;
  }
  return NULL;
}

void rec_delete(struct cenv *env,struct envrec *todel) {
  //unchain
  //printf("rec_delete\n");
  if (todel->prev!=NULL)
    todel->prev->next=todel->next;
  if (todel->next!=NULL)
    todel->next->prev=todel->prev;
  if (env->recs==todel)
    env->recs=todel->next;
  todel->next=NULL;
  todel->prev=NULL;
  env_dump(env);
  //free
  free(todel->ns);
  free(todel->param_name);
  free(todel->param_value);
  free(todel);
  env->nbrecs--;
}

char *env_getvar(struct cenv *env,const char *ns,const char *name) {
  struct envrec *search=search_rec(env,ns,name);
  if (search==NULL) {
    return NULL;
  } else {
    return search->param_value;
  }
}

char env_gettype(struct cenv *env,const char *ns,const char *name) {
  struct envrec *search=search_rec(env,ns,name);
  if (search==NULL) {
    //TODO: add error in libpyrame
    return -1;
  } else {
    return search->param_type;
  }
}

//TODO add a mutex on env
void *thread_synchroenv(void *param) {
  struct cenv *env=(struct cenv *)param;
  struct envrec *search;
  struct cmd_result *pres;
  char all_clean;

  while(1) {
    all_clean=1;
    search=env->recs;
    while(search!=NULL) {
      if (search->dirty) {
	if (search->synchro==1) {
	  all_clean=0;
	  pres=submod_execcmd("set_param@cmod",search->ns,search->param_name,search->param_value,"end");
	  if (pres->retcode==0) {
	    printf("error : try to synchronize unknown variable %s,%s\n",search->ns,search->param_name);
	    //search->synchro=0;
	  }
	  free_cmd_result(pres); 
	} //synchro=1
	if (search->synchro==-1) {
	  pres=submod_execcmd("set_param@cmod",search->ns,search->param_name,search->param_value,"end");
	  if (pres->retcode==0) {
	    debug("local variable %s\n",search->param_name);
	    search->synchro=0;
	  } else {
	    debug("synchronized variable %s\n",search->param_name);
	    search->synchro=1;
	  }
	}
      }
      search->dirty=0;
      search=search->next;
    }
    if (all_clean)
      usleep(100000);
  }
}

//set a variable in environment
//synchro should be 0 or 1 for fixed value or -1 for probing
int env_setvar(struct cenv *env,const char *ns,const char *name,char *value,char type,char synchro) {
  struct envrec *search=NULL;
  struct envrec *newrec;
  if (strcmp(ns,"system")) {
    if (!env_checkns(env,ns)) {
      return 0;
    }
  }
  search=search_rec(env,ns,name);
  //printf("set ns:%s name:%s to %s\n",ns,name,value);
  if (search==NULL) { //if not found
    //create a new entry
    newrec=(struct envrec *)malloc(sizeof(struct envrec));
    newrec->ns=strdup(ns);
    newrec->param_name=strdup(name);
    newrec->param_value=strdup(value);
    newrec->param_type=type;
    newrec->synchro=synchro;
    newrec->dirty=1;
      
    //chain in environment
    if (env->recs) {
      env->recs->prev=newrec;
    }
    newrec->next=env->recs;
    newrec->prev=NULL;
    env->recs=newrec;
    env->nbrecs++;
  } else { //the variable already exists
    if (search->param_value!=NULL) {
      free(search->param_value);
    }
    search->param_value=strdup(value);
    search->param_type=type;
    
    //synchronize the variable with cmod if necessary
    if (search->synchro!=0) {
      search->dirty=1;
    }
  }
  env_dump(env);
  return 1;
}

int env_delvar(struct cenv *env,const char *ns,const char *name) {
  struct envrec *search=search_rec(env,ns,name);
  if (search==NULL) {
    return 0;
  }
  rec_delete(env,search);
  return 1;
}


int ns_rm(struct cenv *env,const char *ns) {
  struct envrec *todelete;
  struct envrec *search=env->recs;
  while(search!=NULL) {
    if (!strcmp(search->ns,ns)) {
      todelete=search;
    } else {
      todelete=NULL;
    }
    search=search->next;
    if (todelete) {
      rec_delete(env,todelete);
    }
  }
  return 1;
}

int env_delns(struct cenv *env,const char *ns_name) {
  if (env_checkns(env,ns_name)==0) {
    return 0;
  } else {
    ns_rm(env,ns_name);
    env_delvar(env,"system",ns_name);
    return 1;
  }
}

char *env_dumpns(struct cenv *env,const char *ns_name) {
  struct incr_string *dump=new_is();
  struct envrec *search=env->recs;
  int first=1;
  fsuffix_is(dump,"{");
  while(search!=NULL) {
    if (!strcmp(search->ns,ns_name)) {
      if (!first) {
	fsuffix_is(dump,",");
      } else {
	first=0;
      }
      fsuffix_is(dump,"\"%s\":\"%s\"",search->param_name,search->param_value);
    }
    search=search->next;
  }
  fsuffix_is(dump,"}");
  return reduce_is(dump);
}

int env_checkns(struct cenv *env,const char *ns_name) {
  char *res=env_getvar(env,"system",ns_name);
  if (res==NULL)
    return 0;
  else
    return 1;
}

int env_newns(struct cenv *env,const char *ns_name) {
  if (env_checkns(env,ns_name)) {
    //printf("ns %s already exists\n",ns_name);
    return 0;
  } else {
    //printf("creating ns %s\n",ns_name);
    return env_setvar(env,"system",ns_name,"ns",ENV_NS,0);
  }
}

void rec_print(struct envrec *rec) {
  printf("ns:%s name:%s value:%s type:%d synchro:%d\n",rec->ns,rec->param_name,rec->param_value,rec->param_type,rec->synchro);
}


int env_dump(struct cenv *env) {
  FILE *f;
  struct envrec *search=NULL;
  struct envrec *name;
  char *sname;
  name=search_rec(env,"system","name");
  if (name==NULL) {
    //printf("Error : cant find system name...cant dump environment\n");
    return 0;
  }
  sname=malloc(strlen(name->param_value)+1024);
  sprintf(sname,"/tmp/%s.env",name->param_value);
  //printf("trying to save env in %s\n",sname);
  f=fopen(sname,"w");
  free(sname);
  if (f==NULL) {
    printf("error : cant save env\n");
    return 0;
  }

  //dump system
  search=env->recs;
  while(search!=NULL) {
    if (!strcmp(search->ns,"system")) {
      fprintf(f,"\"%s\" \"%s\" \"%s\" \"%d\" \"%d\"\n",search->ns,search->param_name,search->param_value,search->param_type,search->synchro);
    }
    search=search->next;
  }


  //dump the rest
  search=env->recs;
  while(search!=NULL) {
    if (strcmp(search->ns,"system")!=0) {
      fprintf(f,"\"%s\" \"%s\" \"%s\" \"%d\" \"%d\"\n",search->ns,search->param_name,search->param_value,search->param_type,search->synchro);
    }
    search=search->next;
  }

  
  fclose(f);
  return 1;
}

char * env_listns(struct cenv *env) {
  struct envrec *search=NULL;
  struct incr_string *result=new_is();
  int first=1;

  search=env->recs;
  while(search!=NULL) {
    if (!strcmp(search->ns,"system") && (search->param_type==ENV_NS) && strcmp(search->param_name,"system")) {
      if (first) {
        suffix_is(result,search->param_name);
        first=0;
      } else {
        fsuffix_is(result,",%s",search->param_name);
      }
    }
    search=search->next;
  }
  return reduce_is(result);
}


int env_load(struct cenv *env,char *filename) {
  FILE *f;
  char *ns;
  char *param_name;
  char *param_value;
  char *param_type;
  char *param_synchro;
  int res;
  f=fopen(filename,"r");
  if (f==NULL) {
    return 0;
  }
  ns=malloc(1024);
  param_name=malloc(1024);
  param_value=malloc(1024);
  param_type=malloc(1024);
  param_synchro=malloc(1024);
  while(!feof(f)) {
    res=fscanf(f,"\"%[^\"]\"",ns);
    if (res!=1) {
      printf("syntax error at 1st field : %s\n",ns);
    }
    res=fscanf(f," \"%[^\"]\"",param_name);
    if (res!=1) {
      printf("syntax error at 2rd field : %s\n",param_name);
    }
    res=fscanf(f," \"%[^\"]\"",param_value);
    if (res!=1) {
      printf("syntax error at 3rd field : %s\n",param_value);
    }
    res=fscanf(f," \"%[^\"]\"\n",param_type);
    if (res!=1) {
      printf("syntax error at 4th field : %s\n",param_type);
    }
    res=fscanf(f," \"%[^\"]\"\n",param_synchro);
    if (res!=1) {
      printf("syntax error at 5th field : %s\n",param_synchro);
    }
    printf("adding to env : %s %s %s %d %d\n",ns,param_name,param_value,atoi(param_type),atoi(param_synchro));
    res=env_setvar(env,ns,param_name,param_value,atoi(param_type),atoi(param_synchro));
  }
  fclose(f);
  free(ns);
  free(param_name);
  free(param_value);
  return 1;
}

void env_print(struct cenv *env) {
  struct envrec *search=env->recs;
  printf("%d records\n",env->nbrecs);
  while(search!=NULL) {
    rec_print(search);
    search=search->next;
  }
}

