#include "cmdmod.h"

//*************************** CONN_DB FUNCTIONS ******************************

struct conn *get_conn(struct cmdmod_ws *ws,char *host,int port) {
  struct conn_db *it=ws->cdb;
  struct conn *res;
  while(it!=NULL) {
    if ((!strcmp(it->conn->host,host)) && (it->conn->port==port))
      return it->conn;
    it=it->next;
  }
  //debug("creating new connection for %s:%d\n",host,port);
  res=(struct conn *)malloc(sizeof(struct conn));
  res->host=strdup(host);
  res->port=port;
  res->socket=-1;
  res->use_mutex=malloc(sizeof(pthread_mutex_t));
  pthread_mutex_init(res->use_mutex,NULL);
  it=(struct conn_db *)malloc(sizeof(struct conn_db));
  it->conn=res;
  it->next=ws->cdb;
  ws->cdb=it;
  return res;
}

void close_all_conns(struct cmdmod_ws *ws) {
  struct conn_db *it=ws->cdb;
  while(it!=NULL) {
    if (it->conn->socket!=-1) {
      close(it->conn->socket);
      it->conn->socket=-1;
    }
    it=it->next;
  }
}

//************************** FUNC_DB FUNCTIONS *******************

struct func_db *init_func_db() {
  struct func_db *res=malloc(sizeof(struct func_db));
  res->first=NULL;
  return res;
}

void debug_func(struct function *func) {
  if (func->name==NULL) 
    debug("function has no name\n");
  else
    debug("function name=%s\n",func->name);
  
  if (func->type==SUBMOD_TYPE_NONE) {
    debug("type=none\n");
  }
  if (func->type==SUBMOD_TYPE_HOST) {
    debug("type=host\n");
    debug("host=%s port=%d\n",func->host,func->port);
  }
  if (func->type==SUBMOD_TYPE_SCRIPT) {
    debug("type=script\n");
  }
  if (func->type==SUBMOD_TYPE_INT) {
    debug("type=internal\n");
  }
  debug("\n");
}

void debug_db(struct func_db *db) {
  struct function *it=db->first;
  while(it!=NULL) {
    debug_func(it);
    it=it->next;
  }
}

struct function *search_func_byname(struct func_db *db,char *name) {
  struct function *it=db->first;
  //printf("searching with chaining\n");
  while(it!=NULL) {
    if (!strcmp(it->name,name)) {
      return it;
    } 
    it=it->next;
  }
  //debug("function %s not found in db\n",name);
  return NULL;
}

void add_func(struct func_db *db,struct function *new_func) {
  new_func->next=db->first;
  db->first=new_func;
}


//************************** INTERNAL FUNC_DB FUNCTIONS *******************

void add_int_func(struct func_db *db,char *name,struct cmd_result *(*fp)(struct cmdmod_ws *ws,struct cmd *command)) {
  struct function *newfunc=malloc(sizeof(struct function));
  newfunc->type=SUBMOD_TYPE_INT;
  newfunc->name=name;
  newfunc->fp=fp;
  newfunc->api=NULL;
  add_func(db,newfunc);
}

void init_ifdb(struct func_db *db,char *name) {
  char tmp[1024];
  add_int_func(db,strdup("printenv_module"),treat_printenv);
  add_int_func(db,strdup("exit_module"),treat_exit);
  add_int_func(db,strdup("register_module"),treat_register);

  add_int_func(db,strdup("getfuncapi_module"),treat_getfuncapi);
  sprintf(tmp,"getfuncapi_%s",name);
  add_int_func(db,strdup(tmp),treat_getfuncapi);

  add_int_func(db,strdup("getapi_module"),treat_getapi);
  sprintf(tmp,"getapi_%s",name);
  add_int_func(db,strdup(tmp),treat_getapi);

  add_int_func(db,strdup("getblocklist_module"),treat_get_blocklist);
  add_int_func(db,strdup("getnextblock_module"),treat_get_nextblock);
  add_int_func(db,strdup("getblock_module"),treat_get_block);
  add_int_func(db,strdup("smartgetblock_module"),treat_smart_get_block);
  add_int_func(db,strdup("getchans_module"),treat_get_chans);
  add_int_func(db,strdup("registerconsumer_module"),treat_register_consumer);
}

//*************************** CMOD FUNC_DB FUNCTIONS **********************

struct function *search_cmod_function_in_db(struct cmdmod_ws *ws,char *name,char *id) {
  struct function *it=ws->cfdb->first;
  while(it!=NULL) {
    if ((!strcmp(it->name,name)) && (!strcmp(it->id,id))) {
      return it;
    } 
    it=it->next;
  }
  return NULL;
}

struct function *ask_cmod(struct cmdmod_ws *ws,char *funcname,char *module,char *id) {
  struct function *res=(struct function*)malloc(sizeof(struct function));
  struct cmd_result *dres;
  char **location;
  int nbloc;
  
  //if the port is cmod we return the cmod ip as cmod is unique in the system
  if (!strcmp(module,"cmod")) {
    res->name=strdup(funcname);
    res->type=SUBMOD_TYPE_HOST;
    res->host=ws->cmod_ip;
    res->port=ws->cmod_port;
    res->portstring=NULL;
    //res->funcname=NULL;
    res->conn=get_conn(ws,res->host,res->port);
    add_func(ws->cfdb,res);
    return res;
  }

  //debug("asking cmod\n");
  
  //really ask the cmod for type (module), ip
  dres=sendcmd(ws->cmod_ip,ws->cmod_port,"get_location_cmod",id,"end");

  //not found
  if (dres->retcode==0) {
    //debug("Error cant find in cmod <- %s\n",dres->str);
    free_cmd_result(dres);
    free(res);
    return NULL;
  }

  location=split(dres->str,',',&nbloc);
  if (nbloc!=2) {
    debug("Error: cmod returned a string with %d items (2 expected)\n",nbloc);
    free_cmd_result(dres);
    free(res);
    return NULL;
  }
  //fill the func function
  res->name=strdup(funcname);
  res->type=SUBMOD_TYPE_CMOD;
  res->host=location[0];
  res->port=get_port(location[1]);
  //res->funcname=NULL;
  res->id=strdup(id);
  res->conn=get_conn(ws,res->host,res->port);
  add_func(ws->cfdb,res);

  free(location[1]);
  free(location);
  free_cmd_result(dres);
  return res;
}



//*************************** SCRIPT FUNC_DB FUNCTIONS **********************


char *dump_script_func_db(struct cmdmod_ws *ws) {
  struct incr_string *is=new_is();
  struct function *it=ws->sfdb->first;
  while(it!=NULL) {
    if (it->api) {
      fsuffix_is(is,"%s:%s;",it->name,it->api); 
    }
    it=it->next;
  }
  return reduce_is(is);
  }


