#include "cmdmod_inter.h"

void export_res(struct inter_ws *iws,int retcode,char *retstr) {
  char *exp=malloc(strlen(retstr)+100);

  //send retcode
  sprintf(exp,"%d\n",retcode);
  write(iws->fds_t[1],exp,strlen(exp));
  //send retstr
  sprintf(exp,"%s\n",retstr);
  write(iws->fds_t[1],exp,strlen(exp));
  
  free(exp);
}

void export_cres(struct inter_ws *iws,struct cmd_result *cres) {
  export_res(iws,cres->retcode,cres->str); 
}

int get_int(struct inter_ws *iws) {
  char *buf=NULL;
  size_t size=0;
  char *endptr=NULL;
  int res;
  getline(&buf,&size,iws->rfb);
  res=strtol(buf,&endptr,10);
  free(buf);
  return res;
}

char *get_string(struct inter_ws *iws) {
  char *buf=NULL;
  char *res=NULL;
  size_t size=0;
  getline(&buf,&size,iws->rfb);
  buf[strlen(buf)-1]=0;
  res=strdup(buf);
  return res;
}

void submod_setres_inter(struct inter_ws *iws) {
  free(iws->retstr);
  iws->retcode=get_int(iws);
  iws->retstr=get_string(iws);
  //mark the action as treated
  iws->treated=1;
}

void submod_callwrap_inter(struct inter_ws *iws) {
  char *func_name;
  char **wargs;
  int i;
  struct submod_wrapper *wrapper;
  struct cmd_result *pres;
  struct cmdmod_ws *ws=cmdmod_getenv();
  
  func_name=get_string(iws);
 
  wrapper=lookup_wrapper(ws->we,func_name);
  if (wrapper==NULL) {
    export_res(iws,0,"unknown function");
    iws->treated=1;
    return;
  }

  wargs=malloc(wrapper->nb_args*sizeof(char *));
  for(i=0;i<wrapper->nb_args;i++) {
    wargs[i]=get_string(iws);
  }
  
  //call the wrapper
  pres=wrapper->wrapper(wargs);
  free(wargs);


  export_res(iws,pres->retcode,pres->str);
  free_cmd_result(pres);

  //mark the action as treated
  iws->treated=1;
}


void submod_execcmd_inter(struct inter_ws *iws) {
  struct cmd_result *cres;
  struct cmd *subcommand;
  int i;
  int nbargs;

  //parse the number of args
  nbargs=get_int(iws);

  //allocate and fill the subcommand
  subcommand=malloc(sizeof(struct cmd));
  subcommand->nb_params=nbargs-1;
  subcommand->params=malloc((nbargs-1)*sizeof(char *));
  subcommand->name=strdup(get_string(iws));
  for(i=0;i<nbargs-1;i++) {
    subcommand->params[i]=strdup(get_string(iws));
  }
  
  //execute the function
  cres=treat_cmd(subcommand);
  
  //free the command
  free_cmd(subcommand);
  
  //write the result to interpreter
  export_cres(iws,cres);
  free_cmd_result(cres);

  //mark the action as treated
  iws->treated=1;
}

void treat_output(struct inter_ws *iws) {
  size_t buf_s=1024;
  char *buf=malloc(sizeof(char)*buf_s);
  ssize_t glres;

  debug("treat_output\n");
  
  buf[0]=0;
  while (strcmp(buf,"+++eot\n")) {
    glres=getline(&buf,&buf_s,iws->rfb);
    if (glres==-1) {
      printf("Interpreter link lost (dead) ...exiting\n");
      cexit(1);
    }
    iws->treated=0;

    //treating end of transaction
    if (!strcmp(buf,"+++eot\n")) {
      //printf("end of transaction\n");
      iws->treated=1;
    }
    
    //treating commands
    if (!strcmp(buf,"+++setres\n")) {
      //printf("setres\n");
      submod_setres_inter(iws);
    }
    if (!strcmp(buf,"+++callwrap\n")) {
      printf("callwrap\n");
      submod_callwrap_inter(iws);
    }
    if (!strcmp(buf,"+++execcmd\n")) {
      //printf("execcmd\n");
      submod_execcmd_inter(iws);
    }

    //treat other messages
    if (!iws->treated) {
      //printf("comment from script : %s",buf);
    }
    
  }
  if (buf!=NULL) {
    free(buf);
  }
  //debug("end of treat_output\n");
}


void *init_inter(struct cmdmod_ws *cws,char *execfile,char *vp[]) {
  /* this function is called by interpreter modules like sh or R
     cws is the cmdmod workspace
  */

  //create the language specific workspace
  struct inter_ws *iws=malloc(sizeof(struct inter_ws));
  memset(iws,0,sizeof(struct inter_ws));

  //creating pipes to communicate with interpreter
  if (pipe(iws->fds_t)<0) {
    printf("cant open t_pipe...exiting\n");
    cexit(1);
  }
  if (pipe(iws->fds_f)<0) {
    printf("cant open f_pipe...exiting\n");
    cexit(1);
  }

  //fork
  iws->pid=fork();
  if (iws->pid==(pid_t)0) {
    //interpreter process
    //connect standard input/output to pipes
    dup2(iws->fds_t[0],STDIN_FILENO);
    close(iws->fds_t[1]);
    dup2(iws->fds_f[1],STDOUT_FILENO);
    close(iws->fds_f[0]);
    //exec interpreter
    if (execvp(execfile,vp)==-1) {
      perror("execvp");
      printf("cant launch %s...exiting\n",execfile);
      cexit(1);
    }
  } else {
    //cmdmod process
    //create the pseudo FILE interface for getline
    iws->rfb=fdopen(iws->fds_f[0],"r");
    if (iws->rfb==NULL) {
      perror("fdopen");
      printf("cant open pseudo-file interface...exiting\n");
      cexit(1);
    } 
    //close the unnecessary pipes
    close(iws->fds_t[0]);
    close(iws->fds_f[1]);
  }
  
  //init return fields
  iws->retcode=0;
  iws->retstr=strdup("init");

  return (void *)iws;
}

struct cmd_result *exec_command_inter(char *command,struct inter_ws *iws) {
  /* This function is called any time a pyrame command is received by cmdmod.
     command contains the name of the pyrame command and the parameters
     func contains the description of the function including its func name.
  */
  int res;
  
  //write command to interpreter
  //printf("sending %s to interpreter\n",command);
  res=write(iws->fds_t[1],command,strlen(command));
  if (res<=0) {
    perror("write");
    printf("cant write to interpreter...exiting\n");
    cexit(1);
  }
  
  free(iws->retstr);
  iws->retcode=0;
  iws->retstr=strdup("No answer from script");
  
  treat_output(iws);
  return raw_build_result(iws->retcode,iws->retstr);
}

void send2inter(struct inter_ws *iws,char *format,...) {
  char test[1];
  int req_size;
  va_list pa;
  char *cmd;
  //get the required size for the string
  va_start(pa,format);
  req_size=vsnprintf(test,1,format,pa)+10;
  va_end(pa);
  //allocate and fill the string
  cmd=malloc(req_size*sizeof(char));
  va_start(pa,format);
  vsprintf(cmd,format,pa);
  va_end(pa);
  //send the string
  write(iws->fds_t[1],cmd,strlen(cmd));
  free(cmd);
}

