#include "cmdmod_sh.h"

char *escape_quote_sh(char *src) {
  int i;
  int j;
  char *res;
  int size=strlen(src)+10;
  
  for (i=0;i<strlen(src);i++) {
    if (src[i]=='"')
      size+=4;
  }
  res=malloc(size);

  j=0;
  for (i=0;i<strlen(src);i++) {
    if (src[i]=='"') {
      res[j]='\\';
      res[j+1]='"';
      j+=2;
    } else {
      res[j]=src[i];
      j++;
    }
  }
  res[j]=0;
  return res;
}

char *build_args_sh(int argc,char **argv) {
  struct incr_string *ires=new_is();
  int i;
  char *esc_param;
  for(i=0;i<argc;i++) {
    esc_param=escape_quote_sh(argv[i]);
    if (i==0) {
      fsuffix_is(ires," \"%s\"",esc_param);
    } else {
      fsuffix_is(ires," \"%s\"",esc_param);
    }
    free(esc_param);
  }
  return reduce_is(ires);
}

void *init_sh(struct cmdmod_ws *cws) {
  /* this function is called by cmdmod if language is sh
     cws is the cmdmod workspace
  */
  char *args;
  
  //create the language specific workspace
  struct sh_ws *bws=malloc(sizeof(struct sh_ws));
  memset(bws,0,sizeof(struct sh_ws));

  //initialize the sh interpreter
  char *vp[]={"sh",NULL};
  bws->iws=init_inter(cws,"/bin/sh",vp);
  
  //import the helper
  debug("sending sh helper\n");
  send2inter(bws->iws,". /opt/pyrame/cmdmod_helper.sh\n");
    
  //load the code file
  debug("sending sh code\n");
  send2inter(bws->iws,". %s\n",submod_getvar("system","code_file_name"));
  
  //execute init function
  debug("calling init function\n");
  args=build_args_sh(cws->init_nb_args,cws->init_args);
  send2inter(bws->iws,"internal_execcmd init %s\n",args);
  free(args);
  
  //treat module output
  treat_output(bws->iws);

  //return workspace
  return (void *)bws;
}

void deinit_sh(void *ws) {
  struct sh_ws *bws=(struct sh_ws *)ws;
  //execute deinit function
  send2inter(bws->iws,"internal_execcmd deinit\n");
  //quit sh
  send2inter(bws->iws,"exit\n");
  treat_output(bws->iws);
}


struct cmd_result *exec_command_sh(struct cmd *command,struct function *func,void *ws) {
  /* This function is called any time a pyrame command is received by cmdmod.
     command contains the name of the pyrame command and the parameters
     func contains the description of the function including its func name.
  */
  struct sh_ws *bws=(struct sh_ws *)ws;
  struct cmd_result *cres;
  struct incr_string *ires=new_is();
  char *args=build_args_sh(command->nb_params,command->params);
  fsuffix_is(ires,"internal_execcmd %s %s\n",command->name,args);
  free(args);
  cres=exec_command_inter(ires->str,bws->iws);
  free_is(ires);
  return cres;
}


