#include "cmdmod.h"

//thread

struct bg_ws *submod_bgcontext(void) {
  struct cmdmod_ws *ws=cmdmod_getenv();
  struct bg_ws *result=malloc(sizeof(struct bg_ws));
  result->index=ws->current_index;
  return result;
}

void submod_sendres(struct bg_ws *context,struct cmd_result *cmdres) {
  struct cmdmod_ws *ws=cmdmod_getenv();
  int i=context->index;
  printf("retcode=%d   res=%s (keepopen)\n",cmdres->retcode,cmdres->str);
  if (send_result(ws->clients->sock_client[i],cmdres)<=0) {
    printf("cmd socket lost (send) %d\n",i);
    perror("write");
    close(ws->clients->sock_client[i]);
    ws->clients->sock_client[i]=-1;
  }
  free_cmd_result(cmdres);
  free(context);
}

//environment

int submod_setvar(const char *ns,const char *name,char *value,char type) {
  struct cmdmod_ws *ws=cmdmod_getenv();
  //printf("set %s %s %s %d\n",ns,name,value,type);
  return env_setvar(ws->env,ns,name,value,type,-1);
}

char *submod_getvar(const char *ns,const char *name) {
  struct cmdmod_ws *ws=cmdmod_getenv();
  return env_getvar(ws->env,ns,name);
}

int submod_delvar(const char *ns,const char *name) {
  struct cmdmod_ws *ws=cmdmod_getenv();
  return env_delvar(ws->env,ns,name);
}

char submod_gettype(const char *ns,const char *name) {
  struct cmdmod_ws *ws=cmdmod_getenv();
  return env_gettype(ws->env,ns,name);
}

int submod_newns(const char *ns) {
  struct cmdmod_ws *ws=cmdmod_getenv();
  return env_newns(ws->env,ns);
}

int submod_checkns(const char *ns) {
  struct cmdmod_ws *ws=cmdmod_getenv();
  return env_checkns(ws->env,ns);
}

int submod_delns(const char *ns) {
  struct cmdmod_ws *ws=cmdmod_getenv();
  return env_delns(ws->env,ns);
}

char *submod_dumpns(const char *ns) {
  struct cmdmod_ws *ws=cmdmod_getenv();
  return env_dumpns(ws->env,ns);
}

char *submod_listns() {
  struct cmdmod_ws *ws=cmdmod_getenv();
  return env_listns(ws->env);
}

//system

void submod_setmyname(char *newname) {
  struct cmdmod_ws *ws=cmdmod_getenv();
  env_setvar(ws->env,"system","name",newname,ENV_STR,0);
}

int submod_getport(char *port_name) {
  return get_port(port_name);
}

void submod_exit(int exit_code) {
  printf("one thread exits with code %d\n",exit_code);
  cexit(exit_code);
}

char *submod_getip(char *func_name,int id) {
  struct cmd *cmd=(struct cmd *)malloc(sizeof(struct cmd));
  struct function *func;
  struct cmdmod_ws *ws=cmdmod_getenv();

  cmd->name=strdup(func_name);
  cmd->params=(char **)malloc(sizeof(char *));
  if (id!=-1) {
    cmd->nb_params=1;
    cmd->params[0]=(char *)malloc(100);
    sprintf(cmd->params[0],"%d",id);
  } else {
    cmd->nb_params=0;
  }
  func=search_ext_function(ws,cmd);
  free_cmd(cmd);

  if (func==NULL) {
    return NULL;
  } 

  return strdup(func->host);
}

//dispatcher

int submod_init_dispatcher(int nb_blocks,char *name,char *schema_str,char *port) {
  struct cmdmod_ws *ws=cmdmod_getenv();
  return init_dispatcher(ws,nb_blocks,name,schema_str,port);
}

int submod_flush_dispatcher() {
  struct cmdmod_ws *ws=cmdmod_getenv();
  return flush_dispatcher(ws->dws);
}

int submod_new_block(int block_id,struct block **result) {
  struct dispatch_ws *ws=cmdmod_getenv()->dws;

  if (ws==NULL) {
    return -ENODISPATCH;
  }

  *result=new_block_from_es(ws->schema,block_id);
  return insert_bl(ws,*result);
}

int submod_new_event(struct block *bl,struct event **result) {
 
  if (bl==NULL) {
    return -EBLNULL;
  }
  if (bl->finalized==1) {
    return -EALFINAL;
  }
  *result=new_event_from_es(bl);
  return 0;
}


void submod_add_block(struct block *bl,int block_id) {
  struct cmdmod_ws *ws=cmdmod_getenv();
  bl->id=block_id;
  insert_bl(ws->dws,bl);
  finalize_bl(ws->dws,bl);
}

int submod_set_field_block(struct block *bl,struct evtstructelem *ese,char *format,...) {
  int req_size;
  va_list pa;
  char *value;
  int i=0;
  int noformat=1;
  //struct cmdmod_ws *ws=cmdmod_getenv();

  if (!bl)
    return -EBLNULL;

  if (bl->finalized==1)
    return -EALFINAL;

  while(format[i]!=0) {
    if (format[i]=='%') {
      noformat=0;
      break;
    }
    i++;
  }

  if (noformat) {
    value=strdup(format);
  } else {

    value=malloc(4096*sizeof(char));
  
    //get the required size for the string
    va_start(pa,format);
    req_size=vsnprintf(value,4096,format,pa);
    va_end(pa);
    
    if (req_size>4096) {
      free(value);
      value=malloc(req_size*sizeof(char));
      va_start(pa,format);
      vsprintf(value,format,pa);
      va_end(pa);
    }
  }

  return set_block_field(bl,ese,value);
}

int submod_set_field_event(struct event *ev,struct evtstructelem *ese,char *format,...) {
  int req_size;
  va_list pa;
  char *value;
  int i=0;
  int noformat=1;
  int opt_size;
  //struct cmdmod_ws *ws=cmdmod_getenv();

  if (!ev)
    return -EEVNULL;

  while(format[i]!=0) {
    if (format[i]=='%') {
      noformat=0;
      break;
    }
    i++;
  }

  if (noformat) {
    value=strdup(format);
  } else {
    opt_size=24;
    value=malloc(opt_size*sizeof(char));
  
    //get the required size for the string
    va_start(pa,format);
    req_size=vsnprintf(value,opt_size,format,pa);
    va_end(pa);
    
    if (req_size>opt_size) {
      free(value);
      value=malloc(req_size*sizeof(char));
      va_start(pa,format);
      vsprintf(value,format,pa);
      va_end(pa);
    }
  }

  return set_event_field(ev,ese,value);
}

int submod_finalize_block(struct block *bl) {
  struct cmdmod_ws *ws=cmdmod_getenv();
  return finalize_bl(ws->dws,bl);
}

int submod_set_chans(char *schema_str) {
  struct cmdmod_ws *ws=cmdmod_getenv();
  return set_chans(ws->dws,schema_str);
}

int submod_wakemeup(int time,char *function,char *param) {
  pthread_t wmu;
  struct ctask *cws=malloc(sizeof(struct ctask));
  cws->function=strdup(function);
  cws->param=strdup(param);
  cws->time=time;
  if (pthread_create(&wmu,NULL,wakemeup_thread,(void*)cws)<0) {
    perror("pthread_create");
    return -ENOTHREAD;
  }
  if (pthread_detach(wmu)!=0) {
    perror("pthread_detach");
    return -ENOTHREAD;
  }
  return 0;
}
