#include "cmdmod.h"

char msg_no_result[] = "no result available from script";
char msg_error_script[] = "error from script";
char msg_unknown_command[] = "unknown command";
char msg_unknown_in_cmod[] = "unknown command in db and cmod";


//************************ INTERNAL FUNCTIONS ******************************


struct cmd_result *treat_exit(struct cmdmod_ws *ws,struct cmd *command) {
  int wres;
  (*ws->deinit_func)(ws->lang_spec_ws);
  wres=write(ws->current_socket,"<res retcode=\"1\">ok</res>\n",26);
  close_all_conns(ws);
  if (wres==0)
    debug("nothing written\n");
  if (wres==-1)
    perror("write");
  cexit(1);
  return NULL;
}


struct cmd_result * treat_register(struct cmdmod_ws *ws,struct cmd *command) {
  if (ws->local) {
    return raw_build_result(0,"already in local mode");
  }
  if (ws->env->registered) {
    return raw_build_result(1,"ok");
  }
  ws->cmod_ip=get_caller_ip();
  if (!strcmp(ws->cmod_ip,"")) {
    return raw_build_result(0,"cant get cmod ip");
  } else {
    debug("registered at %s\n",ws->cmod_ip);
  }
  ws->cmod_port=get_port("cmod");

  if (pthread_create(ws->env->t_synchro,NULL,thread_synchroenv,(void *)ws->env)<0) {
    perror("pthread_create");
    exit(1);
  }
  pthread_detach(*ws->env->t_synchro);

  ws->env->registered=1;
  return raw_build_result(1,"ok");
}


struct cmd_result * treat_getfuncapi(struct cmdmod_ws *ws,struct cmd *command) {
  struct function *f;
  if (command->nb_params!=1) {
    return build_result(0,"1 param needed, %d provided",command->nb_params);
  }
  f=search_func_byname(ws->sfdb,command->params[0]);
  if (f==NULL) {
    return raw_build_result(0,"unknown command");
  }
  if (f->api==NULL) {
    return raw_build_result(0,"no api available for this command");
  }
  return raw_build_result(1,f->api);
}

struct cmd_result * treat_getapi(struct cmdmod_ws *ws,struct cmd *command) {  return raw_build_result(1,dump_script_func_db(ws));
}

struct cmd_result * treat_printenv(struct cmdmod_ws *ws,struct cmd *command) {
  env_print(ws->env);
  return raw_build_result(1,"ok");
}



//*************************** SCRIPT FUNCTIONS ******************************

struct cmd_result * treat_script(struct cmdmod_ws *ws,struct cmd *command,struct function* func) {
  struct cmd_result *cres=NULL;
  struct incr_string *new_cmd_str;
  int i;
  if (ws->debug) {
  new_cmd_str=new_is();
  fsuffix_is(new_cmd_str,"***** EXEC %s(",command->name);
  for (i=0;i<command->nb_params;i++) {
    if (i==0) {
      fsuffix_is(new_cmd_str,"\"%s\"",command->params[i]);
    } else {
      fsuffix_is(new_cmd_str,",\"%s\"",command->params[i]);
    }
  }
  fsuffix_is(new_cmd_str,")\n");
  debug("%s",new_cmd_str->str);
  free_is(new_cmd_str);
  }
  
  //execute the function in embedded language
  cres=(*ws->execcmd_func)(command,func,ws->lang_spec_ws);

  //return a result 
  debug("retcode=%d   res=%s\n",cres->retcode,cres->str);
  return cres;
}

struct cmd_result * treat_intcmd(struct cmdmod_ws *ws,struct cmd *command) {
  /* this function is called any time a pyrame command is received by cmdmod 
     functions are only type script
     command is the corresponding pyrame command structure
     ws is the cmdmod workspace
  */
  struct cmd_result *cres=NULL;
  struct cmd_result *pres;
  //objects for functions lookup
  struct function* func;

  func=search_func_byname(ws->sfdb,command->name);

  
  if (func==NULL) {

    //TODO: move to treat_ext when protocol gets @
    //hub feature : if a function is requested and not local and the module is registered then forward the call to module located by cmod
    if (ws->env->registered) {
      if (!strcmp(command->name+strlen(command->name)-5,"_cmod")) {
        //search first in the host function db
        func=search_func_byname(ws->hfdb,command->name);
        if (func==NULL) {
          func=create_hfdb_function(ws,command->name,"cmod",ws->cmod_ip);
        }
      } else {
        if (command->nb_params>=1) {
          func=search_cmod_function_in_db(ws,command->name,command->params[0]);
          if (func==NULL) {
            func=ask_cmod(ws,command->name,"notcmod",command->params[0]);
          }
        }
      }
      if (func!=NULL) {
	if (func->port!=atoi(env_getvar(ws->env,"system","listen_port"))) {
	  return treat_ext_function(ws,command,func);
	} else {
	  return build_result(0,"cant call myself through cmod on port (%d) func (%s)",func->port,func->name);
	}
      }
    }
    return build_result(0,"%s: %s: %s",env_getvar(ws->env,"system","name"),msg_unknown_command,command->name);
  } else {
    pthread_mutex_lock(ws->exec_mutex);
    if (func->type==SUBMOD_TYPE_SCRIPT) {
      cres=treat_script(ws,command,func);
    }
    if (func->type==SUBMOD_TYPE_INT) {
      cres=(*(func->fp))(ws,command);
    }
    pthread_mutex_unlock(ws->exec_mutex);
    if (cres->retcode==0) {
      pres=build_result(0,"%s: %s",env_getvar(ws->env,"system","name"),cres->str);
      free_cmd_result(cres);
      return pres;
    } else {
      return cres;
    }
  }
}

//****************************** HOST FUNCTIONS **************************


struct cmd_result * treat_ext_function(struct cmdmod_ws *ws,struct cmd *command,struct function* func) {
  int i;
  struct cmd_result *cres=NULL;
  char *tmp=command->name;
  command->name=func->name;

  debug("calling external function %s\n",command->name);
  if (func->conn==NULL) {
    printf("no connection available for func %s...exiting\n",func->name);
    cexit(1);
  }
  pthread_mutex_lock(func->conn->use_mutex);


  for(i=0;i<2;i++) {

    if (func->conn->socket==-1) {
      debug("opening socket for connection %s:%d\n",func->conn->host,func->conn->port);
      func->conn->socket=open_socket(func->conn->host,func->conn->port);
      if (func->conn->socket==-1) {
        pthread_mutex_unlock(func->conn->use_mutex);
        return build_result(0,"cant open socket to %s:%d",func->conn->host,func->conn->port);
      }
    }
    cres=exec_cmd(func->conn->socket,command);

    if (strcmp(cres->str,"Error sending command (socket closed)") && strcmp(cres->str,"Destination module unavailable (socket closed)"))
      break;
    func->conn->socket=-1;
  }
    

  command->name=tmp;
  pthread_mutex_unlock(func->conn->use_mutex);
  //debug("retcode=%d   res=%s\n",cres->retcode,cres->str);
  return cres;
}

struct function *create_hfdb_function(struct cmdmod_ws *ws,char *name,char *module,char *ip) {
  struct function *res=(struct function *)malloc(sizeof(struct function));
  int port;

  //get module port
  port=get_port(module);
  //debug("create host function %s module name=%s on %s:%d\n",name,module,ip,port);
  res->name=strdup(name);
  res->type=SUBMOD_TYPE_HOST;
  res->host=strdup(ip);
  res->port=port;
  res->portstring=NULL;
  res->conn=get_conn(ws,res->host,res->port);
  add_func(ws->hfdb,res);
  return res;
}

struct function *search_ext_function(struct cmdmod_ws *ws,struct cmd *command) {
  //objects for functions lookup
  struct function* func;
  char *module=NULL;
  int i=0;
  int j;
  char *funcname=strdup(command->name);
  char *tmp;
  char *id;
  int under_cnt=0;

  //rename func name from func@module to func_module
  //debug("searching @ in function name : %s\n",funcname);
  while(funcname[i]!=0) {
    if (funcname[i]=='_') {
      under_cnt++;
    }
    if (funcname[i]=='@') {
      funcname[i]='_';
      module=funcname+i+1;
    }
    i++;
  }

  //parse on last _
  if (module==NULL) {
    printf("warning : no @ found in function %s : DEPRECATED please migrate to new adressing system\n",funcname);
    if (under_cnt==0) {
      debug("no underscore found in function %s : unable to discover module name\n",funcname);
      return NULL;
    }
    i=0;
    j=0;
    //debug("find module name after _ number %d in %s\n",under_cnt,funcname);
    while(funcname[i]!=0) {
      if (funcname[i]=='_') {
        j++;
      }
      if (j==under_cnt) {
        module=funcname+i+1;
        //debug("module found : %s\n",module);
        break;
      }
      i++;
    }
  }

  //debug("searching for external function %s on module %s\n",funcname,module);
  
  tmp=command->name;
  command->name=funcname;

  //search first in the host function db
  func=search_func_byname(ws->hfdb,funcname);

  //if not found try cmod if available
  if (func==NULL) {//the function is not found
    //debug("function not found in host db\n");

    if (ws->env->registered) { //if cmod is available
      if (!strcmp(module,"cmod")) {
        func=create_hfdb_function(ws,funcname,module,ws->cmod_ip);
      } else {
        if (command->nb_params>=1) {
          id=command->params[0];
          //debug("searching function %s with id %d in cmod cache\n",funcname,id);
          //check if the func has already been cached
          func=search_cmod_function_in_db(ws,funcname,id);
          if (func==NULL) {
            debug("ask cmod for function %s with id %s\n",funcname,id);
            //not cache ask cmod
            func=ask_cmod(ws,funcname,module,id);
          }
        }
      }
    }
    
    //if everything failed, try to create localhost binding
    if (func==NULL) {
      //debug("function %s not found anywhere, creating localhost binding\n",funcname);
      func=create_hfdb_function(ws,funcname,module,"127.0.0.1");
    }
  }
  command->name=tmp;
  free(funcname);
  return func;
}


struct cmd_result * treat_extcmd(struct cmdmod_ws *ws,struct cmd *command) {
  /* this function is called any time submod.execcmd is fired.
     command is the corresponding pyrame command structure
     data is the cmdmod workspace
  */
  struct cmd_result *cres=NULL;

  //objects for functions lookup
  struct function* func;

  func=search_ext_function(ws,command);

  if (func==NULL) {
    cres=build_result(0,"%s: %s: %s",env_getvar(ws->env,"system","name"),msg_unknown_command,command->name);
  } else {
    cres=treat_ext_function(ws,command,func);
  }
  
  return cres;
}



  

