
#include "cmdmod.h"

void add_wrapper(struct wrapenv *we,char *name,struct cmd_result *(* wrapper)(char **),int nb_args) {
  struct submod_wrapper *new_wrap=malloc(sizeof(struct submod_wrapper));
  memset(new_wrap,0,sizeof(struct submod_wrapper));
  new_wrap->name=strdup(name);
  new_wrap->wrapper=wrapper;
  new_wrap->nb_args=nb_args;
  new_wrap->next=we->wrappers;
  we->wrappers=new_wrap;
}

struct submod_wrapper *lookup_wrapper(struct wrapenv *we,char *name) {
  struct submod_wrapper *iter=we->wrappers;
  while(iter!=NULL) {
    if (!strcmp(name,iter->name))
      return iter;
    iter=iter->next;
  }
  return NULL;
}

struct cmd_result *submod_wrap_bgcontext(char **args) {
  struct bg_ws *bg_context=submod_bgcontext();
  char *ptr;
  struct cmd_result *pres;
  ptr=ptr2str((void *)bg_context);
  pres=build_result(1,ptr);
  free(ptr);
  return pres;
}

struct cmd_result *submod_wrap_sendres(char **args) {
  char *context=args[0];
  int retcode=atoi(args[1]);
  char *retstr=args[2];
  struct cmd_result *cmdres=malloc(sizeof(struct cmd_result));
  struct bg_ws *bg_context=str2ptr(context);
  cmdres->retcode=retcode;
  cmdres->str=strdup(retstr);
  submod_sendres(bg_context,cmdres);
  return build_result(1,"ok");
}

struct cmd_result *submod_wrap_setvar(char **args) {
  char *ns=args[0];
  char *name=args[1];
  char *value=args[2];
  char type=atoi(args[3]);
  int res;
  res=submod_setvar(ns,name,value,type);
  if (res==0)
    return build_result(0,"no ns with name %s",ns);
  else
    return build_result(1,"ok");
}

struct cmd_result *submod_wrap_getvar(char **args) {
  char *ns=args[0];
  char *name=args[1];
  char *res;
  res=submod_getvar(ns,name);
  if (res==NULL)
    return build_result(0,"cant find variable %s in ns %s",name,ns);
  else
    return build_result(1,res);
}

struct cmd_result *submod_wrap_gettype(char **args) {
  char *ns=args[0];
  char *name=args[1];
  char type;
  type=submod_gettype(ns,name);
  if (type==-1)
    return build_result(0,"cant find variable %s in ns %s",name,ns);
  else
    return build_result(1,"%d",type);
}

struct cmd_result *submod_wrap_delvar(char **args) {
  char *ns=args[0];
  char *name=args[1];
  int res=submod_delvar(ns,name);
   if (res==0)
     return build_result(0,"cant find variable %s in ns %s",name,ns);
   else
     return build_result(1,"ok");
}

struct cmd_result *submod_wrap_newns(char **args) {
  char *ns=args[0];
  int res=submod_newns(ns);
  if (res==0)
    return build_result(0,"unable to create ns with that name");
  else
    return build_result(1,"ok");
}

struct cmd_result *submod_wrap_checkns(char **args) {
  char *ns=args[0];
  int res=submod_checkns(ns);
  if (res==0)
    return build_result(0,"unknwon ns");
  else
    return build_result(1,"ok");
}

struct cmd_result *submod_wrap_delns(char **args) {
  char *ns=args[0];
  int res=submod_delns(ns);
  if (res==0)
    return build_result(0,"unknown ns");
  else
    return build_result(1,"ok");
}

struct cmd_result *submod_wrap_listns(char **args) {
  char *res=submod_listns();
  return nofree_build_result(1,res);
}

struct cmd_result *submod_wrap_dumpns(char **args) {
  char *ns=args[0];
  char *res=submod_dumpns(ns);
  struct cmd_result *pres;
  if (res==NULL)
    return build_result(0,"unknwon ns");
  else  {
    pres=build_result(1,res);
    free(res);
    return pres;
  }
}

//************************* DISPATCHER ***********************

struct cmd_result *submod_wrap_setfieldblock(char **args) {
  char *block_addr=args[0];
  char *field_name=args[1];
  char *value=args[2];
  struct block *bl;
  struct evtstructelem *ese;
  int res;
  bl=str2ptr(block_addr);
  ese=get_ese(bl->es,field_name);
  if (ese==NULL) {
    return build_result(0,"unknown field name %s",field_name);
  }
  res=submod_set_field_block(bl,ese,value);
  if (res<0) {
    return raw_build_result(0,pyrame_errors[-res]);
  }
  return raw_build_result(1,"ok");
}

struct cmd_result *submod_wrap_setfieldevent(char **args) {
  char *event_addr=args[0];
  char *field_name=args[1];
  char *value=args[2];
  struct event *ev;
  struct evtstructelem *ese;
  int res;
  ev=str2ptr(event_addr);
  ese=get_ese(ev->block->es,field_name);
  if (ese==NULL) {
    return build_result(0,"unknown field name %s",field_name);
  }
  res=submod_set_field_event(ev,ese,value);
  if (res<0) {
    return raw_build_result(0,pyrame_errors[-res]);
  }
  return raw_build_result(1,"ok");
}

struct cmd_result *submod_wrap_newblock(char **args) {
  int block_id=atoi(args[0]);
  struct block *bl;
  int res;
  res=submod_new_block(block_id,&bl);
  if (res<0) {
    return raw_build_result(0,pyrame_errors[-res]);
  }
  return nofree_build_result(1,ptr2str((void *)bl));
}

struct cmd_result *submod_wrap_newevent(char **args) {
  struct block *bl=str2ptr(args[0]);
  struct event *ev;
  int res;
  res=submod_new_event(bl,&ev);
  if (res<0) {
    return raw_build_result(0,pyrame_errors[-res]);
  }
  return nofree_build_result(1,ptr2str((void *)ev));
}

struct cmd_result *submod_wrap_finalizeblock(char **args) {
  struct block *bl=str2ptr(args[0]);
  int res;
  res=submod_finalize_block(bl);
  if (res<0) {
    return raw_build_result(0,pyrame_errors[-res]);
  }
  return raw_build_result(1,"ok");
}

struct cmd_result *submod_wrap_setchans(char **args) {
  char *schema=args[0];
  int res;
  res=submod_set_chans(schema);
  if (res<0) {
    return raw_build_result(0,pyrame_errors[-res]);
  }
  return raw_build_result(1,"ok");
}

struct cmd_result *submod_wrap_initdispatcher(char **args) {
  int nb_blocks=atoi(args[0]);
  char *name=args[1];
  char *schema_str=args[2];
  char *port=args[3];
  int res;
  res=submod_init_dispatcher(nb_blocks,name,schema_str,port);
  if (res<0) {
    return raw_build_result(0,pyrame_errors[-res]);
  }
  return build_result(1,"%d",res);
}

struct cmd_result *submod_wrap_flushdispatcher(char **args) {
  int res=submod_flush_dispatcher();
  if (res<0) {
    return raw_build_result(0,pyrame_errors[-res]);
  }
  return build_result(1,"%d",res);
}

struct cmd_result *submod_wrap_wakemeup(char **args) {
  int time=atoi(args[0]);
  char *func=args[1];
  char *param=args[2];
  int res=submod_wakemeup(time,func,param);
  if (res<0) {
    return raw_build_result(0,pyrame_errors[-res]);
  }
  return raw_build_result(1,"ok");
}

struct wrapenv *new_wrapenv() {
  struct wrapenv *result=malloc(sizeof(struct wrapenv));
  memset(result,0,sizeof(struct wrapenv));
  add_wrapper(result,"bgcontext",submod_wrap_bgcontext,0);
  add_wrapper(result,"sendres",submod_wrap_sendres,3);
  add_wrapper(result,"setvar",submod_wrap_setvar,4);
  add_wrapper(result,"getvar",submod_wrap_getvar,2);
  add_wrapper(result,"gettype",submod_wrap_gettype,2);
  add_wrapper(result,"delvar",submod_wrap_delvar,2);
  add_wrapper(result,"newns",submod_wrap_newns,1);
  add_wrapper(result,"delns",submod_wrap_delns,1);
  add_wrapper(result,"checkns",submod_wrap_checkns,1);
  add_wrapper(result,"dumpns",submod_wrap_dumpns,1);
  add_wrapper(result,"listns",submod_wrap_listns,0);
  add_wrapper(result,"newblock",submod_wrap_newblock,1);
  add_wrapper(result,"newevent",submod_wrap_newevent,1);
  add_wrapper(result,"setfieldblock",submod_wrap_setfieldblock,3);
  add_wrapper(result,"setfieldevent",submod_wrap_setfieldevent,3);
  add_wrapper(result,"setchans",submod_wrap_setchans,1);
  add_wrapper(result,"finalizeblock",submod_wrap_finalizeblock,1);
  add_wrapper(result,"initdispatcher",submod_wrap_initdispatcher,4);
  add_wrapper(result,"flushdispatcher",submod_wrap_flushdispatcher,0);
  add_wrapper(result,"wakemeup",submod_wrap_wakemeup,3);
  return result;
}




