/*
Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
This file is part of Pyrame.

Pyrame is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Pyrame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
*/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <pthread.h>
#include <dlfcn.h>
#include <math.h>
#include <cmdmod_helper.h>

#ifndef CONVERTER_H
#define CONVERTER_H

#define SHARED_BUFFER_SIZE 8192

#define SLOT_NB 1000000
#define BLOCK_NB 500

#define EBROK -1
#define EAVOID -2

enum input_type {UNKNOWN,SHMEM,TCP,OFILE};
enum mode_type {ONLINE,OFFLINE};

struct conv_ws {
  void *sws;
  char *ds_name;
  char *cmd_port;
  enum input_type itype;
  pthread_t *t_converter;
  enum mode_type output_mode;
  //shmem mode
  int shmemid;
  struct shbuffer *shbuf;
  //tcp mode 
  char *host;
  int iport;
  int data_socket;
  //file mode 
  char *filename;
  FILE *finput;
  //convert plugin
  char * plugin_name;
  void *lib;
  void (*convert)(void *);
  void (*init)(void *);
  void (*reinit)(void *);
  struct convlib_ws *lws;
  char *schema_str;
  struct evtstruct *schema;
  //statistics
  char *nbe_stat_name;
  struct stats_ws *stats;
  //clock
  int cport;
  char * clock;
  int clock_socket;
  //ribbon structure
  char *buffer;
  char *data_start;
  int data_size;
  int buffer_size;
  //deinit 
  int request_deinit;
  int finished;
  //start_acq
  char ready;
  //debug
  char debug;
  //params
  int argc;
  char **argv;
};

//private functions prototypes
struct conv_ws * init_ws();


//ribbon functions
void init_ribbon(struct conv_ws *ws,int size);
void flush_ribbon(struct conv_ws *ws);
//drop the data previous to the current position
int forget_before_current(struct conv_ws *ws);
//get the current character of the ribbon
int cur_char(struct conv_ws *ws,char *c);
//moving forward on the ribbon of "size" bytes
int seek_forward(struct conv_ws *ws,int size);
//moving forward on the ribbon of "size" bytes : CAUTION : no protection
int seek_backward(struct conv_ws *ws,int size);
//moving forward on the ribbon until finding the char term
int forward_until(struct conv_ws *ws,char term);
//moving forward on the ribbon until finding the string term_str
int forward_until_str(struct conv_ws *ws,char *term_str,char *avoid_str);
//read "size" bytes forward
int read_nb(struct conv_ws *ws,char **result,int size);
//read the ribbon until finding the char term1. If the avoid char is found EAVOID is returned
int read_until(struct conv_ws *ws,char **result,char term1,char avoid);
//read the ribbon until finding the char term1 or term2. If the avoid char is found EAVOID is returned
int read_until_or(struct conv_ws *ws,char **result,char term1,char term2,char avoid);
//read the ribbon until a string is found : return all the data from the data_start to the string included
int read_until_str(struct conv_ws *ws,char **result,char *term_str, char *avoid_str);


//tools functions
int request_new_data(struct conv_ws *ws,char * buffer);
void update_clock(struct conv_ws *ws,char * new_clock);

//plugin functions prototype
void init_plugin(void *vdata);
void convert(void *);

#endif
