//
// Copyright 2012-2014 Frédéric Magniette, Miguel Rubio-Roy, LLR
//

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#include "../cmd_converter.h"


struct hle_ws {
  int current_block_nb;
  struct block *current_block;
  char *blockid_field;
  char first_time;
  int debug;
  struct evtstruct *schema;
};

//read a hle
int parse_hle(struct conv_ws *ws,struct hle_ws *sws) {
  char *block_str;
  char current;
  //int res;
  int block_nb;

  if (sws->first_time) {
    sws->first_time=0;
    printf("no need for \\n because first time\n");
  } else {
    //check for \n at end of block
    if (cur_char(ws,&current)==-1)
      return -1;
    if (current!='\n') {
      printf("lost : moving to next line\n");
      if (forward_until(ws,'\n')==EBROK)
        return EBROK;
      forget_before_current(ws);
      return 0;
    }
    if (seek_forward(ws,1)==EBROK)
      return EBROK;
    //printf("\\n found\n");
  }

  //printf("reading from ribbon until \\n\n");
  if (read_until(ws,&block_str,'\n',-1)==EBROK)
    return EBROK;
  
  //printf("parsing block\n");
  sws->current_block=saf_parse_block(block_str);
  if (sws->current_block==NULL) {
    printf("error in parsing current block\n");
    forget_before_current(ws);
    return 0;
  }
  sws->current_block->es=sws->schema;
  if (sws->debug) {
    printf("blockid field=%s\n",sws->blockid_field);
    printf("new block : \n");
    print_block(sws->current_block);
  }
    
  sws->current_block_nb++;
  if (check_block(sws->current_block)==0) { 
    if (!strcmp(sws->blockid_field,"undef")) {
      block_nb=sws->current_block_nb;
    } else {
      block_nb=atoi(get_block_field(sws->current_block,sws->blockid_field));
    }
    submod_add_block(sws->current_block,block_nb);
    //printf("new block %d added\n",block_nb);
  } else {
    printf("block %d : bad format %s\n",sws->current_block_nb,block_str);
  }
  //printf("forget before current\n");
  forget_before_current(ws);
  return 0;
  
}

void finalize(struct conv_ws *ws,struct hle_ws *sws) {
  printf("no more data...finalizing\n");
  while(1) {
    usleep(1000000);
    //printf("decoder finalized\n");
  }
}
      
void init_plugin(void *vdata) {
  //global workspace herited from converter/dispatcher
  struct conv_ws *ws=vdata;

  //create local workspace
  struct hle_ws *sws=malloc(sizeof(struct hle_ws));
  sws->current_block_nb=0;
  sws->first_time=1;
  //change next line to activate debug
  sws->debug=0;
  
  if (ws->argc!=2) {
    printf("usage: hle_decoder needs 2 args : schema,blockid_field but has %d \n",ws->argc);
    finalize(ws,sws);
  }

  // fill in converter struct
  ws->sws=sws;
  ws->schema_str=ws->argv[0];
  sws->schema=saf_parse_chans(ws->argv[0]);
  if (strcmp(ws->argv[1],"undef")) {
    if (!exists_prop(sws->schema,ws->argv[1])) {
      printf("blockfield_id %s is not a valid property\n",ws->argv[1]);
      finalize(ws,sws);
    } 
  }
  sws->blockid_field=strdup(ws->argv[1]);
  printf("blockid_field=%s\n",sws->blockid_field);

  //initialize the data stream
  init_ribbon(ws,SHARED_BUFFER_SIZE*10);
}

void reinit(void *vdata) {
  struct conv_ws *ws=vdata;
  struct hle_ws *sws=(struct hle_ws *)ws->sws;
  sws->first_time=1;
  sws->current_block_nb=0;
}

void convert(void *vdata) {
  struct conv_ws *ws=vdata;
  struct hle_ws *sws=(struct hle_ws *)ws->sws;
  pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS,NULL);
  ws->ready=1;
  while(1) {
    if (parse_hle(ws,sws)==-1) {
      finalize(ws,sws);
      return;
    }
  }
}

