#include "skiroc2cms_decoder.h"

#ifdef ROOTP
#include "skiroc_ROOTana.hh" 
#endif

int gray2bin(long n,int size) {
  int ish, ans, idiv;
  ish = 1;
  ans = n;
  while(1) {
    idiv = ans >> ish;
    ans ^= idiv;
    if (idiv <= 1 || ish == size) 
      return ans;
    ish <<= 1; // double le nb de shifts la prochaine fois
  }
}



void print_data(char *str,int size) {
  int i;
  for (i=0;i<size;i++) {
    printf("%02x",(unsigned char)str[i]);
  }
  printf("\n");
}

int convert_difint(char *intstr) {
  unsigned short integ[2];
  integ[0]=*((unsigned short *)(intstr+2));
  integ[1]=*((unsigned short *)(intstr));
  return *((int *)integ);
}

//search for the stop spill pattern in the data 
//manage the error pattern
//return 0 if success
//return -1 if failure
int search_next_spill_stop(struct conv_ws *ws,struct skiroc2cms_ws *sws) {
  char *data;
  int size;
  size=read_until_str(ws,&data,spill_trailer,NULL);
  if (size==EBROK)
    return EBROK;
  //printf("spill stop found\n");
  //printf("data=");
  //print_data(data,size);
  free(data);
  seek_backward(ws,2);
  return 0;
}

//search for the start spill pattern in the data 
//manage the error pattern
//return 0 if success
//return -1 if failure
int search_next_spill_start(struct conv_ws *ws,struct skiroc2cms_ws *sws) {
  char *data;
  int size;
  size=read_until_str(ws,&data,spill_header,NULL);
  if (size==EBROK)
    return EBROK;
  //printf("spill start found\n");
  //printf("data=");
  //print_data(data,size);
  free(data);
  seek_backward(ws,2);
  return 0;
}



//parse the data beam for finding the headers and go to the data
//return 0 if no new action is required
//return 1 if chip data is available
//return -1 if no data
int parse_headers(struct conv_ws *ws,struct skiroc2cms_ws *sws) {
  char *str_head_code;
  unsigned short head_code;
  char *str_spill_nb;
  unsigned int spill_nb;
  char *str_chip_nb;
  int chip_nb;
  unsigned int corrected_spill;
  int res;
  struct block *b;
  
  //printf("parse_headers\n");

  //read the header code
  if (read_nb(ws,&str_head_code,2)==EBROK)
    return EBROK;
  head_code=*(unsigned short *)str_head_code;
  free(str_head_code);
  //printf("head_code=%x\n",head_code);
  
  if (head_code==start_spill_code) { //found a new spill
    
    //verify that the previous block had been finalized
    if (sws->block!=NULL) {
      printf("error : block has not been finalized\n");
      if (ws->stats)
        set_stats(ws->stats,"nb_spilltrail_error",SOP_PLUS,1.0); 
      submod_finalize_block(sws->block);
    }

    //read the spill number
    if (read_nb(ws,&str_spill_nb,4)==EBROK)
      return EBROK;
    
    //get the spill from data
    spill_nb=convert_difint(str_spill_nb);
    free(str_spill_nb);
    printf("new spill %x\n",spill_nb);
    corrected_spill=spill_nb;
    //printf("offset=%d cumul=%d spill=%d corrected=%d\n",sws->spill_offset,sws->spill_cumul,spill_nb,corrected_spill);

    //
    sws->current_spill=corrected_spill;

    //set stats
    if (ws->stats) {
      set_stats(ws->stats,"nb_spills",SOP_PLUS,1.0);
      set_stats(ws->stats,"cur_spill",SOP_SET,corrected_spill);
    }
    
    //create a data block
    res=submod_new_block(spill_nb,&b);
    if (res<0) {
      printf("warning : spill nb desynchronization : going to next spill start\n");
      res=search_next_spill_start(ws,sws);
      if (res==EBROK)
        return EBROK;
      forget_before_current(ws);
      return 0;
    }
    sws->block=b;

    
    submod_set_field_block(sws->block,sws->ese_spill,"%d",corrected_spill);
    
    //read and discard the ascii code
    if (seek_forward(ws,6)==EBROK)
      return EBROK;
   
    return 0;
  }
  
  
  if (head_code==start_chip_code) { //found a new chip
      
    //read chip number
    if (read_nb(ws,&str_chip_nb,1)==EBROK)
      return EBROK;
    chip_nb=(*(char *)str_chip_nb);
    free(str_chip_nb);
    //printf("new chip %x\n",chip_nb);
    
    //update the number of corresponding stream
    sws->current_chip=chip_nb-1;
    
    //discard a useless byte
    if (seek_forward(ws,1)==EBROK)
      return EBROK;
    
    //discard the ascii code
    if (seek_forward(ws,6)==EBROK)
      return EBROK;

    return 1;
  }
    
  if (head_code==stop_spill_code) { //found a spill trailer
    
    //read the spill number
    if (read_nb(ws,&str_spill_nb,4)==EBROK)
      return EBROK;
    spill_nb=convert_difint(str_spill_nb);
    free(str_spill_nb);
    //printf("spill trailer %x\n",spill_nb);

    //finalize the data block corresponding to this spill
    submod_finalize_block(sws->block);
    sws->block=NULL;

    //allow converter to recycle previous data space
    forget_before_current(ws);
    //printf("spill trailer : searching next spill start\n");
    res=search_next_spill_start(ws,sws);
    if (res==EBROK)
      return EBROK;
    return 0;
  }

  if (head_code==sws->current_spill+1) {
    //printf("considering as a lost spill header\n");
    sws->current_spill++;
    //read and discard the ascii code
    if (seek_forward(ws,6)==EBROK)
      return EBROK;
   
    return 0;
  }
  
  printf("I'm lost... moving to next spill start\n");
  res=search_next_spill_start(ws,sws);
  if (res==EBROK)
    return EBROK;
  forget_before_current(ws);
  return 0;
}

int create_online_sca_events(struct conv_ws *ws,struct skiroc2cms_ws *sws,int *gain1_energy,int *gain2_energy,int *hits,char *cut,int bcid) {
  int chan;
  int res;
  //double spill=(double)sws->current_spill;
  int roc;
  struct event *e;

  if (sws->block) {
    for(chan=0;chan<64;chan++) {
      if (cut[chan])
          continue;
      if (hits[chan]==1) {
        res=submod_new_event(sws->block,&e);
        if (res<0) {
          printf("cant create event\n");
          return 0;
        }
        //strdup("spill,dif,z!bcid|roc,chan|hit,en1,en2")
        submod_set_field_event(e,sws->ese_bcid,"%d",bcid);
        roc=sws->current_chip;
        submod_set_field_event(e,sws->ese_roc,"%d",roc);
        submod_set_field_event(e,sws->ese_chan,"%d",chan);
        submod_set_field_event(e,sws->ese_hit,"%d",hits[chan]);
        submod_set_field_event(e,sws->ese_en,"%d",gain1_energy[chan]);
        submod_set_field_event(e,sws->ese_en2,"%d",gain2_energy[chan]);
      }
    }
  } else {
    printf("error : events are available but no block to handle them\n");
  }
  return 0;
}

int create_offline_sca_events(struct conv_ws *ws,struct skiroc2cms_ws *sws,int *gain1_energy,int *gain2_energy,int *hits,char *cut,int bcid) {
  //double spill=(double)sws->current_spill;
  return 0;
}


//read the data for a chip and output it
//return -1 in case of read error
//return 0 in other cases
int parse_chip(struct conv_ws *ws,struct skiroc2cms_ws *sws) {
  char *data;
  int nb_words;
  int i;
  int tot;
  int chan;
  int sca;
  int hg_charge,hg_hit,lg_charge,lg_hit;
  int extra_bit,global_ts,roll_pos;

  unsigned short *sdata;
  int rsize;
  int res;

  //searching a chip trailer
  //printf("searching for chip trailer, avoiding spill header\n");
  rsize=read_until_str(ws,&data,chip_trailer,spill_header);
  if (rsize==EBROK)
    return EBROK;
  if (rsize==EAVOID) {
    printf("warning : spill start detected while parsing chip data...moving to next spill start\n");
    res=search_next_spill_start(ws,sws);
    if (res==EBROK)
      return EBROK;
    forget_before_current(ws);
    return 0;
  }

  //searching for an error in the data
  for(i=0;i<rsize-1;i++) {
    if (*((unsigned short *)(data+i))==loss_code) {
      printf("warning : loss detected at position %d in the data...moving to next spill start\n",i);
      free(data);
      res=search_next_spill_start(ws,sws);
      if (res==EBROK)
        return EBROK;
      forget_before_current(ws);
      return 0;
    }
  }

  //computing the size of data
  nb_words=rsize/2;
  
  if (nb_words!=1925) {
    printf("bad size of data chunk : %d (sould be 1925)\n",nb_words);
  }

  //convert data string to shorts
  sdata=(unsigned short *)data;

  //reading chip id
  sws->current_chip=gray2bin(sdata[1923]&0xff,8);
  printf("chip id : %d\n",sws->current_chip);

  extra_bit=sdata[1922]&1;
  global_ts=gray2bin((sdata[1921]<<14)+(sdata[1922]>>1),26);
  printf("global ts=%d and (extra bit=%d)\n",global_ts,extra_bit);

  roll_pos=sdata[1920];
  printf("roll_pos=%d\n",roll_pos);
	
  printf("extracting TOT fast information\n");
  for(chan=0;chan<64;chan++) {
    tot=gray2bin(sdata[1919-chan]&0xfff,12);
    printf("channel %d : value=%d\n",chan,tot);
  }

  printf("extracting TOT slow information\n");
  for(chan=0;chan<64;chan++) {
    tot=gray2bin(sdata[1919-64-chan]&0xfff,12);
    printf("channel %d : value=%d\n",chan,tot);
  }

  printf("extracting TOA rising information\n");
  for(chan=0;chan<64;chan++) {
    tot=gray2bin(sdata[1919-128-chan]&0xfff,12);
    printf("channel %d : value=%d\n",chan,tot);
  }

  printf("extracting TOA falling information\n");
  for(chan=0;chan<64;chan++) {
    tot=gray2bin(sdata[1919-192-chan]&0xfff,12);
    printf("channel %d : value=%d\n",chan,tot);
  }
  

  //reading data
  for (sca=0;sca<13;sca++) {
    for(chan=0;chan<64;chan++) {

      hg_charge=gray2bin(sdata[1663-chan-sca*128]&0xfff,12);
      hg_hit=(sdata[1663-chan-sca*128]>>12)&1;

      lg_charge=gray2bin(sdata[1663-chan-sca*128-64]&0xfff,12);
      lg_hit=(sdata[1663-chan-sca*128-64]>>12)&1;

      printf("sca %d chan %d : low gain charge=%d ",sca,chan,lg_charge);
      if (lg_hit)
        printf("   HIT!\n");
      else
        printf("\n");

      printf("sca %d chan %d : high gain charge=%d ",sca,chan,hg_charge);
      if (hg_hit)
        printf("   HIT!\n");
      else
        printf("\n");
    }
  }


  /*if (ws->output_mode==ONLINE)
    create_online_sca_events(ws,sws,(int *)gain1_energy,(int *)gain2_energy,(int *)hits,(char *)cut,bcid);
  else
  create_offline_sca_events(ws,sws,(int *)gain1_energy,(int *)gain2_energy,(int *)hits,(char *)cut,bcid);*/

  //free the data
  free(data);
  
  //trash the chip trailer 6 bytes
  if (seek_forward(ws,6)==EBROK)
    return EBROK;
  
  //allow converter to recycle previous data space
  forget_before_current(ws);
  
  return 0;
}
      
void finalize(struct conv_ws *ws,struct skiroc2cms_ws *sws) {
  printf("no more data...finalizing\n");
  if (ws->output_mode==ONLINE) {
    while(1) {
      usleep(1000000);
      printf("decoder finalized\n");
    }
  } else {
    #ifdef ROOTP
    printf("dumping root tree\n");
    ROOTana *sktree=(ROOTana *)sws->root_tree;
    sktree->Save("test");
    #endif
    return;
  }
}

int parse_args(struct conv_ws *ws,struct skiroc2cms_ws *sws) {
  if (ws->argc!=0) {
    printf("skiroc2cms decoder needs 0 parameters\n");
    return -1;
  }
  
  return 0;
}

void init_plugin(void *vdata) {
  //global workspace herited from converter/dispatcher
  struct conv_ws *ws=(struct conv_ws *)vdata;
  
  //create local workspace
  struct skiroc2cms_ws *sws=(struct skiroc2cms_ws *)malloc(sizeof(struct skiroc2cms_ws));
  memset(sws,0,sizeof(struct skiroc2cms_ws));
  //sws->dif=ws->streamid;
  sws->spill_offset=-1;
  sws->spill_cumul=0;
  sws->current_spill=0;
  sws->block=NULL;

  #ifdef ROOTP
  sws->root_tree=(void *)new ROOTana( );
  #endif

  if (parse_args(ws,sws)<0) {
    //finalize(ws,sws);
    sleep(1);
    exit(1);
  }

  ws->sws=sws;
  //ws->schema_str=strdup("spill,dif,p,z!bcid,time|roc,chan,c,l,x,y|hit,en,en2");
  ws->schema_str=strdup("spill!bcid|roc,chan|hit,en,en2");
  ws->schema=saf_parse_chans(ws->schema_str);
  sws->ese_bcid=get_ese(ws->schema,"bcid");
  sws->ese_roc=get_ese(ws->schema,"roc");
  sws->ese_chan=get_ese(ws->schema,"chan");
  sws->ese_hit=get_ese(ws->schema,"hit");
  sws->ese_en=get_ese(ws->schema,"en");
  sws->ese_en2=get_ese(ws->schema,"en2");

  //initialize the stats system
  if (ws->stats) {
    add_acc(ws->stats,"nb_hits",0);
    add_acc(ws->stats,"nb_spills",0);
    add_inst(ws->stats,"cur_spill",0);
    
    set_stats(ws->stats,"nb_spills",SOP_SET,0);
    set_stats(ws->stats,"cur_spill",SOP_SET,0);
  }    

  //initialize the data stream
  init_ribbon(ws,8192*50);

}

void reinit(void *vdata) {
  struct conv_ws *ws=(struct conv_ws *)vdata;
  struct skiroc2cms_ws *sws=(struct skiroc2cms_ws *)ws->sws;
  sws->spill_offset=-1;
  sws->spill_cumul=0;
  sws->current_spill=0;
}

void convert(void *vdata) {
  struct conv_ws *ws=(struct conv_ws *)vdata;
  struct skiroc2cms_ws *sws=(struct skiroc2cms_ws *)ws->sws;
  int res;
  pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS,NULL);
  ws->ready=1;
  while(1) {
    res=parse_headers(ws,sws);
    if (res==EBROK) {
      finalize(ws,sws);
      return;
    }
    if (res==1) {
      res=parse_chip(ws,sws);
      if (res==EBROK) {
        finalize(ws,sws);
        return;
      }
    }
  }
}
