#include <skiroc_ROOTana.hh>

// Constructor
//============================================================
ROOTana::ROOTana()
{
   // create TTree
   tree = new TTree("SK2CMS_data","Data from SKIROC chip");
   
   // define tree branches
   SetTreeBranches();
   
   // set counters/init data
   EventNumber = 0;

   cout << "[INFO] Root file created. Ready to save data." << endl;
}
//============================================================


// Destructor
//====================
ROOTana::~ROOTana()
{
}
//====================


// Set Branches
//============================================================================================
void ROOTana::SetTreeBranches(){
   
   name = "EventNumber/I";
   tree->Branch("EventNumber",&EventNumber,name);
   
   name = "ChipId/I";
   tree->Branch("ChipId",&ChipId,name);
   
   name = "BunchCrossingId/I";
   tree->Branch("BunchCrossingId",&BunchCrossingId,name);
   
   name = "TOT_Fast[";name+=NCHANNELS; name+="]/I";
   tree->Branch("TOT_Fast",&TOT_Fast,name);
   
   name = "TOT_Slow[";name+=NCHANNELS; name+="]/I";
   tree->Branch("TOT_Slow",&TOT_Slow,name);
   
   name = "TOA_Rising[";name+=NCHANNELS; name+="]/I";
   tree->Branch("TOA_Rising",&TOA_Rising,name);
   
   name = "TOA_Falling[";name+=NCHANNELS; name+="]/I";
   tree->Branch("TOA_Falling",&TOA_Falling,name);
   
   name = "Hit_TOT_Fast[";name+=NCHANNELS; name+="]/I";
   tree->Branch("Hit_TOT_Fast",&Hit_TOT_Fast,name);
   
   name = "Hit_TOT_Slow[";name+=NCHANNELS; name+="]/I";
   tree->Branch("Hit_TOT_Slow",&Hit_TOT_Slow,name);
   
   name = "Hit_TOA_Rising[";name+=NCHANNELS; name+="]/I";
   tree->Branch("Hit_TOA_Rising",&Hit_TOA_Rising,name);
   
   name = "Hit_TOA_Falling[";name+=NCHANNELS; name+="]/I";
   tree->Branch("Hit_TOA_Falling",&Hit_TOA_Falling,name);
   
   name = "RollPosition[";name+=MEMDEPTH; name+="]/I";
   tree->Branch("RollPosition",&RollPosition,name);
   
   name = "HG_Charge[";name+=MEMDEPTH; name+="]["; name+=NCHANNELS; name+="]/I";
   tree->Branch("HG_Charge",&HG_Charge,name);
   
   name = "LG_Charge[";name+=MEMDEPTH; name+="]["; name+=NCHANNELS; name+="]/I";
   tree->Branch("LG_Charge",&LG_Charge,name);
   

}
//============================================================================================



// Add SCA
//============================================================================================
void ROOTana::AddSCA(int ChipId_input, int i_sca, int *HG_Charge_input,int *LG_Charge_input,int RollPosition_input)
{
   // Stores info from single SCA memory into array tree variables
   if(ChipId != ChipId_input)
   {
      cout << "[ERROR] Chip ID does not match!" << endl;
      abort();
   }
   // fill BCID
   RollPosition[i_sca] = RollPosition_input;
   
   // fill the charges
   for (int i_chan = 0; i_chan < NCHANNELS; i_chan++)
   {
      HG_Charge[i_sca][i_chan] = HG_Charge_input[i_chan];
      LG_Charge[i_sca][i_chan] = LG_Charge_input[i_chan];
   }
}
//============================================================================================


// Add Timing
//============================================================================================
void ROOTana::AddTiming(int ChipId_input, int *TOT_Fast_input, int *TOT_Slow_input, int *TOA_Rising_input, int *TOA_Falling_input)
{
   // Stores info from single SCA memory into array tree variables
   if(ChipId != ChipId_input)
   {
      cout << "[ERROR] Chip ID does not match!" << endl;
      abort();
   }
   // Fill the timing info
   for (int i_chan = 0; i_chan < NCHANNELS; i_chan++)
   {
      TOT_Fast[i_chan]    = TOT_Fast_input[i_chan];
      TOT_Slow[i_chan]    = TOT_Slow_input[i_chan];
      TOA_Rising[i_chan]  = TOA_Rising_input[i_chan];
      TOA_Falling[i_chan] = TOA_Falling_input[i_chan];
   }
}
//============================================================================================



// Add Hit
//============================================================================================
void ROOTana::AddHit(int ChipId_input, int *Hit_TOT_Fast_input, int *Hit_TOT_Slow_input, int *Hit_TOA_Rising_input, int *Hit_TOA_Falling_input)
{
   // Stores info from single SCA memory into array tree variables
   if(ChipId != ChipId_input)
   {
      cout << "[ERROR] Chip ID does not match!" << endl;
      abort();
   }
   // Fill the timing info
   for (int i_chan = 0; i_chan < NCHANNELS; i_chan++)
   {
      Hit_TOT_Fast[i_chan]    = Hit_TOT_Fast_input[i_chan];
      Hit_TOT_Slow[i_chan]    = Hit_TOT_Slow_input[i_chan];
      Hit_TOA_Rising[i_chan]  = Hit_TOA_Rising_input[i_chan];
      Hit_TOA_Falling[i_chan] = Hit_TOA_Falling_input[i_chan];
   }
}
//============================================================================================



// Start Event
//=======================================================================
void ROOTana::StartEvent(int ChipId_input, int BunchCrossingId_input)
{
   // Set Event data
   EventNumber++;
   ResetData(ChipId_input, BunchCrossingId_input);
}
//=======================================================================


// Reset Data
//=======================================================================
void ROOTana::ResetData(int ChipId_input, int BunchCrossingId_input)
{
   // Set ChipId and init data values to -999
   ChipId = ChipId_input;
   BunchCrossingId = BunchCrossingId_input;
   
   for (int i_chan=0; i_chan < NCHANNELS; i_chan++)
   {
      TOT_Fast[i_chan]       =-999;
      TOT_Slow[i_chan]       =-999;
      TOA_Rising[i_chan]     =-999;
      TOA_Falling[i_chan]    =-999;
      
      Hit_TOT_Fast[i_chan]   =-999;
      Hit_TOT_Slow[i_chan]   =-999;
      Hit_TOA_Rising[i_chan] =-999;
      Hit_TOA_Falling[i_chan]=-999;
      
      for (int i_sca=0; i_sca < MEMDEPTH; i_sca++)
      {
         HG_Charge[i_sca][i_chan]=-999;
         LG_Charge[i_sca][i_chan]=-999;
      }
   }
   
   for (int i_sca=0; i_sca < MEMDEPTH; i_sca++)
   {
      RollPosition[i_sca]=-999;
   }
   
}
//=======================================================================


// Fill Event
//=======================================================================
void ROOTana::FillEvent()
{
   // Fill event into tree
   tree->Fill();
}
//=======================================================================


// Save
//=======================================================================
void ROOTana::Save(TString outFileName)
{
   // Write tree to a root file
   fout = new TFile(outFileName+".root","RECREATE");
   tree->Write();
   fout->Close();
   cout << "[INFO] Everything done and saved in a root file." << endl;
}
//=======================================================================

