#include "skiroc_decoder.h"

struct map_cell **load_mapping(char *filename,int nb_chips,int nb_chans) {
  
  struct map_cell **mapping;
  FILE *f;
  int i;
  int chip,chan;
  int c,l;
  float x,y;
  int res;

  //allocate mapping
  mapping=malloc(sizeof(struct map_cell *)*nb_chips);
  for (i=0;i<nb_chips;i++)
    mapping[i]=malloc(sizeof(struct map_cell)*nb_chans);

  //open mapping file
  f=fopen(filename,"r");
  if (f==NULL) {
    printf("cant open file %s\n",filename);
    return NULL;
  }
  
  //reading mapping
  while(!feof(f)) {
    res=fscanf(f,"%d %d %f %f %d %d",&chip,&chan,&x,&y,&c,&l);
    if ((res!=6)&&(res!=-1)) {
      printf("syntax error in file %s obtained %d items instead of 6\n",filename,res);
    } /*else {
      printf("chip=%d chan=%d x=%f y=%f c=%d l=%d\n",chip,chan,x,y,c,l); 
      }*/
    mapping[chip][chan].x=x;
    mapping[chip][chan].y=y;
    mapping[chip][chan].c=c;
    mapping[chip][chan].l=l;
  }
  fclose(f);
  return mapping;
}

void free_mapping(struct map_cell **mapping,int nb_chips) {
  int i;
  for (i=0;i<nb_chips;i++)
    free(mapping[i]);
  free(mapping);
}

void print_data(char *str,int size) {
  int i;
  for (i=0;i<size;i++) {
    printf("%02x",(unsigned char)str[i]);
  }
  printf("\n");
}

int convert_difint(char *intstr) {
  unsigned short integ[2];
  integ[0]=*((unsigned short *)(intstr+2));
  integ[1]=*((unsigned short *)(intstr));
  return *((int *)integ);
}

//search for the stop spill pattern in the data 
//manage the error pattern
//return 0 if success
//return -1 if failure
int search_next_spill_stop(struct conv_ws *ws,struct skiroc_ws *sws) {
  char *data;
  int size;
  size=read_until_str(ws,&data,spill_trailer,NULL);
  if (size==EBROK)
    return EBROK;
  //printf("spill stop found\n");
  //printf("data=");
  //print_data(data,size);
  free(data);
  seek_backward(ws,2);
  return 0;
}

//search for the start spill pattern in the data 
//manage the error pattern
//return 0 if success
//return -1 if failure
int search_next_spill_start(struct conv_ws *ws,struct skiroc_ws *sws) {
  char *data;
  int size;
  size=read_until_str(ws,&data,spill_header,NULL);
  if (size==EBROK)
    return EBROK;
  //printf("spill start found\n");
  //printf("data=");
  //print_data(data,size);
  free(data);
  seek_backward(ws,2);
  return 0;
}



//parse the data beam for finding the headers and go to the data
//return 0 if no new action is required
//return 1 if chip data is available
//return -1 if no data
int parse_headers(struct conv_ws *ws,struct skiroc_ws *sws) {
  char *str_head_code;
  unsigned short head_code;
  char *str_spill_nb;
  unsigned int spill_nb;
  char *str_chip_nb;
  int chip_nb;
  int res;
  struct block *b;
  
  //printf("parse_headers\n");

  //read the header code
  if (read_nb(ws,&str_head_code,2)==EBROK)
    return EBROK;
  head_code=*(unsigned short *)str_head_code;
  free(str_head_code);
  //printf("head_code=%x\n",head_code);
  
  if (head_code==start_spill_code) { //found a new spill
    
    //verify that the previous block had been finalized
    if (sws->block!=NULL) {
      printf("error : block has not been finalized\n");
      set_stats(ws->stats,"nb_spilltrail_error",SOP_PLUS,1.0); 
      submod_finalize_block(sws->block);
    }

    //read the spill number
    if (read_nb(ws,&str_spill_nb,4)==EBROK)
      return EBROK;
    
    //get the spill from data
    spill_nb=convert_difint(str_spill_nb);
    free(str_spill_nb);
    printf("new spill %x\n",spill_nb);

    if (sws->current_spill==-1)
      if (spill_nb!=1) {
	printf("trashing config spill\n");
	sws->current_spill=0;
	res=search_next_spill_start(ws,sws);
	if (res==EBROK)
	  return EBROK;
	forget_before_current(ws);
	return 0;
      }

    sws->current_spill=spill_nb;

    //set stats
    if (sws->ref) {
      set_stats(ws->stats,"nb_spills",SOP_PLUS,1.0);
      set_stats(ws->stats,"cur_spill",SOP_SET,spill_nb);
    }
    
    //create a data block
    res=submod_new_block(spill_nb,&b);
    if (res<0) {
      printf("warning : spill nb desynchronization : going to next spill start\n");
      res=search_next_spill_start(ws,sws);
      if (res==EBROK)
        return EBROK;
      forget_before_current(ws);
      return 0;
    }
    sws->block=b;

    
    submod_set_field_block(sws->block,sws->ese_spill,"%d",spill_nb);
    //submod_set_field_block(sws->block,sws->ese_dif,"%d",sws->dif);
    submod_set_field_block(sws->block,sws->ese_p,"%d",sws->plan);
    submod_set_field_block(sws->block,sws->ese_z,"%f",sws->plan*15+sws->offset_z);
    
    //read and discard the ascii code
    if (seek_forward(ws,6)==EBROK)
      return EBROK;
   
    return 0;
  }
  
  
  if (head_code==start_chip_code) { //found a new chip
      
    //read chip number
    if (read_nb(ws,&str_chip_nb,1)==EBROK)
      return EBROK;
    chip_nb=(*(char *)str_chip_nb);
    free(str_chip_nb);
    //printf("new chip %x\n",chip_nb);
    
    //update the number of corresponding stream
    sws->current_chip=chip_nb-1;
    
    //discard a useless byte
    if (seek_forward(ws,1)==EBROK)
      return EBROK;
    
    //discard the ascii code
    if (seek_forward(ws,6)==EBROK)
      return EBROK;

    return 1;
  }
    
  if (head_code==stop_spill_code) { //found a spill trailer
    
    //read the spill number
    if (read_nb(ws,&str_spill_nb,4)==EBROK)
      return EBROK;
    spill_nb=convert_difint(str_spill_nb);
    free(str_spill_nb);
    //printf("spill trailer %x\n",spill_nb);

    //finalize the data block corresponding to this spill
    submod_finalize_block(sws->block);
    sws->block=NULL;

    //allow converter to recycle previous data space
    forget_before_current(ws);
    //printf("spill trailer : searching next spill start\n");
    res=search_next_spill_start(ws,sws);
    if (res==EBROK)
      return EBROK;
    return 0;
  }

  if (head_code==sws->current_spill+1) {
    //printf("considering as a lost spill header\n");
    sws->current_spill++;
    //read and discard the ascii code
    if (seek_forward(ws,6)==EBROK)
      return EBROK;
   
    return 0;
  }
  
  printf("I'm lost... moving to next spill start\n");
  res=search_next_spill_start(ws,sws);
  if (res==EBROK)
    return EBROK;
  forget_before_current(ws);
  return 0;
}

int create_sca_events(struct conv_ws *ws,struct skiroc_ws *sws,int *gain1_energy,int *gain2_energy,int *hits,int bcid,int sca,int tag,int *error) {
  int chan;
  int res;
  double spill=(double)sws->current_spill;
  int roc,col,row;
  float x,y;
  struct event *e;

  if (sws->block) {
    for(chan=0;chan<64;chan++) {
      if (sws->cut_wrong_events && (tag!=0 || error[chan]!=0))
        continue;
      if (hits[chan]==1 || (!sws->cut_non_hit)) {
        res=submod_new_event(sws->block,&e);
        if (res<0) {
          printf("cant create event\n");
          return 0;
        }
        //strdup("spill,dif,z!bcid,time|roc,chan,c,l,x,y|hit,en1,en2,sca,tag,error")
        submod_set_field_event(e,sws->ese_bcid,"%d",bcid);
        submod_set_field_event(e,sws->ese_time,"%.10lf",spill*(double)sws->spill_period+(double)bcid*(double)sws->bx_period);
        roc=sws->current_chip;
        submod_set_field_event(e,sws->ese_roc,"%d",roc);
        submod_set_field_event(e,sws->ese_chan,"%d",chan);
        col=sws->mapping[roc][chan].c;
        row=sws->mapping[roc][chan].l;
	x=sws->mapping[roc][chan].x+sws->offset_x;
	y=sws->mapping[roc][chan].y+sws->offset_y;
        submod_set_field_event(e,sws->ese_c,"%d",col);
        submod_set_field_event(e,sws->ese_l,"%d",row);
        submod_set_field_event(e,sws->ese_x,"%f",x);
        submod_set_field_event(e,sws->ese_y,"%f",y);
        submod_set_field_event(e,sws->ese_hit,"%d",hits[chan]);
        submod_set_field_event(e,sws->ese_en,"%d",gain1_energy[chan]);
        submod_set_field_event(e,sws->ese_en2,"%d",gain2_energy[chan]);
        submod_set_field_event(e,sws->ese_tag,"%d",tag);
        submod_set_field_event(e,sws->ese_sca,"%d",sca);
        submod_set_field_event(e,sws->ese_error,"%d",error[chan]);
      }
    }
  } else {
    printf("error : events are available but no block to handle them\n");
  }
  return 0;
}


int parse_sca(struct conv_ws *ws,struct skiroc_ws *sws,unsigned short *sdata,int bcid,int sca,int nbcol,int ptag) {
  int gain1_energy[64];
  int gain1_hit[64];
  int gain2_energy[64];
  int gain2_hit[64];
  int hits[64];
  int errors[64];
  int tag=ptag;
  int fb1;
  int fb2;
  int nb_hits=0;
  unsigned short ddata;
  int chan;

  
  for(chan=0;chan<64;chan++) {

    errors[chan]=0;
    
    //read high gain datas (en1) the one we want
    ddata=sdata[-2-nbcol-sca*128-64-chan]; //TODO
    gain1_energy[chan]=ddata&0x0fff;
    gain1_hit[chan]=(ddata&0x1000)>>12;
    fb1=ddata&0x4000;
    fb2=ddata&0x8000;
    if (fb1!=0) {
      set_stats(ws->stats,"nb_frontbit_errors",SOP_PLUS,1.0);
      errors[chan]|=frontbit_error;
    }
    if (fb2!=0) {
      set_stats(ws->stats,"nb_frontbit_errors",SOP_PLUS,1.0); 
      errors[chan]|=frontbit_error;
    }
    //printf("chan %d : hit=%d gain 1 energy=%d\n",chan,gain1_hit[chan],gain1_energy[chan]);
    
    //read low gain datas (en2)
    ddata=sdata[-2-nbcol-sca*128-chan]; //TODO
    gain2_energy[chan]=ddata&0x0fff;
    gain2_hit[chan]=(ddata&0x1000)>>12;
    fb1=ddata&0x4000;
    fb2=ddata&0x8000;
    if (fb1!=0) {
      set_stats(ws->stats,"nb_frontbit_errors",SOP_PLUS,1.0); 
      errors[chan]|=frontbit_error;
    }
    if (fb2!=0) {
      set_stats(ws->stats,"nb_frontbit_errors",SOP_PLUS,1.0); 
      errors[chan]|=frontbit_error;
    }
    //printf("chan %d : hit=%d gain 2 energy=%d\n",chan,gain2_hit[chan],gain2_energy[chan]);
    
    if (gain1_energy[chan]==4) {
      tag|=underflow_tag;
    }

    if (gain1_energy[chan]==0) {
      tag|=overflow_tag;
    }

    
    //count the hits and hitbit errors
    if (gain1_hit[chan]|gain2_hit[chan]) { //hit
      hits[chan]=1;
      nb_hits++;
      //hit and hit error stats
      if (gain1_hit[chan]!=gain2_hit[chan]) {
	set_stats(ws->stats,"nb_hitbit_errors",SOP_PLUS,1.0);
	errors[chan]|=hitbit_error;
      }
    } else {
      hits[chan]=0;
    }
  }

  //detect plane events
  if (nb_hits>sws->plane_thresh) {
    //printf("plane events : %d hits (thresh=%d)...skipping sca\n",nb_hits,sws->plane_thresh);
    set_stats(ws->stats,"nb_plane_events",SOP_PLUS,1.0);
    tag|=plane_tag;
  }

  //fill hits stats
  set_stats(ws->stats,"nb_hits",SOP_PLUS,(float)nb_hits);

  //build the events
  create_sca_events(ws,sws,(int *)gain1_energy,(int *)gain2_energy,(int *)hits,bcid,sca,tag,errors);

  //everythings fine - returning ok
  return 0;
}


//read the data for a chip and output it
//return -1 in case of read error
//return 0 in other cases
int parse_chip(struct conv_ws *ws,struct skiroc_ws *sws) {
  int sca;
  char *data;
  int nbcol;
  int nb_words;
  int bc;
  int i;
  int prev_bcid;
  int bcid_base=0;

  unsigned short *sdata;
  int rsize;
  int res;

  //searching a chip trailer
  //printf("searching for chip trailer, avoiding spill header\n");
  rsize=read_until_str(ws,&data,chip_trailer,spill_header);
  if (rsize==EBROK)
    return EBROK;
  if (rsize==EAVOID) {
    printf("warning : spill start detected while parsing chip data...moving to next spill start\n");
    res=search_next_spill_start(ws,sws);
    if (res==EBROK)
      return EBROK;
    forget_before_current(ws);
    return 0;
  }

  //searching for an error in the data
  for(i=0;i<rsize-1;i++) {
    if (*((unsigned short *)(data+i))==loss_code) {
      printf("warning : loss detected at position %d in the data...moving to next spill start\n",i);
      free(data);
      res=search_next_spill_start(ws,sws);
      if (res==EBROK)
        return EBROK;
      forget_before_current(ws);
      return 0;
    }
  }

  //computing the size of data
  nb_words=rsize/2;
  
  nbcol=(nb_words-2)/129;
  //printf("data packet of size %d bytes %d words %d columns \n",rsize,nb_words,nbcol);

  if ((nb_words-2)!=nbcol*129) {
    printf("size is not good : expected size=%d real size=%d\n",nbcol*129+2,nb_words-2);
  }

  //checking size compatibility
  if (nbcol>(NB_SCA)) {
    printf("error : number of columns is too big...skipping spill\n");
    set_stats(ws->stats,"nb_sca_errors",SOP_PLUS,1.0); 
    free(data);
    res=search_next_spill_start(ws,sws);
    if (res==EBROK)
      return EBROK;
    forget_before_current(ws);
    return 0;
  }

  //convert data string to shorts
  sdata=(unsigned short *)data+nb_words-1;

  if (nbcol*129+2==nb_words-1) {
    //printf("duplicate chip id\n");
    sdata--;
  }

  //reading chip id
  sws->current_chip=sdata[-1];
  //printf("chip id : %d\n",sws->current_chip);
  if (sws->current_chip>sws->max_roc) {
    printf("error : chip id %d is too big (max=%d)...moving to next spill start\n",sws->current_chip,sws->max_roc);
    set_stats(ws->stats,"nb_chipid_error",SOP_PLUS,1.0);
    free(data);
    res=search_next_spill_start(ws,sws);
    if (res==EBROK)
      return EBROK;
    forget_before_current(ws);
    return 0;
  }

  //initialize retriggering detector
  prev_bcid=-1000;
	
  //reading data
  for (sca=0;sca<nbcol;sca++) {

    //get the bunch crossing of the data
    bc=sdata[-2-sca]+bcid_base;
    //printf("bcid=%d\n",bc);

    //correct the cyclic bcid
    if (bc<prev_bcid) {
      bcid_base+=4096;
      bc+=4096;
    }

    //detect bcid + n retriggering
    if ((bc-prev_bcid)<sws->bxpn_thresh) {
      parse_sca(ws,sws,sdata,bc,sca,nbcol,bxpn_tag);
    } else {
      parse_sca(ws,sws,sdata,bc,sca,nbcol,0);
    }

    prev_bcid=bc;
  }

  //free the data
  free(data);
  
  //trash the chip trailer 6 bytes
  if (seek_forward(ws,6)==EBROK)
    return EBROK;
  
  //allow converter to recycle previous data space
  forget_before_current(ws);
  
  return 0;
}
      
void finalize(struct conv_ws *ws,struct skiroc_ws *sws) {
  printf("no more data...finalizing\n");
  while(1) {
    usleep(1000000);
    printf("decoder finalized\n");
  }
}

int parse_args(struct conv_ws *ws,struct skiroc_ws *sws) {
  if (ws->argc!=13) {
    printf("skiroc decoder needs 13 parameters (%d provided) : ref,cut_non_hit,bxpn_thresh,plane_thresh,cut_wrong_events,spill_period,bx_period,mapping_filename,plan,offset_x,offset_y,offset_z,max_roc\n",ws->argc);
    return -1;
  }

  //ref param
  sws->ref=atoi(ws->argv[0]);

  //cut non hit
  sws->cut_non_hit=atoi(ws->argv[1]);
  
  //cut parameters
  sws->bxpn_thresh=atoi(ws->argv[2]);
  sws->plane_thresh=atoi(ws->argv[3]);
  
  sws->cut_wrong_events=atoi(ws->argv[4]);
  
  //time conversion parameters
  sws->spill_period=atof(ws->argv[5]);
  sws->bx_period=atof(ws->argv[6]);

  //coordinate conversion parameters
  sws->mapping_filename=ws->argv[7];
  sws->mapping=load_mapping(sws->mapping_filename,16,64);
  if (sws->mapping==NULL)
    return -1;
  sws->plan=atoi(ws->argv[8]);
  sws->offset_x=atof(ws->argv[9]);
  sws->offset_y=atof(ws->argv[10]);
  sws->offset_z=atof(ws->argv[11]); 

  //max roc ic
  sws->max_roc=atoi(ws->argv[12]);
  
  return 0;
}

void init_plugin(void *vdata) {
  //global workspace herited from converter/dispatcher
  struct conv_ws *ws=vdata;
  
  //create local workspace
  struct skiroc_ws *sws=malloc(sizeof(struct skiroc_ws));
  memset(sws,0,sizeof(struct skiroc_ws));
  //sws->dif=ws->streamid;
  sws->spill_offset=-1;
  sws->spill_cumul=0;
  sws->current_spill=-1;
  sws->block=NULL;
  ws->sws=sws;

  if (parse_args(ws,sws)<0) {
    //finalize(ws,sws);
    sleep(1);
    exit(1);
  }

  //ws->schema_str=strdup("spill,dif,p,z!bcid,time|roc,chan,c,l,x,y|hit,en,en2");
  ws->schema_str=strdup("spill,p,z!bcid,time|roc,chan,c,l,x,y|hit,en,en2,sca,tag,error");
  ws->schema=saf_parse_chans(ws->schema_str);
  sws->ese_spill=get_ese(ws->schema,"spill");
  sws->ese_p=get_ese(ws->schema,"p");
  sws->ese_z=get_ese(ws->schema,"z");
  sws->ese_bcid=get_ese(ws->schema,"bcid");
  sws->ese_time=get_ese(ws->schema,"time");
  sws->ese_roc=get_ese(ws->schema,"roc");
  sws->ese_chan=get_ese(ws->schema,"chan");
  sws->ese_c=get_ese(ws->schema,"c");
  sws->ese_l=get_ese(ws->schema,"l");
  sws->ese_x=get_ese(ws->schema,"x");
  sws->ese_y=get_ese(ws->schema,"y");
  sws->ese_hit=get_ese(ws->schema,"hit");
  sws->ese_en=get_ese(ws->schema,"en");
  sws->ese_en2=get_ese(ws->schema,"en2");
  sws->ese_tag=get_ese(ws->schema,"tag");
  sws->ese_sca=get_ese(ws->schema,"sca");
  sws->ese_error=get_ese(ws->schema,"error");

  //initialize the stats system
  add_acc(ws->stats,"nb_hits",0);
  add_acc(ws->stats,"nb_spills",0);
  add_inst(ws->stats,"cur_spill",0);
  add_acc(ws->stats,"nb_plane_events",0);

  add_acc(ws->stats,"nb_spilltrail_error",0);
  add_acc(ws->stats,"nb_hitbit_errors",0);
  add_acc(ws->stats,"nb_frontbit_errors",0);
  add_acc(ws->stats,"nb_chipid_errors",0);
  add_acc(ws->stats,"nb_sca_errors",0);

  if (sws->ref) {
    set_stats(ws->stats,"nb_spills",SOP_SET,0);
    set_stats(ws->stats,"cur_spill",SOP_SET,0);
  }
  
  //initialize the data stream
  init_ribbon(ws,8192*50);

}

void reinit(void *vdata) {
  struct conv_ws *ws=vdata;
  struct skiroc_ws *sws=(struct skiroc_ws *)ws->sws;
  sws->spill_offset=-1;
  sws->spill_cumul=0;
  sws->current_spill=-1;
  sws->block=NULL;
}

void convert(void *vdata) {
  struct conv_ws *ws=vdata;
  struct skiroc_ws *sws=(struct skiroc_ws *)ws->sws;
  int res;
  pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS,NULL);
  ws->ready=1;
  while(1) {
    res=parse_headers(ws,sws);
    if (res==EBROK) {
      finalize(ws,sws);
      return;
    }
    if (res==1) {
      res=parse_chip(ws,sws);
      if (res==EBROK) {
        finalize(ws,sws);
        return;
      }
    }
  }
}
