#include "spiroc2d_decoder.h"

struct map_cell **load_mapping(char *filename,int nb_chips,int nb_chans,int dif) {
  
  struct map_cell **mapping;
  FILE *f;
  int i;
  int chip,chan;
  int plane,channel;
  float x,y,z;
  int res;
  int dif_id;

  //allocate mapping
  mapping=malloc(sizeof(struct map_cell *)*nb_chips);
  for (i=0;i<nb_chips;i++)
    mapping[i]=malloc(sizeof(struct map_cell)*nb_chans);

  //open mapping file
  f=fopen(filename,"r");
  if (f==NULL) {
    printf("cant open file %s\n",filename);
    return NULL;
  }
  
  //reading mapping
  while(!feof(f)) {
    res=fscanf(f,"%d %d %d %f %f %f %d %d",&dif_id,&chip,&chan,&x,&y,&z,&plane,&channel);
    if ((res!=8)&&(res!=-1)) {
      printf("syntax error in file %s obtained %d items instead of 8\n",filename,res);
    } else {
      printf("dif_id=%d chip=%d chan=%d x=%f y=%f z=%f pln=%d ch=%d\n",dif_id,chip,chan,x,y,z,plane,channel); 
      }
    if (dif==dif_id) {
      mapping[chip][chan].x=x;
      mapping[chip][chan].y=y;
      mapping[chip][chan].z=z;
      mapping[chip][chan].plane=plane;
      mapping[chip][chan].channel=channel;
    }
  }
  fclose(f);
  return mapping;
}

void free_mapping(struct map_cell **mapping,int nb_chips) {
  int i;
  for (i=0;i<nb_chips;i++)
    free(mapping[i]);
  free(mapping);
}

void print_data(char *str,int size) {
  int i;
  for (i=0;i<size;i++) {
    printf("%02x",(unsigned char)str[i]);
  }
  printf("\n");
}

int convert_difint(char *intstr) {
  unsigned short integ[2];
  integ[0]=*((unsigned short *)(intstr+2));
  integ[1]=*((unsigned short *)(intstr));
  return *((int *)integ);
}

//search for the stop spill pattern in the data 
//manage the error pattern
//return 0 if success
//return -1 if failure
int search_next_spill_stop(struct conv_ws *ws,struct spiroc_ws *sws) {
  int size;
  size=forward_until_str(ws,stop_spill_str,NULL);
  if (size==EBROK)
    return EBROK;
  //printf("spill stop found\n");
  //printf("data=");
  //print_data(data,size);
  return 0;
}

//search for the start spill pattern in the data 
//manage the error pattern
//return 0 if success
//return -1 if failure
int search_next_spill_start(struct conv_ws *ws,struct spiroc_ws *sws) {
  char *data;
  int size;

  //searching for a spill start or a spill number tag
  size=forward_until_str(ws,start_spill_str,spill_nb_str);
  if (size==EBROK)
    return EBROK;

  if (size==EAVOID) { //we found a spill number tag
    printf("found a spill tag\n");
    //go to the tag
    size=forward_until_str(ws,spill_nb_str,NULL);
    if (size==EBROK)
      return EBROK;
    
    //remove the tag
    size=seek_forward(ws,2);
    if (size==EBROK)
      return EBROK;
   
    //get the spill number
    size=read_nb(ws,&data,2);
    if (size==EBROK)
      return EBROK;
    sws->current_spill=*(unsigned short *)data;
    printf("got spill %d from spill tag\n",sws->current_spill);
    free(data);

    //get the spill flag
    size=read_nb(ws,&data,2);
    if (size==EBROK)
      return EBROK;
    sws->current_spill_flag=*(unsigned short *)data>>15;
    printf("got spill flag %x from spill tag\n",sws->current_spill_flag);
    free(data);
  }
  return 0;
}



//parse the data beam for finding the headers and go to the data
//return 0 if no new action is required
//return 1 if chip data is available
//return -1 if no data
int parse_headers(struct conv_ws *ws,struct spiroc_ws *sws) {
  char *str_head_code;
  unsigned short head_code;
  char *str_chip_nb;
  int chip_nb;
  int res;
  struct block *b;
  
  printf("parse_headers\n");
  
  //read the header code
  if (read_nb(ws,&str_head_code,2)==EBROK)
    return EBROK;
  head_code=*(unsigned short *)str_head_code;
  free(str_head_code);
  //printf("head_code=%x\n",head_code);
  
  if (head_code==spill_nb_code) {
    printf("warning : spill code reached. Should never happen\n");
    res=search_next_spill_start(ws,sws);
    if (res==EBROK)
      return EBROK;
    return 0;
  }


  if (head_code==start_spill_code) { //found a new spill

    if (ws->debug)
      printf("new spill tag\n");

    if (sws->current_spill==-1) {
      printf("warning : spill nb not yet received : going to next spill start\n");
      res=search_next_spill_start(ws,sws);
      if (res==EBROK)
        return EBROK;
      forget_before_current(ws);
      return 0;
    }
    
    //verify that the previous block had been finalized
    if (sws->block!=NULL) {
      printf("error : block has not been finalized...");
      set_stats(ws->stats,"nb_spilltrail_error",SOP_PLUS,1.0); 
      submod_finalize_block(sws->block);
      printf("now it is\n");
    }
    
    //set stats
    if (sws->ref) {
      set_stats(ws->stats,"nb_spills",SOP_PLUS,1.0);
      set_stats(ws->stats,"cur_spill",SOP_SET,sws->current_spill);
    }
    
    
    //TODO : find a way to flush dispatcher and go on
    //create a data block
    res=submod_new_block(sws->block_cnt,&b);
    sws->block_cnt++;
    sws->block=b;
    
    
    //set the properties of the block
    submod_set_field_block(sws->block,sws->ese_spill,"%d",sws->current_spill);
    submod_set_field_block(sws->block,sws->ese_spill_flag,"%d",sws->current_spill_flag);
    
    //read and discard the ascii code
    if (seek_forward(ws,10)==EBROK)
      return EBROK;
    
    return 0;
  }
  
  
  if (head_code==start_chip_code) { //found a new chip

    //read chip number
    if (read_nb(ws,&str_chip_nb,1)==EBROK)
      return EBROK;
    chip_nb=(*(char *)str_chip_nb);
    free(str_chip_nb);
    
    if (ws->debug)
      printf("new chip %d (0x%x)\n",chip_nb,chip_nb);
   
    //discard a useless byte
    if (seek_forward(ws,1)==EBROK)
      return EBROK;
    
    //discard the ascii code
    if (seek_forward(ws,6)==EBROK)
      return EBROK;
    
    return 1;
  }
  
  if (head_code==stop_spill_code) { //found a spill trailer
    
    if (ws->debug)
      printf("stop spill found\n");

    //finalize the data block corresponding to this spill
    submod_finalize_block(sws->block);
    sws->block=NULL;
    
    //allow converter to recycle previous data space
    forget_before_current(ws);
    //printf("spill trailer : searching next spill start\n");
    res=search_next_spill_start(ws,sws);
    if (res==EBROK)
      return EBROK;
    return 0;
  }
  
  
  printf("I'm lost (head code=%x)... moving to next spill start\n",head_code);
  res=search_next_spill_start(ws,sws);
  if (res==EBROK)
    return EBROK;
  forget_before_current(ws);
  return 0;
}

int create_sca_events(struct conv_ws *ws,struct spiroc_ws *sws,int *energy,int *time,int *hits,int *gain,char *cut,int bcid,int sca) {
  int rocchan;
  int res;
  int roc,plane,channel;
  float x,y,z;
  struct event *e;

  if (sws->block) {
    for(rocchan=0;rocchan<32;rocchan++) {
      if (cut[rocchan])
          continue;
      if (hits[rocchan]==1 || (!sws->cnh)) {
        res=submod_new_event(sws->block,&e);
        if (res<0) {
          printf("cant create event\n");
          return 0;
        }
	//"spill,spill_flag!bcid,sca|roc,rocchan,plane,channel,x,y,z|hit,en,gain,time"
        submod_set_field_event(e,sws->ese_bcid,"%d",bcid);
        submod_set_field_event(e,sws->ese_sca,"%d",sca);
        roc=sws->current_chip;
        submod_set_field_event(e,sws->ese_roc,"%d",roc);
        submod_set_field_event(e,sws->ese_rocchan,"%d",rocchan);
        plane=sws->mapping[roc][rocchan].plane;
        channel=sws->mapping[roc][rocchan].channel;
	if (plane>7||channel>79)
	  printf("error : plane or channel is too big : roc=%d rocchan=%d plane=%d channel=%d\n",roc,rocchan,plane,channel);
	x=sws->mapping[roc][rocchan].x+sws->offset_x;
	y=sws->mapping[roc][rocchan].y+sws->offset_y;
	z=sws->mapping[roc][rocchan].z;
        submod_set_field_event(e,sws->ese_plane,"%d",plane);
        submod_set_field_event(e,sws->ese_channel,"%d",channel);
        submod_set_field_event(e,sws->ese_x,"%f",x);
        submod_set_field_event(e,sws->ese_y,"%f",y);
	submod_set_field_event(e,sws->ese_z,"%f",z);
        submod_set_field_event(e,sws->ese_hit,"%d",hits[rocchan]);
        submod_set_field_event(e,sws->ese_en,"%d",energy[rocchan]);
        submod_set_field_event(e,sws->ese_gain,"%d",gain[rocchan]);
        submod_set_field_event(e,sws->ese_time,"%d",time[rocchan]);
      }
    }
  } else {
    printf("error : events are available but no block to handle them\n");
  }
  return 0;
}


int parse_sca(struct conv_ws *ws,struct spiroc_ws *sws,unsigned short *sdata,int bcid,int sca,int nbcol) {
  int energy[CHAN_NB];
  int hit[CHAN_NB];
  int gain[CHAN_NB];
  int time[CHAN_NB];
  int time_hit[CHAN_NB];
  int hits[CHAN_NB];
  char cut[CHAN_NB];
  int fb1;
  int fb2;
  int nb_hits=0;
  unsigned short ddata;
  int chan;

  
  for(chan=0;chan<CHAN_NB;chan++) {
    cut[chan]=0;
    
    //read high gain datas (en1) the one we want
    ddata=sdata[-2-nbcol-sca*(2*CHAN_NB)-CHAN_NB-chan]; //TODO
    time[chan]=ddata&0x0fff;
    time_hit[chan]=(ddata&0x1000)>>12;
    fb1=ddata&0x4000;
    fb2=ddata&0x8000;
    if (fb1!=0) {
      set_stats(ws->stats,"nb_frontbit_errors",SOP_PLUS,1.0); 
    }
    if (fb2!=0) {
      set_stats(ws->stats,"nb_frontbit_errors",SOP_PLUS,1.0); 
    }
    //printf("chan %d : hit=%d gain 1 energy=%d\n",chan,gain1_hit[chan],gain1_energy[chan]);
    
    //read low gain datas (en2)
    ddata=sdata[-2-nbcol-sca*(2*CHAN_NB)-chan]; //TODO
    energy[chan]=ddata&0x0fff;
    hit[chan]=(ddata&0x1000)>>12;
    gain[chan]=(ddata&0x2000)>>13;
    fb1=ddata&0x4000;
    fb2=ddata&0x8000;
    if (fb1!=0) {
      set_stats(ws->stats,"nb_frontbit_errors",SOP_PLUS,1.0); 
    }
    if (fb2!=0) {
      set_stats(ws->stats,"nb_frontbit_errors",SOP_PLUS,1.0); 
    }
    //printf("chan %d : hit=%d gain 2 energy=%d\n",chan,gain2_hit[chan],gain2_energy[chan]);
    
    //check that energy for gain1 is higher than energy min
    if (energy[chan]<sws->min_energy) {
      //printf("gain1 energy %d lower than energy min %d...skipping channel\n",gain1_energy[chan],sws->min_energy);
      cut[chan]=1;
      continue;
    }

    
    //count the hits and hitbit errors
    if (hit[chan]|time_hit[chan]) { //hit
      hits[chan]=1;
      nb_hits++;
      //hit and hit error stats
      if (hit[chan]!=time_hit[chan]) {
	set_stats(ws->stats,"nb_hitbit_errors",SOP_PLUS,1.0);
      }
    } else {
      hits[chan]=0;
    }
  }

  //fill hits stats
  set_stats(ws->stats,"nb_hits",SOP_PLUS,(float)nb_hits);

  //every cut are applied so build the events
  create_sca_events(ws,sws,(int *)energy,(int *)time,(int *)hits,(int *)gain,(char *)cut,bcid,sca);

  //everythings fine - returning ok
  return 0;
}


//read the data for a chip and output it
//return -1 in case of read error
//return 0 in other cases
int parse_chip(struct conv_ws *ws,struct spiroc_ws *sws) {
  int sca;
  char *data;
  int nbcol;
  int nb_words;
  int bc;
  int i;
  int prev_bcid;
  int exp_size;

  unsigned short *sdata;
  int rsize;
  int res;

  if (ws->debug)
    printf("parse chips\n");

  //searching a chip trailer
  rsize=read_until_str(ws,&data,stop_chip_str,start_spill_str);
  if (rsize==EBROK)
    return EBROK;
  if (rsize==EAVOID) {
    printf("warning : spill start detected while parsing chip data...moving to next spill start\n");
    res=search_next_spill_start(ws,sws);
    if (res==EBROK)
      return EBROK;
    forget_before_current(ws);
    return 0;
  }

  //searching for an error in the data
  for(i=0;i<rsize-1;i++) {
    if (*((unsigned short *)(data+i))==loss_code) {
      printf("warning : loss detected at position %d in the data...moving to next spill start\n",i);
      free(data);
      res=search_next_spill_start(ws,sws);
      if (res==EBROK)
        return EBROK;
      forget_before_current(ws);
      return 0;
    }
  }

  //computing the size of data
  nb_words=rsize/2;
  nbcol=(nb_words-2)/73; //integer division
  //expected size for firmware compiled with xilinx
  exp_size=nbcol*73+2;
  

  if (nb_words!=exp_size) {
    printf("warning : size is not good : expected size=%d real size=%d\n",exp_size,nb_words);
  }
  if (ws->debug)
    printf("reading %d sca\n",nbcol);

  //checking size compatibility
  if (nbcol>(NB_SCA)) {
    printf("error : number of columns is too big...skipping spill\n");
    set_stats(ws->stats,"nb_sca_errors",SOP_PLUS,1.0); 
    free(data);
    res=search_next_spill_start(ws,sws);
    if (res==EBROK)
      return EBROK;
    forget_before_current(ws);
    return 0;
  }

  //convert data string to shorts
  sdata=(unsigned short *)data+nb_words-1;

  if (nbcol*73+2==nb_words-1) {
    //printf("duplicate chip id\n");
    sdata--;
  }

  //reading chip id
  sws->current_chip=sdata[-1];
  //printf("chip id : %d\n",sws->current_chip);
  if (sws->current_chip>sws->max_roc) {
    printf("error : chip id %d is too big (max=%d)...moving to next spill start\n",sws->current_chip,sws->max_roc);
    set_stats(ws->stats,"nb_chipid_error",SOP_PLUS,1.0);
    free(data);
    res=search_next_spill_start(ws,sws);
    if (res==EBROK)
      return EBROK;
    forget_before_current(ws);
    return 0;
  }

  //initialize retriggering detector
  prev_bcid=-1000;
	
  //reading data
  for (sca=0;sca<nbcol;sca++) {

    //get the bunch crossing of the data
    bc=sdata[-2-sca]; 

    //correct the cyclic bcid
    if (bc<prev_bcid) {
      bc+=4096;
    }
    
    parse_sca(ws,sws,sdata,bc,sca,nbcol);
  }

  //free the data
  free(data);
  
  //trash the chip trailer 6 bytes
  if (seek_forward(ws,6)==EBROK)
    return EBROK;
  
  //allow converter to recycle previous data space
  forget_before_current(ws);
  
  return 0;
}
      
void finalize(struct conv_ws *ws,struct spiroc_ws *sws) {
  printf("no more data...finalizing\n");
  while(1) {
    usleep(1000000);
    printf("decoder finalized\n");
  }
}

int parse_args(struct conv_ws *ws,struct spiroc_ws *sws) {
  if (ws->argc!=8) {
    printf("spiroc decoder needs 8 parameters (%d provided) : dif_id,ref,cut_non_hit,min_energy,mapping_filename,offset_x,offset_y,max_roc\n",ws->argc);
    return -1;
  }

  //dif id
  sws->dif=atoi(ws->argv[0]);

  //ref param
  sws->ref=atoi(ws->argv[1]);

  //cut non hit
  sws->cnh=atoi(ws->argv[2]);
  
  //cut parameters
  sws->min_energy=atoi(ws->argv[3]);
  
  //coordinate conversion parameters
  sws->mapping_filename=ws->argv[4];
  
  sws->offset_x=atof(ws->argv[5]);
  sws->offset_y=atof(ws->argv[6]);

  //max roc ic
  sws->max_roc=atoi(ws->argv[7]);

  sws->mapping=load_mapping(sws->mapping_filename,sws->max_roc,CHAN_NB,sws->dif);
  if (sws->mapping==NULL)
    return -1;
  
  return 0;
}

void reinit(void *vdata) {
  struct conv_ws *ws=vdata;
  struct spiroc_ws *sws=(struct spiroc_ws *)ws->sws;
  sws->spill_offset=-1;
  sws->spill_cumul=0;
  sws->current_spill=-1;
  sws->block=NULL;
  sws->block_cnt=0;
}

void init_plugin(void *vdata) {
  //global workspace herited from converter/dispatcher
  struct conv_ws *ws=vdata;
  
  //create local workspace and bind it in the global workspace
  struct spiroc_ws *sws=malloc(sizeof(struct spiroc_ws));
  memset(sws,0,sizeof(struct spiroc_ws));
  ws->sws=sws;

  //initialize local workspace 
  reinit(ws);
 
  //parse the arguments
  if (parse_args(ws,sws)<0) {
    //finalize(ws,sws);
    sleep(1);
    exit(1);
  }

  //initialize schema and field pointers
  ws->schema_str=strdup("spill,spill_flag!bcid,sca|roc,rocchan,plane,channel,x,y,z|hit,en,gain,time");
  ws->schema=saf_parse_chans(ws->schema_str);
  sws->ese_spill=get_ese(ws->schema,"spill");
  sws->ese_spill_flag=get_ese(ws->schema,"spill_flag");
  sws->ese_bcid=get_ese(ws->schema,"bcid");
  sws->ese_sca=get_ese(ws->schema,"sca");
  sws->ese_time=get_ese(ws->schema,"time");
  sws->ese_roc=get_ese(ws->schema,"roc");
  sws->ese_rocchan=get_ese(ws->schema,"rocchan");
  sws->ese_plane=get_ese(ws->schema,"plane");
  sws->ese_channel=get_ese(ws->schema,"channel");
  sws->ese_x=get_ese(ws->schema,"x");
  sws->ese_y=get_ese(ws->schema,"y");
  sws->ese_z=get_ese(ws->schema,"z");
  sws->ese_hit=get_ese(ws->schema,"hit");
  sws->ese_en=get_ese(ws->schema,"en");
  sws->ese_gain=get_ese(ws->schema,"gain");

  //initialize the stats system
  add_acc(ws->stats,"nb_hits",0);
  add_acc(ws->stats,"nb_spills",0);
  add_inst(ws->stats,"cur_spill",0);

  add_acc(ws->stats,"nb_spilltrail_error",0);
  add_acc(ws->stats,"nb_hitbit_errors",0);
  add_acc(ws->stats,"nb_frontbit_errors",0);
  add_acc(ws->stats,"nb_chipid_errors",0);
  add_acc(ws->stats,"nb_sca_errors",0);

  if (sws->ref) {
    set_stats(ws->stats,"nb_spills",SOP_SET,0);
    set_stats(ws->stats,"cur_spill",SOP_SET,0);
  }
  
  //initialize the data stream
  init_ribbon(ws,8192*50);

}

void convert(void *vdata) {
  struct conv_ws *ws=vdata;
  struct spiroc_ws *sws=(struct spiroc_ws *)ws->sws;
  int res;
  pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS,NULL);
  ws->ready=1;
  ws->debug=1;
  while(1) {
    res=parse_headers(ws,sws);
    if (res==EBROK) {
      finalize(ws,sws);
      return;
    }
    if (res==1) {
      res=parse_chip(ws,sws);
      if (res==EBROK) {
        finalize(ws,sws);
        return;
      }
    }
  }
}
