/*
 * Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
 * This file is part of Pyrame.
 *
 * Pyrame is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Pyrame is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
 * */

//System
#include <stdio.h>
#include <pthread.h>

//STL
#include <map>
#include <algorithm>

//Pyrame C++
#include <cmdmod_helper.hpp>

//Root
#include <TGraph.h>
#include <TGraph2D.h>
#include <TH1F.h>
#include <TCanvas.h>
#include <TString.h>
#include <TApplication.h>
#include <TThread.h>
#include <THttpServer.h>

//Root graphics
#include <TGFrame.h>
#include <TGButton.h>
#include <TGLabel.h>
#include <TText.h>
#include <TRootEmbeddedCanvas.h>
#include <TQObject.h>
#include <RQ_OBJECT.h>

enum graph_type {HISTO,GRAPH,D3,LABEL};
enum logical_op {INVALID,EQUAL,NEQUAL,LT,GT,LTE,GTE};

// Filter CLASS

class Filter {
  char *chan;
  enum logical_op op;
  float value;
  public:
  Filter(char *filter_s);
  ~Filter();
  short valid();
  void print();
  short checkEvent(struct event *e);
};

class DataPlotter;

// DPGraph CLASS

struct DPGraph {
  DataPlotter *dp_ws;
  char *title;
  int title_len;
  char *chanx;
  char *chany;
  char *chanz;
  struct evloop *loop;
  short first;
  float xmin,xmax,ymin,ymax,zmin,zmax;
  std::vector<Filter*> filters;
  TGraph *graph;
  TGraph2D *graph2d;
  TH1F *histo;
  TText *label;
  int pad;
  int id;
  enum graph_type gtype;
  void newblock(struct block *b);
  void newevt(struct event *e);
  DPGraph(DataPlotter *dp_ws,char *ds_name,int id,char *title,char *chanx,char *chany,char *chanz,char *type,char *color);
  ~DPGraph();
};

struct updater_ws {
  TCanvas *canvas;
  pthread_mutex_t *updater_mutex;
};

// DataPlotter CLASS

class DataPlotter {
  RQ_OBJECT("DataPlotter")
  TGMainFrame *fMain;
  TRootEmbeddedCanvas *fEcanvas;
  TApplication *app;
  THttpServer *serv;
  pthread_t *app_t;
  TThread *updater_t;
  std::map<int,DPGraph*> plots;
  int nb_plots;
  void remap_pads(void);
  void clrscr(DPGraph*);
  void clrscrall(void);
  public:
  pthread_mutex_t *updater_mutex;
  TCanvas *canvas;
  DataPlotter();
  int new_plot(char *ds_name,char *title,char *chanx,char *chany,char *chanz,char *type,char *color);
  int del_plot(char *id_s);
  int add_filter(char *id_s,char *filter);
  int del_filters(char *id_s);
  void rewind(void);
  void stop(void);
  void run(void);
  void clean(void);
  char *list_plots(void);
  void onestep(void);
  int change_policy(char *newpol);
};

void new_block(void *plot,struct block *b);
void new_evt(void *plot,struct event *e);
void end_block(void *plot,struct block *b);
void reinit(void *plot);
