/*
Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
This file is part of Pyrame.

Pyrame is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Pyrame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
*/

#include "cmd_dp.hpp"

Filter::Filter(char *filter_s) {
  unsigned int i;
  unsigned int filter_len=strlen(filter_s);
  chan=NULL;
  op=INVALID;
  value=0;
  for (i=0;i<filter_len-1;i++) {
    if (filter_s[i]=='=' && filter_s[i+1]=='=') {
      op=EQUAL;
      filter_s[i]=0;
      i+=2;
      break;
    }
    if (filter_s[i]=='!' && filter_s[i+1]=='=') {
      op=NEQUAL;
      filter_s[i]=0;
      i+=2;
      break;
    }
    if (filter_s[i]=='<') {
      op=LT;
      filter_s[i]=0;
      i++;
      break;
    }
    if (filter_s[i]=='>') {
      op=GT;
      filter_s[i]=0;
      i++;
      break;
    }
    if (filter_s[i]=='<' && filter_s[i+1]=='=') {
      op=LTE;
      filter_s[i]=0;
      i+=2;
      break;
    }
    if (filter_s[i]=='>' && filter_s[i+1]=='=') {
      op=GTE;
      filter_s[i]=0;
      i+=2;
      break;
    }
  }
  chan=strdup(filter_s);
  value=atof(filter_s+i);
}

Filter::~Filter() {
  printf("deleting filter\n");
  free(chan);
}

short Filter::valid() {
  return (op!=INVALID);
}

void Filter::print() {
  printf("filter chan=%s, operator=",chan);
  switch (op) {
    case EQUAL:
      printf("EQUAL");
      break;
    case NEQUAL:
      printf("NOT EQUAL");
      break;
    case LT:
      printf("LESS THAN");
      break;
    case GT:
      printf("GREATER THAN");
      break;
    case LTE:
      printf("LESS THAN OR EQUAL");
      break;
    case GTE:
      printf("GREATER THAN OR EQUAL");
      break;
    default:
      printf("INVALID OPERATOR");
  }
  printf(", value=%.3e\n",value);
}

short Filter::checkEvent(struct event *e) {
  char *field=get_event_field(e,chan);
  if (field==NULL) {
    return 0;
  }
  //printf("filter: checking %s=%s against %f\n",chan,field,value);
  float e_val=atof(get_event_field(e,chan));
  switch (op) {
    case EQUAL:
      return (e_val==value);
    case NEQUAL:
      return (e_val!=value);
    case LT:
      return (e_val<value);
    case GT:
      return (e_val>value);
    case LTE:
      return (e_val<=value);
    case GTE:
      return (e_val>=value);
    default:
      return 0;
  }
}
