/*
Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
This file is part of Pyrame.

Pyrame is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Pyrame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
*/

#include <pyrame.h>

void free_id_list(struct id_list *il) {
  free(il->ids);
  free(il);
}

void print_id_list(struct id_list *il) {
  int i;
  printf("id_list(%d)=",il->nb_ids);
  for(i=0;i<il->nb_ids;i++) {
    printf("%d ",il->ids[i]);
  }
  printf("\n");
}

struct id_list *bl2ip(char *bl) {

  struct id_list *result=malloc(sizeof(struct id_list));
  int size=0;
  char *p=bl;
  int rd;
  int i=0;

  if (p==NULL)
    return result;

  if (*p!=0) {
    while (*p!=0) {
      if (*p==',')
	size++;
      p++;
    }
    size++;
  }

  result->nb_ids=size;
  result->ids=malloc(size*sizeof(int));
  
  p=bl;
  while (*p!=0) {
    sscanf(p,"%d",&rd);
    result->ids[i]=rd;
    i++;
    while(*p!=',' && *p!=0) {
      p++;
    }
    if (*p==',') {
      p++;
    }
  }
  return result;
}

char *get_block_str(int socket,int blockid) {
  struct cmd_result *pyres;
  char sblockid[128];
  char *result;

  if (socket==-1)
    return strdup("");
  
  //getting data
  sprintf(sblockid,"%d",blockid);
  pyres=execcmd(socket,"getblocklist_module",sblockid,"end");
  if (pyres->retcode==0) {
    printf("cant get block list : %s\n",pyres->str);
    free_cmd_result(pyres);
    return NULL;
  } else {
    result=pyres->str;
    free(pyres);
    return result;
  }
}


struct id_list *get_block_list(int socket,unsigned int block_index) {
  struct id_list *bl;
  char *bs;
  bs=get_block_str(socket,block_index);
  if (bs==NULL)
    return NULL;
  bl=bl2ip(bs);
  free(bs);
  return bl;
}

int get_next_block(int socket,unsigned int block_lim) {
  struct cmd_result *pyres;
  int result;
  char sblockid[128];

  if (socket==-1)
    return -1;
  
  //getting data
  sprintf(sblockid,"%d",block_lim);
  pyres=execcmd(socket,"getnextblock_module",sblockid,"end");
  if (pyres->retcode==0) {
    result=-1;
  } else {
    result=atoi(pyres->str);
  }
  free_cmd_result(pyres);
  return result;
}
