/*
Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
This file is part of Pyrame.

Pyrame is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Pyrame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
*/

#include <pyrame.h> 

struct block *new_block() {
  struct block *b=malloc(sizeof(struct block));
  memset(b,0,sizeof(struct block));
  return b;
}

struct block *new_block_from_es(struct evtstruct *es,int id) {
  struct block *b=malloc(sizeof(struct block));
  memset(b,0,sizeof(struct block));
  b->props=malloc(sizeof(char *)*es->nb_props);
  memset(b->props,0,sizeof(char *)*es->nb_props);
  b->props_size=es->nb_props;
  b->id=id;
  b->es=es;
  return b;
}

void free_block(struct block *b) {
  int i;
  if (!b) {
    return;
  }
  if (b->props) {
    for (i=0;i<b->props_size;i++) {
      if (b->props[i])
        free(b->props[i]);
    }
    free(b->props);
  }
  if (b->bstr)
    free(b->bstr);
  if (b->events)
    free_event(b->events);
  free(b);
}

void add_event(struct block *b,struct event *e) {
  e->block=b;
  e->next=b->events;
  b->events=e;
  b->nb_events++;
}

void add_event_inorder(struct block *b,struct event *e) {
  struct event *iter;
  if (b->events==NULL) {
    b->events=e;
    e->next=NULL;
  } else {
    iter=b->events;
    while(iter->next!=NULL)
      iter=iter->next;
    iter->next=e;
  }
  b->nb_events++;
}

void print_block(struct block *b) {
  int i;
  struct event *e;
  printf("block = \n");
  printf("schema=");
  print_evtstruct(b->es);
  if (b->props_size>0) {
    printf("props[%d]=",b->props_size);
    for(i=0;i<b->props_size;i++)
      printf("%s ",b->props[i]);
    printf("\n");
  } else {
    printf("no props\n");
  }
  printf("%d events\n",b->nb_events);
  e=b->events;
  i=0;
  while(e!=NULL) {
    printf("event %d : \n",i);
    print_event(e);
    e=e->next;
    i++;
  }
}

struct block * pull_block(int socket,unsigned int block_index,struct evtstruct *es) {
  struct cmd_result *cres;
  struct block *result=NULL;
  char bis[128];
  sprintf(bis,"%d",block_index);
  if (socket==-1)
    return NULL;
  cres=execcmd(socket,"getblock_module",bis,"end");
  if (cres->retcode!=0) {
    result=saf_parse_block(cres->str);
    result->es=es;
    result->id=block_index;
  }
  free_cmd_result(cres);
  return result;
}

int parse_dispatcher_res(char *dres,struct block **b,int *block_nb,struct evtstruct *es) {
  char *ptr=dres;
  while (*ptr!=',') {
    ptr++;
    if (*ptr==0) {
      return -4;
    }
  }
  *ptr=0;
  ptr++;
  *block_nb=atoi(dres);
  if (*ptr==0)
    return -4;
  saf_flexparse_block(ptr,es,b);
  if (*b==NULL) {
    return -4;
  }
  return 1;
}

int smart_get_block(int socket,char policy,int block_nb,int *run_nb,int *new_block_nb,struct block **b,struct evtstruct *es) {
  struct cmd_result *pres;
  int result;
  char s_run_nb[128];
  char s_policy[128];
  char s_block_nb[128];

  if (socket==-1)
    return -3;
  
  //getting data
  sprintf(s_run_nb,"%d",*run_nb);
  sprintf(s_policy,"%c",policy);
  sprintf(s_block_nb,"%d",block_nb);
  pres=execcmd(socket,"smartgetblock_module",s_run_nb,s_policy,s_block_nb,"end");
  switch (pres->retcode) {
    case 1:
      result=parse_dispatcher_res(pres->str,b,new_block_nb,es);
      if (result==-4) {
        printf("error: syntax error in dispatcher response %s\n",pres->str);
      }
      break;
    case -1:
      result=-1;
      break;
    case -2:
      *run_nb=atoi(pres->str);
      result=-2;
      break;
    default:
      result=pres->retcode;
      printf("error: on smartgetblock %d,%s\n",pres->retcode,pres->str);
  }
  free_cmd_result(pres);
  return result;
}
