/*
 * Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
 * This file is part of Pyrame.
 *
 * Pyrame is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Pyrame is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
 * */

#include <pyrame.h>

//********************** SEND COMMAND FUNCTIONS ***************************

struct cmd_result *send_cmd(char * host,int port,struct cmd *command) {
  char *cmd;
  struct cmd_result *result;

  //convert the command in string command
  cmd=cmd_2_strcmd(command);

  //send the command and get the result
  result=send_strcmd(host,port,cmd);

  //free the command
  free(cmd);

  //return the result
  return result;
}

struct cmd_result *sendcmd(char * host,int port,char * func_name,...) {
  char *cmd;
  struct cmd_result *result;

  //converting arguments in string command
  va_list pa;
  va_start(pa,func_name);
  cmd=args_2_strcmd(func_name,pa);
  va_end(pa);

  //send the command and get the result
  result=send_strcmd(host,port,cmd);

  //free the command
  free(cmd);

  //return the result
  return result;
}
  

struct cmd_result *send_strcmd(char * host,int port,char * strcmd) {
  int socket;
  int i;
  struct cmd_result *result;
  
  //open a socket
  socket=open_socket(host,port);
  if (socket==-1) {
    return raw_build_result(0,"unable to connect TCP socket");
  }

  //send the command
  result=exec_strcmd(socket,strcmd);
 
  if (result->retcode==2) { //wakeup

    //free the result
    free_cmd_result(result);
    //close the socket
    close(socket);

    for (i=0;i<10;i++) {
      //wait for the close to be effective 
      usleep(500000);

      //reopen the socket
      socket=open_socket(host,port);
      if ((socket==-1) && (i==9)) {
        return raw_build_result(0,"unable to connect TCP socket");
      }
      if (socket!=-1) {
        break;
      }
    }

    //send the command
    result=exec_strcmd(socket,strcmd);
  }
    
  close(socket);
  return result;
}

//********************** EXEC COMMAND FUNCTIONS ***************************

  
struct cmd_result *execcmd(int socket,char * func_name,...) {
  char *cmd;
  struct cmd_result *result;

  //converting arguments in command_string
  va_list pa;
  va_start(pa,func_name);
  cmd=args_2_strcmd(func_name,pa);
  va_end(pa);
  
  //send the command
  result=exec_strcmd(socket,cmd);

  //free the command
  free(cmd);

  //return the result
  return result;
}
 
struct cmd_result *exec_cmd(int socket,struct cmd *command) {
  char *cmd;
  struct cmd_result *result;
  
  //convert the command in string command
  cmd=cmd_2_strcmd(command);
  
  //send the command
  result=exec_strcmd(socket,cmd);
  
  //free the command
  free(cmd);
  
  //return the result
  return result;
}

struct cmd_result *exec_strcmd(int socket,char * strcmd) {
  int size;
  struct cmd_result *result;
  signal(SIGPIPE,SIG_IGN);
  size=write(socket,strcmd,strlen(strcmd));
  if (size<=0) {
    close(socket);
    return raw_build_result(0,"Error sending command (socket closed)");
  }
  result=get_cmd_result(socket);
  return result;
}
