/*
Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
This file is part of Pyrame.

Pyrame is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Pyrame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
*/


#include <pyrame.h>

//print a pyrame command
void print_cmd(struct cmd *command) {
  int i;
  if (command==NULL) {
    printf("null command\n");
    return;
  }
  if (command->name==NULL) {
    printf("command with no name\n");
    return;
  }
  printf("command %s with %d params : ",command->name,command->nb_params);
  if (command->nb_params==0)
    printf("\n");
  for (i=0;i<command->nb_params;i++) {
    if (i==command->nb_params-1)
      printf("%s\n",command->params[i]);
    else
      printf("%s,",command->params[i]);
  }
}

//start handler for pyrame command parser
void cmd_handler_start(void *data, const char *el, const char **attr) {
  struct cmd *toparse=(struct cmd*)data;
  if (!strcmp(el,"cmd")) {
    strncpy(toparse->name,attr[1],1024);
    toparse->nb_params=0;
  }
  if (!strcmp(el,"param")) {
    toparse->nb_params++;
  }
}

//end handler for pyrame command parser
void cmd_handler_end(void *data, const char *el) {
  struct cmd *toparse=(struct cmd*)data;
  if (!strcmp(el,"param")) {
    if (toparse->params[toparse->nb_params-1]==NULL) {
      toparse->params[toparse->nb_params-1]=strdup("");
    }
  }
}

//data handler for pyrame command parser
void cmd_handler_data(void *data,const char *value,int len) {
  int i;
  struct cmd *toparse=(struct cmd*)data;

  if (toparse->nb_params==toparse->allocated) {
    printf("reallocating params\n");
    toparse->params=realloc(toparse->params,(toparse->allocated+100)*sizeof(char *));
    for(i=toparse->allocated;i<toparse->allocated+100;i++) {
      toparse->params[i]=NULL;
    }
    toparse->allocated+=100;
    if (toparse->params==NULL) {
      printf("ERROR : cant reallocated toparse->params...exiting\n");
      sleep(1);
      exit(1);
    }
  }

  toparse->params[toparse->nb_params-1]=malloc((len+1)*sizeof(char));
  strncpy(toparse->params[toparse->nb_params-1],value,len);
  toparse->params[toparse->nb_params-1][len]=0;
}

//convert an xml buffer containing a pyrame command in struct cmd
struct cmd *parse_command(char * buf) {

  //allocate the command structure to fill with data from xml
  struct cmd *toparse=malloc(sizeof(struct cmd));
  toparse->name=malloc(1024*sizeof(char));
  memset(toparse->name,0,1024);
  toparse->nb_params=0;
  toparse->params=malloc(100*sizeof(char *));
  memset(toparse->params,0,100*sizeof(char*));
  toparse->allocated=100;
  struct cmd *result;
  struct incr_string *buf_encoded=new_is();

  //create the parser
  XML_Parser parser=XML_ParserCreate(NULL);
  XML_SetUserData(parser,toparse);
  XML_SetElementHandler(parser,cmd_handler_start,cmd_handler_end);
  XML_SetCharacterDataHandler(parser,cmd_handler_data);

  prefix_is(buf_encoded,"<?xml version=\"1.0\" encoding=\"utf-8\"?>");
  suffix_is(buf_encoded,buf);

  //parse the command
  if (!XML_Parse(parser,buf_encoded->str,buf_encoded->size,1)) {
    printf("error in command XML_Parse : error=%d\n",XML_GetErrorCode(parser));
    printf("%s\n",XML_ErrorString(XML_GetErrorCode(parser)));
    printf("buffer=%s\n",buf_encoded->str);
    free_cmd(toparse);
    result=NULL;
  } else {
    result=toparse;
  }
  free_is(buf_encoded);
  XML_ParserFree(parser);
  return result;
}

void free_cmd(struct cmd *tofree) {
  int i;
  for (i=0;i<tofree->nb_params;i++) 
    free(tofree->params[i]);
  free(tofree->params);
  free(tofree->name);
  free(tofree);
}

char * args_2_strcmd(char * func_name,va_list pa) {
  struct incr_string *ires=new_is();
  char *param;
  fsuffix_is(ires,"<cmd name=\"%s\">",func_name);
  while(1) { 
    param=va_arg(pa,char *);
    if (!strcmp(param,"end")) {
      break;
    } else {
      fsuffix_is(ires,"<param>%s</param>",param);
    }
  }
  fsuffix_is(ires,"</cmd>\n");
  return reduce_is(ires);
}

char * cmd_2_strcmd(struct cmd *command) {
  struct incr_string *ires=new_is();
  int i;
  fsuffix_is(ires,"<cmd name=\"%s\">",command->name);
  for (i=0;i<command->nb_params;i++) {
    fsuffix_is(ires,"<param>%s</param>",command->params[i]);
  }
  fsuffix_is(ires,"</cmd>\n");
  return reduce_is(ires);
}





