/*
 * Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
 * This file is part of Pyrame.
 *
 * Pyrame is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Pyrame is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
 * */

#include <pyrame.h>

//events containers

//create a new empty container
struct evtc *new_evtc() {
  struct evtc *result=malloc(sizeof(struct evtc));
  memset(result,0,sizeof(struct evtc));
  return result;
}


//free the container, all its blocks and events
void free_evtc(struct evtc *c) {
  struct block *todel;
  struct block *b=c->fb;
  while(b!=NULL) {
    todel=b;
    b=b->next;
    free_block(todel);
  }
}

//add a block b to a container c 
void add_block_evtc(struct evtc *c,struct block *b) {
  b->next=c->fb;
  c->fb=b;
  c->nb_blocks++;
}

//return the lesser "field" float value from all the non-used event of the container c
//return -1.0 if no non-used event are found
float find_smallest_evtc(struct evtc *c,char *field) {
  struct block *b=c->fb;
  struct event *e=NULL;
  char first=1;
  float lesser_time=-1.0;
  float current_time;
  while(b!=NULL) {
    e=b->events;
    while(e!=NULL) {
      if (e->used==1) {
        e=e->next;
        continue;
      }
      if (first==1) {
        first=0;
        lesser_time=atof(get_event_field(e,field));
        //printf("found %f\n",lesser_time);
      } else {
        current_time=atof(get_event_field(e,field));
        //printf("found %f\n",current_time);
        if (current_time<lesser_time) {
          lesser_time=current_time;
          //printf("better\n");
        }
      }
      e=e->next;
    }
    b=b->next;
  }
  return lesser_time;
}

//return the next non used event in container c, where float value of field is greater or equal to ref but lesser or equal to ref+tolerance
//    ref <= e->field <=ref+tolerance && e->used==0
//mark the returned event as used
//return NULL if such an event does not exists
struct event *find_related_evtc(struct evtc *c,char *field,float ref,float tolerance) {
  struct block *b=c->fb;
  struct event *e=NULL;
  float evtf;
  //printf("find_related_evtc\n");
  while(b!=NULL) {
    e=b->events;
    while(e!=NULL) {
      if (e->used!=1) {
        evtf=atof(get_event_field(e,field));
        //printf("event has field %f\n",evtf);
        if ((evtf>=ref) && (evtf<=ref+tolerance)) {
          e->used=1;
          return e;
        }
      } else {
        //printf("event is used\n");
      }
      e=e->next;
    }
    b=b->next;
  }
  //printf("return null\n");
  return NULL;
}


