%option nounput
%option noinput
%{
#include<pyrame.h>
enum field saf_state=PROPS;
struct block *cur_block=NULL;
struct event *cur_event=NULL;
struct evtstruct *schema=NULL;
int field_counter=0;
pthread_mutex_t *mutex=NULL;

void fl_new_event() {
  cur_event=new_event_from_es(cur_block);
  //printf("new event %p\n",cur_event);
  saf_state=TIME;
  field_counter=0;
}

void fl_move_state() {
  //printf("changing state\n");
  if (saf_state==TIME) 
    saf_state=SPACE; 
  else 
    saf_state=DATA;
  field_counter=0;
}

void fl_set_field(char *new_field) {

  //debug
  /*printf("new field %s in state ",new_field);
  print_field(saf_state);
  printf(" on block %p, event %p\n",cur_block,cur_event);*/


  if (!cur_block) {
    printf("flsaf error: syntax error of block\n");
    return;
  }
  if (saf_state==PROPS) {
    if (field_counter>=cur_block->es->nb_props) {
      printf("flsaf error: prop field outside es range\n");
      return;
    }
    cur_block->props[field_counter]=strdup(new_field);
  }

  if (saf_state!=PROPS && !cur_event) {
    printf("flsaf error: event is null and state is ");
    print_field(saf_state);
    printf("\n");
    return;
  }
  if (saf_state==TIME) { 
    if (field_counter>=cur_event->block->es->nb_time) {
      printf("flsaf error: time field outside es range\n");
      return;
    }
    cur_event->time[field_counter]=strdup(new_field);
  }
  if (saf_state==SPACE) {
    if (field_counter>=cur_event->block->es->nb_space) {
      printf("flsaf error: space field outside es range\n");
      return;
    }
    cur_event->space[field_counter]=strdup(new_field);
  }
  if (saf_state==DATA) {
    if (field_counter>=cur_event->block->es->nb_data) {
      printf("flsaf error: data field outside es range\n");
      return;
    }
    cur_event->data[field_counter]=strdup(new_field);
  }
  field_counter++;
}

%}
%%
!        fl_new_event();
\|       fl_move_state();
[^,!|]+  fl_set_field(yytext);
,        /* ignore this token */
%%


void saf_flexparse_block(char *input,struct evtstruct *es,struct block **b) {
  YY_BUFFER_STATE yy_buf;
  if (mutex==NULL) {
    mutex=malloc(sizeof(pthread_mutex_t));
    pthread_mutex_init(mutex,NULL);
  }
  pthread_mutex_lock(mutex);
  cur_event=NULL;
  cur_block=new_block_from_es(es,0);
  //printf("flsaf: created new block %p\n",cur_block);
  saf_state=PROPS;
  field_counter=0;
  //printf("flsaf input: %s\n",input);
  yy_buf=yy_scan_string(input);
  if (yy_buf) {
    yylex();
    yy_delete_buffer(yy_buf);
  } else {
    printf("flsaf error: null buffer\n");
  }
  *b=cur_block;
  pthread_mutex_unlock(mutex);
}
