/*
 * Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
 * This file is part of Pyrame.
 *
 * Pyrame is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Pyrame is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
 * */

#include <pyrame.h>

struct ports_table *bindpyrame_internal_table=NULL;

struct ports_table *init_ports_table(char * filename) {
  struct ports_table *table=malloc(sizeof(struct ports_table));
  FILE *f;
  int i=0, j;
  char line[2048];
  char *tmp;
  memset(table,0,sizeof(struct ports_table));
  //printf("opening port table file %s\n",filename);
  f=fopen(filename,"r");
  if (f==NULL) {
    printf("cant open port table file\n");
    perror("fopen");
    return table;
  }
  while(fgets(line,sizeof(line),f)) {
    if (line[0]=='#' || line[0]=='\n')
      continue;
    table->nb_ports++;
  } 
  table->names=malloc(table->nb_ports*sizeof(char *));
  table->values=malloc(table->nb_ports*sizeof(int));
  rewind(f);
  while(fgets(line,sizeof(line),f)) {
    if (line[0]=='#' || line[0]=='\n')
      continue;
    tmp=line;
    while(*tmp!='=') {
      tmp++;
      if (*tmp==0) {
        fprintf(stderr,"Line not properly formatted : %s",line);
        for (j=0;j<i;j++) {
          free(table->names[j]);
        }
        free(table->names);
        free(table->values);
        table->names = NULL;
        table->values = NULL;
        fclose(f);
        return NULL;
      }
    }
    *tmp=0;
    tmp++;
    table->names[i]=strdup(line);
    table->values[i]=atoi(tmp);
    i++;
  }
  fclose(f);
  return table;
}

void free_ports_table(struct ports_table *table) {
  int i;
  for (i=0;i<table->nb_ports;i++) {
    free(table->names[i]);
  }
  free(table->names);
  free(table->values);
}

int get_pyrame_port(char *name,struct ports_table *table) {
  int i,onlydigits=1;
  char *tmp;
  if (name==NULL)
    return 0;
  for (tmp=name;*tmp != 0 && onlydigits;tmp++)
    if (*tmp < 48 || *tmp > 57) onlydigits = 0;
  if (!onlydigits) {
    for (i=0;i<table->nb_ports;i++) {
      if (!strcmp(name,table->names[i]))
        return table->values[i];
    }
    return -1;
  } else return atoi(name);
}

int get_port(char *name) {
  int port;
  char port_str[1024];
  int i;
  if (bindpyrame_internal_table==NULL) {
    bindpyrame_internal_table=init_ports_table("/opt/pyrame/ports.txt");
  }
  if (bindpyrame_internal_table==NULL) {
    printf("internal table is not available\n");
    return -1;
  }
  port=get_pyrame_port(name,bindpyrame_internal_table);
  if (port!=-1)
    return port;
  for(i=0;i<strlen(name);i++) {
    if (name[i]>='a' && name[i]<='z') {
      port_str[i]=name[i]-'a'+'A';
    } else {
      port_str[i]=name[i];
    }
  }
  sprintf(port_str+i,"_PORT");
  return get_pyrame_port(port_str,bindpyrame_internal_table);
}

