/*
Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
This file is part of Pyrame.

Pyrame is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Pyrame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
*/

#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <netdb.h>
#include <net/if.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <unistd.h>
#include <pthread.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <arpa/inet.h>
#include <sys/ipc.h>
#include <math.h>
#include <expat.h>
#include <errno.h>
#include <stdarg.h>
#include <signal.h>
#include <search.h>


#ifndef PYRAME_H
#define PYRAME_H

#ifdef __cplusplus
extern "C" {
#endif

  //PORTS
  
struct ports_table {
  char **names;
  int *values;
  int nb_ports;
};

  //struct ports_table *init_ports_table(char * filename);
  //void free_ports_table(struct ports_table *table);
  //int get_pyrame_port(char *name,struct ports_table *table);
int get_port(char *name);
struct cmd_result {
  int retcode;
  char *str;
};

  //RESULT
  
struct cmd_result *get_cmd_result(int socket);
void free_cmd_result(struct cmd_result *result);
int send_result(int socket,struct cmd_result *result);
struct cmd_result *build_result(int retcode,char *format,...);
struct cmd_result *build_error(struct cmd_result *res,char *format);  
struct cmd_result *raw_build_result(int retcode,char *retstr);
struct cmd_result *nofree_build_result(int retcode,char *retstr);


  //COMMAND

struct cmd {
  char *name;
  int nb_params;
  char **params;
  int allocated;
}; 
  
struct cmd *parse_command(char * buf);
void print_cmd(struct cmd *command);
void free_cmd(struct cmd *tofree);
char * args_2_strcmd(char * func_name,va_list pa);
char * cmd_2_strcmd(struct cmd *command);

  //SHBUFFER
  
#include <sys/sem.h>
#include <sys/shm.h>

#define SHARED_BUFFER_SIZE 8192
#define MAX_DATA_SIZE SHARED_BUFFER_SIZE-2

struct shbuffer {
  int full_semid;
  int empty_semid;
  int mutex_semid;
  int ready_semid;
  int shmid;
  char * real_shared_buffer;
  char * shdata;
  int flushed;
  int tmp_size;
  char *tmp;
  int data_size;
  int timeout;
  int lastrel;
  int portid;
  struct timeval last_flush;
  pthread_mutex_t data_mutex;
  pthread_t t_monitor;
};

#define CREATE 1
#define NO_CREATE 0


struct shbuffer * open_shbuffer(int port_id,int create,int timeout);
void close_shbuffer(struct shbuffer *shbuf);
void flush_shbuffer(struct shbuffer *shbuf);
int isready_shbuffer(struct shbuffer *shbuf);
void ready_shbuffer(struct shbuffer *shbuf);
void unready_shbuffer(struct shbuffer *shbuf);
int read_shbuffer(struct shbuffer *shbuf,char *buffer);
int write_shbuffer(struct shbuffer *shbuf,char *buffer,int size);
void flush_shbuffer(struct shbuffer *shbuf);
void *monitor_shbuffer(void *);


struct incr_string {
  char * str;
  int alloc_size;
  int size;
};

  //INCR_STRINGS

struct incr_string *new_is(void);
void suffix_is(struct incr_string *res,char * addstr);
void fsuffix_is(struct incr_string *res,char *format,...);
void prefix_is(struct incr_string *res,char * addstr);
void free_is(struct incr_string *res);
  char *reduce_is(struct incr_string *ires);


  //UTILS
  
int open_socket(char * host, int port);
int file_open(char * filename);
int iohtoi(char * string);
void *str2ptr(char *addr);
char *ptr2str(void *ptr);
char **split(char *src,char sep,int *nb_res);
void free_split(char **split,int nb_elem);

  //COMM
  
struct cmd_result *sendcmd(char * host,int port,char * func_name,...);
struct cmd_result *send_cmd(char * host,int port,struct cmd *command);
struct cmd_result *send_strcmd(char * host,int port,char * strcmd);
struct cmd_result *execcmd(int socket,char * func_name,...);
struct cmd_result *exec_cmd(int socket,struct cmd *command);
struct cmd_result *exec_strcmd(int socket,char * strcmd);


  //NETSERV
  
#define BACKLOG 100
  //TODO remove limit
#define NETSERV_MAX_CLIENTS 10000
#define NETMAXCLIENTS 255

struct netserv_client {
  int *sock_client;
  int *control_socket;
  int nb_sock_client;
  int port;
  int max_clients;
  pthread_mutex_t mutex;
  int client_id;
  char initialized;
  int error_tries;
};

struct netserv_workspace {
  struct netserv_client **clients;
  int nb_clients;
  int reload;
  int maxclients;
  pthread_mutex_t mutex;
  int *listen_sockets;
};

void *netserv(void *);
struct netserv_client * init_netserv_client(int*,int,int,int);
void close_n_delete_netserv_client(struct netserv_client *);
void print_netserv_client(struct netserv_client *);
struct netserv_workspace * start_monoport_netserv(unsigned short,int);
struct netserv_workspace * start_empty_netserv(int);
struct netserv_client * netserv_add_new_port(struct netserv_workspace *,unsigned short);
int netserv_remove_port(struct netserv_workspace *,unsigned short);
void display_netserv(struct netserv_workspace *);
struct netserv_client *first_client(struct netserv_workspace *);
int send2client(struct netserv_client *,unsigned char *,int);


  //STATS
//variables operations
#define SOP_PLUS 1
#define SOP_SET  2
#define SOP_MIN  3

//statistics types
#define TYPE_ACC  0 //accumulator : just add the input values
#define TYPE_RATE 1 //compute a rate on another stat
#define TYPE_INST 2 //instantaneous value
#define TYPE_SW   3 //sliding window for stable rates
#define TYPE_PC   4 //percentage

#define MALLOC_STEP 1000

struct pyr_stat {
  char *name;
  int type;
  double varvalue;//the value for varmod
  double value;//the real value
  char *ref; //for rate or sliding window
  char *refsum; //for percentage
  double *mem; //for sliding window
  int win_size; //for sliding window
  char initialized; //for accumulator to know what to do : set or add
  struct timeval last_time;
};

struct stats_ws {
  struct pyr_stat **stats; //contain the stats structures
  int nb_stats; //number of active stats
  int malloced; //numver of allocated cells for stats
  int vport; //varmod port
  char *vip; //varmod ip
  int varmod_socket; //socket to varmod
  pthread_mutex_t *stat_mutex; //mutex to protect stats access
  pthread_t *t_stats; //update stats thread
  int periodicity; //of the varmod update in microsecond
  char socket_pb; //1 if problems appears with the varmod socket
  char block_update;
  char in_update;
};

struct stats_ws *init_stats(char *,int);
void add_rate(struct stats_ws *,char *,char *);
void add_acc(struct stats_ws *,char *,double);
void add_sw(struct stats_ws *,char *,char *,int);
void add_pc(struct stats_ws *,char *,char *,char *);
void add_inst(struct stats_ws *,char *,double);
void deinit_stats(struct stats_ws *);
void zero_stats(struct stats_ws *);
void set_stats(struct stats_ws *,char *,int,double);
char *get_stats(struct stats_ws *);
void dump_stats_to_log(struct stats_ws *,int);
void *thread_stats(void *);
void update_stats(struct stats_ws *);
void compute_stats(struct stats_ws *);

//DATA


struct evtstruct;

//block/event structure 

struct block {
  int id;
  char finalized;
  int nb_events;
  char **props;
  int props_size;
  char *bstr;
  struct evtstruct *es;
  struct event *events;
  struct block *next;
};
  
struct event {
  char **time;
  int time_size;
  char **space;
  int space_size;
  char **data;
  int data_size;
  char used;
  struct event *next;
  struct block *block;
};

//event container
struct evtc {
  struct block *fb;
  int nb_blocks;
};

//event container
struct evtab {
  struct event **data;
  int nb_data;
  int lim_data;
};

//event loops
  
enum run_mode {STOPPED,ONESTEP,ONEBLOCK,RUNNING};

struct evloop {
  int id;
  pthread_t *t_loop;
  struct event *e;
  struct block *b;
  int command;
  char * ds_name;
  int dataport;
  int datareadyport;
  char *host;
  void *workspace;
  void (*newevt_func)(void *,struct event *);
  void (*newblock_func)(void *,struct block *);
  void (*endblock_func)(void *,struct block *);
  void (*reinit_func)(void *);
  char policy;
  enum run_mode mode;
  int run_nb;
  char new_pol;
  int dr_socket;
  int last_key;
  struct evtstruct *es;
  int avail_data;
};

#define CMD_REWIND 1
#define CMD_STOP 2
#define CMD_RUN 3
#define CMD_ONESTEP 4
#define CMD_CHANGE_POL 5
#define CMD_CLEAN 6
#define CMD_ONEBLOCK 7

#define CMD_SLEEP_TIME 10000

//evtstruct is a data describing structure
  
enum field {PROPS,TIME,SPACE,DATA};
  
struct evtstructelem {
  char *name;
  enum field domain;
  int index;
  struct evtstructelem *next;
};
  
struct evtstruct {
  int nb_time;
  int nb_space;
  int nb_data;
  int nb_props;
  struct evtstructelem *first;
};

struct id_list {
  int nb_ids;
  int *ids;
};

#define ENOFIELD 1
#define EBLNULL 2
#define EALFINAL 3
#define ENODISPATCH 4
#define EMONOTONY 5
#define EEVNULL 6
#define EBADBLOCK 7
#define EBADSCHEMA 8
#define ENOVARMOD 9
#define ECANTREGDS 10
#define ENOTHREAD 11

extern char *pyrame_errors[11];

//saf
struct evtstruct *saf_parse_chans(char *chans);
struct block *saf_parse_block(char *input);
  void saf_flexparse_block(char *input,struct evtstruct *es,struct block **b);
struct event *saf_parse_event(char **toparse,struct block *b);
void saf_dump_event(struct event *e,int output_fd);
void saf_dump_block(struct block *b,int output_fd);
char * saf_str_event(struct event *e);
char *saf_str_block(struct block *b);
  
//evtstruct
void free_evtstruct(struct evtstruct *es);
void print_evtstruct(struct evtstruct *es);
char *get_block_field(struct block *b,char *field_name);
char *get_event_field(struct event *e,char *field_name);
void print_field(enum field domain);
void convert_from_evtstruct(struct event *source,struct evtstruct *convert_es,struct block *b);
struct evtstructelem *get_ese(struct evtstruct *es,char *field_name);
int set_event_field(struct event *e,struct evtstructelem *ese,char *value);
int set_block_field(struct block *b,struct evtstructelem *ese,char *value);
int check_block(struct block *bl);
int check_event(struct event *e);
char exists_prop(struct evtstruct *es,char *name);

//event
void print_event(struct event *e);
char *parse_string(char *s,struct event *e);
void free_event(struct event *e);
struct event *new_event(struct block *b);
struct event *new_event_from_es(struct block *b);

//blocks
struct block *new_block(void);
struct block *new_block_from_es(struct evtstruct *es,int id);
void free_block(struct block *b);
void print_block(struct block *b);
struct block * pull_block(int socket,unsigned int block_index,struct evtstruct *es);
void add_event(struct block *b,struct event *e);
void add_event_inorder(struct block *b,struct event *e);
int smart_get_block(int socket,char policy,int block_nb,int *run_nb,int *new_block_nb,struct block **b,struct evtstruct *es);

//block list
struct id_list *bl2ip(char *bl);
void print_id_list(struct id_list *il);
void free_id_list(struct id_list *il);
struct id_list *get_block_list(int socket,unsigned int block_index);
int get_next_block(int socket,unsigned int block_lim);

//event loop
struct evloop * new_event_loop(char * ds_name,void *workspace,void (*newevt_func)(void *,struct event *),void (*newblock_func)(void *,struct block *),void (*endblock_func)(void *,struct block *),void (*reinit_func)(void *),char policy);
void free_event_loop(struct evloop *el);
void rewind_loop(struct evloop *ws);
void onestep_loop(struct evloop *ws);
void oneblock_loop(struct evloop *ws);
void stop_loop(struct evloop *ws);
void run_loop(struct evloop *ws);
void clean_loop(struct evloop *ws);
void changepol_loop(struct evloop *ws,char new_policy);

//event container
struct evtc *new_evtc(void);
void free_evtc(struct evtc *c);
void add_block_evtc(struct evtc *c,struct block *b);
float find_smallest_evtc(struct evtc *c,char *field);
struct event *find_related_evtc(struct evtc *c,char *field,float ref,float tolerance);

#ifdef __cplusplus
}
#endif

#endif
