/*
Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
This file is part of Pyrame.

Pyrame is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Pyrame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
*/


#include <pyrame.h>

int send_result(int socket,struct cmd_result *result) {
  char *sres;
  int res;

  if (result==NULL) {
    sres=malloc(50);
    sprintf(sres,"<res retcode=\"0\">No result</res>\n");
  } else { 
    if (result->str==NULL) {
      sres=malloc(50);
      sprintf(sres,"<res retcode=\"%d\">No result</res>\n",result->retcode);
    } else { 
      sres=malloc(50+strlen(result->str));
      sprintf(sres,"<res retcode=\"%d\"><![CDATA[%s]]></res>\n",result->retcode,result->str);
    }
  }
  signal(SIGPIPE,SIG_IGN);
  res=write(socket,sres,strlen(sres));
  if (res!=strlen(sres)) {
    printf("Error : transmission incomplete : %d instead of %d : sres=%s\n",res,(int)strlen(sres),sres);
  }
  free(sres);
  return res;
}

void result_handler_start(void *data, const char *el, const char **attr) {
  struct cmd_result *toparse=(struct cmd_result*)data;
  if (!strcmp(el,"res")) {
    toparse->retcode=atoi(attr[1]);   
  }
}

void result_handler_end(void *data, const char *el) {
  return;
}

void result_handler_data(void *data,const char *value,int len) {
  struct cmd_result *toparse=(struct cmd_result*)data;
  char *tmp;
  int tmplen;
  if (toparse->str==NULL) {
    toparse->str=malloc(sizeof(char)*len+2);
    strncpy(toparse->str,value,len);
    toparse->str[len]=0;
  } else {
    tmp=toparse->str;
    tmplen=strlen(tmp);
    toparse->str=realloc(tmp,tmplen+len+2);
    strncpy(toparse->str+tmplen,value,len);
    toparse->str[tmplen+len]=0;
  }
}

struct cmd_result *parse_result(char * buf) {
  struct cmd_result *result=malloc(sizeof(struct cmd_result));
  memset(result,0,sizeof(struct cmd_result));
  XML_Parser parser = XML_ParserCreate(NULL);
  XML_SetUserData(parser,result);
  XML_SetElementHandler(parser,result_handler_start,result_handler_end);
  XML_SetCharacterDataHandler(parser,result_handler_data);

  if (!XML_Parse(parser,buf,strlen(buf),1)) {
    fprintf(stderr,"error in result XML_Parse : error=%d\n",XML_GetErrorCode(parser));
    fprintf(stderr,"%s\n",XML_ErrorString(XML_GetErrorCode(parser)));
    fprintf(stderr,"buffer=%s\n",buf);
    result->str=strdup("Malformed command");
    result->retcode=0;
  }
  XML_ParserFree(parser);
  if (result->str==NULL) {
    result->str=malloc(1);
    result->str[0]=0;
  }
  return result;
}

struct cmd_result *get_cmd_result(int socket){
  int msg_s=4096;
  char *msg;
  int j=0;
  int i;
  int size;
  struct cmd_result *result;
  int end_found=0;
  fd_set read_fds;
  //struct timeval tv;
  FD_ZERO(&read_fds);
  FD_SET(socket,&read_fds);
  int res;
  res = select(socket+1,&read_fds,NULL,NULL,NULL);

  //error checking
  if (res==-1) {
    perror("select");
    return raw_build_result(0,"Error in select");
  }

  //effective reading of result
  msg=malloc(msg_s*sizeof(char));
  do {
    if (j+1024>=msg_s) {
      msg_s+=4096;
      msg=realloc(msg,msg_s);
    }
    size=read(socket,msg+j,1024);
    if (size<=0) {
      free(msg);
      msg=NULL;
      break;
    } else {
      for (i=j;i<j+size;i++)
	if (msg[i]=='\n') {
	  end_found=1;
	  msg[i+1]=0;
	}
      j+=size;
    }  
  } while(end_found==0); 

  //parsing result
  if (msg) {
    result=parse_result(msg);
    free(msg);
  } else {
    result=build_result(0,"Destination module unavailable (socket closed)");
  }
  return result;
}

void free_cmd_result(struct cmd_result *result) {
  free(result->str);
  free(result);
}

struct cmd_result *nofree_build_result(int retcode,char *retstr) {
  struct cmd_result*cres=malloc(sizeof(struct cmd_result));
  cres->retcode=retcode;
  cres->str=retstr;
  return cres;
}

struct cmd_result *raw_build_result(int retcode,char *retstr) {
  struct cmd_result*cres=malloc(sizeof(struct cmd_result));
  cres->retcode=retcode;
  cres->str=strdup(retstr);
  return cres;
}

struct cmd_result *build_result(int retcode,char *format,...) {
  int req_size;
  va_list pa;
  struct cmd_result*cres=malloc(sizeof(struct cmd_result));
  cres->retcode=retcode;

  //get the required size for the string
  cres->str=malloc(4096*sizeof(char));
  va_start(pa,format);
  req_size=vsnprintf(cres->str,4096,format,pa);
  va_end(pa);

  //allocate and fill the string
  if (req_size>4096) {
    free(cres->str);
    cres->str=malloc((req_size+10)*sizeof(char));
    va_start(pa,format);
    vsprintf(cres->str,format,pa);
    va_end(pa);
  }

  //return res
  return cres;
}

struct cmd_result *build_error(struct cmd_result *res,char *format) {
  struct cmd_result *pres;
  pres=build_result(0,format,res->str);
  free_cmd_result(res);
  return pres;
}


