/*
Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
This file is part of Pyrame.

Pyrame is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Pyrame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
*/

#include <pyrame.h>

//************************** GENERIC PARSER ************************

int saf_count_word(char *string,int begin,char delim) {
  int i=begin;
  int counter=0;
  if (string[i]==delim || string[i]==0)
    return 0;
  while(string[i]!=0 && string[i]!=delim) {
    i++;
    if (string[i]==',')
      counter++;
  }
  return counter+1;
}

int saf_parse_word(char **toparse,char delim,char next_delim,void *ws,int (*treat_field)(enum field,int,void *),int (*treat_data)(enum field,int,char *,void *),enum field domain) {
  int j=0;
  int b=0;
  int counter;
  int parse_counter;
  char *result;

  //printf("parsing word on %.*s...\n",50,*toparse);

  //verifying the delimiter and removing it
  if (delim!=-1) {
    if (*toparse[j]!=delim) {
      printf("error : first char should be %c and is %c...moving to next delimiter\n",delim,*toparse[j]);
      while(*toparse[j]!=delim && *toparse[j]!=0) {
        j++;
        b++;
      }
    }
    
    if (*toparse[j]==0) {
      printf("no delimiter found\n");
      return -1;
    }
  
    //printf("delimiter is %c\n",*toparse[j]);
    j++;
    b++;
  }

  //count the number of subwords
  counter=saf_count_word(*toparse,j,next_delim);
  //printf("counter=%d\n",counter);

  //allocate space for data
  treat_field(domain,counter,ws);
  
  //extract the data
  parse_counter=0;
  while((*toparse)[j]!=next_delim && (*toparse)[j]!=0) {
    if ((*toparse)[j]==',') {
      if (j!=b) {
        //treat the data
        result=strndup((*toparse)+b,j-b);
        result[j-b]=0;
        treat_data(domain,parse_counter,result,ws);
        //printf("adding a field (1) %s\n",result);
        parse_counter++;
        b=j+1;
      } else {
        printf("corruption : aborting\n");
        return -1;
      }
    }
    j++;
  }

  //treat the last data
  if (j!=b) {
    result=strndup((*toparse)+b,j-b);
    result[j-b]=0;
    treat_data(domain,parse_counter,result,ws);
    //printf("adding a field (2) %s\n",result);
  }

  //moving toparse after data
  *toparse=(*toparse)+j;
  return 0;
}

//******************************* EVENTS **********************************

int event_treat_field(enum field domain,int nb_data,void *ws) {
  struct event *e=(struct event *)ws;
  switch(domain) {
  case TIME:
    e->time=malloc(nb_data*sizeof(char *));
    e->time_size=nb_data;
    break;
  case SPACE:
    e->space=malloc(nb_data*sizeof(char *));
    e->space_size=nb_data;
    break;
  case DATA:
    e->data=malloc(nb_data*sizeof(char *));
    e->data_size=nb_data;
    break;
  case PROPS:
    printf("error : props should not be processed in event\n");
    break;
  }
  return 0;
}

int event_treat_data(enum field domain,int index,char *value,void *ws) {
  struct event *e=(struct event *)ws;
  switch(domain) {
  case TIME:
    e->time[index]=value;
    break;
  case SPACE:
    e->space[index]=value;
    break;
  case DATA:
    e->data[index]=value;
    break;
  case PROPS:
    printf("error : props should not be processed in event\n");
    break;
  }
  return 0;
}

struct event *saf_parse_event(char **toparse,struct block *b) {
  struct event *e=new_event(b);
  int error;
 
  //parse time
  //printf("parsing %s in domain time\n",*toparse);
  error=saf_parse_word(toparse,'!','|',(void *)e,event_treat_field,event_treat_data,TIME);
  if (error!=0) {
    printf("parse error in time : aborting this event\n");
    free_event(e);
    return NULL;
  }
    
  //parse space
  //printf("parsing %s in domain space\n",*toparse);
  error=saf_parse_word(toparse,'|','|',(void *)e,event_treat_field,event_treat_data,SPACE);
  if (error!=0) {
    printf("parse error in space : aborting this event\n");
    free_event(e);
    return NULL;
  }
    
  //parse data
  //printf("parsing %s in domain data\n",*toparse);
  error=saf_parse_word(toparse,'|','!',(void *)e,event_treat_field,event_treat_data,DATA);
  if (error!=0) {
    printf("parse error in data  : aborting this event\n");
    free_event(e);
    return NULL;
  }
  
  return e;
}




void saf_dump_event(struct event *e,int output_fd) {
  int i;
  if (e==NULL)
    return;

  dprintf(output_fd,"!");
  for(i=0;i<e->time_size;i++)
    if (i==0) {
      dprintf(output_fd,"%s",e->time[i]);
    } else {
      dprintf(output_fd,",%s",e->time[i]);
    }
  dprintf(output_fd,"|");
  for(i=0;i<e->space_size;i++)
    if (i==0) {
      dprintf(output_fd,"%s",e->space[i]);
    } else {
      dprintf(output_fd,",%s",e->space[i]);
    }
  dprintf(output_fd,"|");
  for(i=0;i<e->data_size;i++)
    if (i==0) {
      dprintf(output_fd,"%s",e->data[i]);
    } else {
      dprintf(output_fd,",%s",e->data[i]);
    }
}

char * saf_str_event(struct event *e) {
  int i;
  struct incr_string* res=new_is();

  fsuffix_is(res,"!");
  for(i=0;i<e->time_size;i++) {
    if (i==0) {
      fsuffix_is(res,"%s",e->time[i]);
    } else {
      fsuffix_is(res,",%s",e->time[i]);
    }
  }
  fsuffix_is(res,"|");
  for(i=0;i<e->space_size;i++) {
    if (i==0) {
      fsuffix_is(res,"%s",e->space[i]);
    } else {
      fsuffix_is(res,",%s",e->space[i]);
    }
  }
  fsuffix_is(res,"|");
  for(i=0;i<e->data_size;i++) {
    if (i==0) {
      fsuffix_is(res,"%s",e->data[i]);
    } else {
      fsuffix_is(res,",%s",e->data[i]);
    }
  }
  return reduce_is(res);
}

//***************************** BLOCKS *************************************

char *saf_str_block(struct block *b) {
  struct incr_string* res=new_is();
  int i;
  struct event *e;
  char *ste;
  
  for(i=0;i<b->props_size;i++) {
    if (i==0) {
      fsuffix_is(res,"%s",b->props[i]);
    } else {
      fsuffix_is(res,",%s",b->props[i]);
    }
  }
  e=b->events;
  while(e!=NULL) {
    ste=saf_str_event(e);
    fsuffix_is(res,"%s",ste);
    free(ste);
    e=e->next;
  }
  return reduce_is(res);
}

void saf_dump_block(struct block *b,int output_fd) {
  int i;
  struct event *e;
  if (b==NULL)
    return;
  for(i=0;i<b->props_size;i++)
    if (i==0) {
      dprintf(output_fd,"%s",b->props[i]);
    } else {
      dprintf(output_fd,",%s",b->props[i]);
    }
  e=b->events;
  while(e!=NULL) {
    saf_dump_event(e,output_fd);
    e=e->next;
  }
  dprintf(output_fd,"\n");
}

int block_treat_field(enum field domain,int nb_data,void *ws) {
  struct block *b=(struct block *)ws;
  b->props=malloc(nb_data*sizeof(char *));
  b->props_size=nb_data;
  return 0;
}

int block_treat_data(enum field domain,int index,char *value,void *ws) {
  struct block *b=(struct block *)ws;
  b->props[index]=value;
  return 0;
}


struct block *saf_parse_block(char *input) {
  struct block *b=new_block();
  struct event *e;
  char *toparse=input;
  int error;
  //parse props
  //printf("parsing %s in domain props\n",*toparse);
  error=saf_parse_word(&toparse,-1,'!',(void *)b,block_treat_field,block_treat_data,PROPS);
  if (error!=0) {
    printf("parse error in props : aborting this event\n");
    free_block(b);
    return NULL;
  }
  while(toparse[0]!=0) {
    //printf("still some data to parse : %s\n",toparse);
    e=saf_parse_event(&toparse,b);
    if (e==NULL) {
      continue;
    }
    add_event_inorder(b,e);
  }
  return b;
}

//**************************** CHANNELS *********************************


int chans_treat_field(enum field domain,int nb_data,void *ws) {
  //struct evtstruct *es=(struct evtstruct *)ws;
  return 0;
}

int chans_treat_data(enum field domain,int index,char *value,void *ws) {
  struct evtstruct *es=(struct evtstruct *)ws;
  struct evtstructelem *ese=malloc(sizeof(struct evtstructelem));
  ese->name=value;
  ese->domain=domain;
  ese->index=index;
  ese->next=es->first;
  es->first=ese;
  switch(domain) {
  case TIME:
    es->nb_time++;
    break;
  case SPACE:
    es->nb_space++;
    break;
  case DATA:
    es->nb_data++;
    break;
  case PROPS:
    es->nb_props++;
    break;
  }
  return 0;
}

struct evtstruct *saf_parse_chans(char *chans) {
  struct evtstruct *result=malloc(sizeof(struct evtstruct));
  int error;
  result->first=NULL;
  char *toparse=chans;

  memset(result,0,sizeof(struct evtstruct));

  error=saf_parse_word(&toparse,-1,'!',(void *)result,chans_treat_field,chans_treat_data,PROPS);
  if (error!=0) {
    printf("error during parsing props chans\n");
    free_evtstruct(result);
    return NULL;
  }
  error=saf_parse_word(&toparse,'!','|',(void *)result,chans_treat_field,chans_treat_data,TIME);
  if (error!=0) {
    printf("error during parsing time chans\n");
    free_evtstruct(result);
    return NULL;
  }
  error=saf_parse_word(&toparse,'|','|',(void *)result,chans_treat_field,chans_treat_data,SPACE);
  if (error!=0) {
    printf("error during parsing space chans\n");
    free_evtstruct(result);
    return NULL;
  }
  error=saf_parse_word(&toparse,'|','!',(void *)result,chans_treat_field,chans_treat_data,DATA);
  if (error!=0) {
    printf("error during parsing data chans\n");
    free_evtstruct(result);
    return NULL;
  }

  
  return result;
}





