/*
 * Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
 * This file is part of Pyrame.
 *
 * Pyrame is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Pyrame is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
 * */

#include <pyrame.h>

int main() {

  //data parsing
  char *test[]={"!||","!1,2,10|3,4,30|5,6,50","1!||","0,10!1,2|3,4|5,6!7,8|9,10|11,12"};
  char *errors[]={"!1|2","!!1|40|60"};
  int nb_tests=4;
  int nb_errors=2;
  struct event*e;
  struct block *b;

  //channels parsing
  char *test_chans[]={"!t1,t2,t3|s1,s2|d1,d2,d3,d4,d5"};
  int nb_chan_tests=1;
  struct evtstruct *es;
  char *map_chan;
  char *map_event;
 
  int i;

  //chan parsing test
  for(i=0;i<nb_chan_tests;i++) {
    printf("toparse=%s\n",test_chans[i]);
    es=saf_parse_chans(test_chans[i]);
    print_evtstruct(es);
    free_evtstruct(es);
  }
  printf("\n\n");


  //event parsing test
  for(i=0;i<nb_tests;i++) {
    printf("to parse=%s\n",test[i]);
    b=saf_parse_block(test[i]);
    printf("parsed block : \n");
    print_block(b);
    free_block(b);
    printf("\n");
  }
  printf("\n");

  //error detection in parsing test
  for(i=0;i<nb_errors;i++) {
    printf("to parse=%s\n",errors[i]);
    b=saf_parse_block(errors[i]);
    printf("parsed block : \n");
    print_block(b);
    free_block(b);
    printf("\n");
  }
  printf("\n");
  
  //extraction test
  map_chan="hour,minut,z!second|x,y|energy,charge";
  map_event="1,24,12!37|0,1|123,42.3!38|1,3|125,43.6";
  b=saf_parse_block(map_event);
  e=b->events;
  printf("map_chan=%s\n",map_chan);
  while(e!=NULL) {
    print_event(e);
    printf("extracting hour=%s minut=%s second=%s x=%s y=%s z=%s energy=%s charge=%s\n",get_event_field(e,"hour"),get_event_field(e,"minut"),get_event_field(e,"second"),get_event_field(e,"x"),get_event_field(e,"y"),get_event_field(e,"z"),get_event_field(e,"energy"),get_event_field(e,"charge"));
    printf("test bad name toto=%s\n",get_event_field(e,"toto"));
    e=e->next;
  }
  free_block(b);
  
  return 0;
}

  
