/*
Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
This file is part of Pyrame.

Pyrame is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Pyrame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
*/


#include <pyrame.h>

int open_socket(char * host, int port) {
  int ddid;
  struct addrinfo hints={0,AF_UNSPEC,SOCK_STREAM},*res,*rp;
  char *s_port = malloc(((int)floor(log10(abs(port))) + 2)*sizeof(char));
  sprintf(s_port,"%d",port);
  int s = getaddrinfo(host,s_port,&hints,&res);
  if (s != 0) {
    //fprintf(stderr,"bindpyrame : Error getting host by name\n");
    return -1;
  }
  for (rp=res;rp!=NULL;rp=rp->ai_next) {
    ddid=socket(rp->ai_family,rp->ai_socktype,rp->ai_protocol);
    if (connect(ddid,rp->ai_addr,rp->ai_addrlen) != -1) 
      break;
    close(ddid);
  }
  freeaddrinfo(res);
  free(s_port);
  if (rp == NULL) {
    //perror("bindpyrame : Error connecting to socket");
    return -1;
  }
  return ddid;
}

int file_open(char * filename) {
  int ddid;
  ddid=open(filename,O_RDONLY);
  if (ddid<0) {
    perror("open");
    exit(1);
  }
  //printf("ddid=%d\n",ddid);
  return ddid;
}

int iohtoi(char * string) {
  if (string[0]=='0') { //zero, octal or hexa
    if (string[1]==0) //zero
      return 0;
    if (string[1]=='x') //hexa
      return strtol(string,NULL,16);
    else //octal
      return strtol(string,NULL,8);
  } else { //decimal
    return atoi(string);
  }
}

void *str2ptr(char *addr) {

  char *junk;
  unsigned long ptr;
  ptr=strtol(addr,&junk,10);
  if (junk[0]!='\0' || addr[0]=='\0') {
    return NULL;
  }
  return (void *)ptr;
}

char *ptr2str(void *ptr) {
  char *addr=malloc(30);
  sprintf(addr,"%lu",(long int)ptr);
  return addr;
}

char **split(char *src,char sep,int *nb_res) {
  char *tmp=src;
  char **result;
  int nb_elem=1;
  int i,j;
  i=0;
  
  if (src[0]==0)
    return 0;

  while(tmp[i]!=0) {
    if (tmp[i]==sep)
      nb_elem++;
    i++;
  }
  result=malloc(nb_elem*sizeof(char *));
  *nb_res=nb_elem;

  tmp=src;
  j=0;
  i=0;
  while(tmp[i]!=0) {
    if (tmp[i]==sep) {
      tmp[i]=0;
      result[j]=strdup(tmp);
      tmp=tmp+i+1;
      i=0;
      j++;
    }
    i++;
  }
  result[j]=strdup(tmp);
  return result;
}

void free_split(char **split,int nb_elem) {
  int i;
  for(i=0;i<nb_elem;i++) {
    free(split[i]);
  }
  free(split);
}

