#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# 
# Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
# This file is part of Pyrame.
# 
# Pyrame is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyrame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyrame.  If not, see <http://www.gnu.org/licenses/>

import socket,time,json,threading,Queue,pools,bindpyrame,penv,random

socket_off=-1
log_pool=None
env=penv.penv()

def init():
    global log_pool,log_queue
    env["init_time"]=-1.0
    env["initialized"]=0
    env["block_counter"]=0
    env["stop_flag"]=0
    env["paused"]=0
    env["varstr"]=""
    env["chain_started"]=0
    log_queue=Queue.Queue(1000)
    log_logger_t=threading.Thread(target=logger_thread,args=(log_queue,))
    log_logger_t.start()
    #start a dispatcher
    retcode,res=submod_init_dispatcher(10000,"logger","!time,realtime||%s"%(getnames_logger("logger")[1]),"%d"%submod_getport("LOGGER_PORT"))
    if retcode==0:
        print("error : cant init dispatcher : %s"%(res))
        submod_exit(1)
        
def init_logger(logger_id,datadir,storage_id="undef",undef_pattern="-"):
    """initialize the logger"""
    global log_pool,socket_off

    env["init_time"]=time.time()
    env["datadir"]=datadir
    env["undef_pattern"]=undef_pattern
    log_pool=pools.pool("log")

    #register an acquisition unit for offline data and get stream id
    retcode,res=submod_execcmd("init@acq","datalog","1","/opt/pyrame/tcps_acq.so","/opt/pyrame/uncap_dummy.so","54321","null","null","1","0",datadir,"/opt/pyrame/mv.py")
    if retcode==0:
        return 0,"cant register the offline acqunit <- %s"%(res)


    #open data socket with acquisition chain
    #socket_off=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
    #socket_off.connect(("localhost",54321))

    # get storage ip if present
    env["storage_ip"]="undef"
    env["storage_id"]=storage_id
    if storage_id!="undef":
        retcode,res=submod_execcmd("get_ip@cmod",storage_id)
        if retcode==0:
            env["storage_ip"]="localhost"
        env["storage_ip"]=res

    env["initialized"]=1
    return 1,"ok"

def deinit_logger(logger_id):
    """deinitialize the logger"""
    global log_pool,socket_off
    log_pool=None

    env["stop_flag"]=0
    if socket_off!=-1:
        socket_off.close()
    retcode,res=submod_execcmd("deinit@acq","datalog")
    if retcode==0:
        return 0,"cant deinit acquisition unit <- %s"%(res)

    #wait for socket to close
    time.sleep(1)

    submod_flush_dispatcher()
    env["block_counter"]=0
    env["varstr"]=""
    env["initialized"]=0
    return 1,"ok"

def addvar_logger(logger_id,var_name,device,host,port,periodicity,persistence,command,*params):
    """add a variable to log"""
    global log_pool

    #check initialization
    if env["initialized"]==0:
        return 0,"not initialized"

    if len(params)==0:
        sparams="void"
    else:
        sparams=",".join(params)
    #add the variable in the pool
    log_pool.new(var_name,{"name":var_name,"device":device,"host":host,"port":port,"periodicity":float(periodicity),"persistence":persistence,"last_value":"undef","command":command,"params":sparams,"next_time":0.0,"active":0})
    env["chain_started"]=0

    #add the new variable to the variable name list
    if env["varstr"]=="":
        env["varstr"]=var_name
    else:
        env["varstr"]+=",%s"%(var_name)

    #update the dispatcher scheme
    retcode,res=submod_set_chans("!time||%s"%(getnames_logger(logger_id)[1]))
    if retcode==0:
        return 0,"cant update dispatcher schema: %s"%(res)

    return 1,"ok"

def getnames_logger(logger_id):
    return 1,env["varstr"]

def set_period_logger(logger_id,var_name,periodicity):
    global log_pool

    #check initialization
    if env["initialized"]==0:
        return 0,"not initialized"
   
    #check variable exists
    try:
        var=log_pool.get(var_name)
    except Exception as e:
        return 0,"unknown variable %s"%(var_name)

    #set periodicity
    var["periodicity"]=float(periodicity)

    return 1,"ok"

def set_command_logger(logger_id,var_name,func,*params):
    global log_pool

    #check initialization
    if env["initialized"]==0:
        return 0,"not initialized"
   
    #check variable exists
    try:
        var=log_pool.get(var_name)
    except Exception as e:
        return 0,"unknown variable %s"%(var_name)

    var["command"]=func
    if params=="void":
        var["params"]="void"
    else:
        var["params"]=",".join(params)

    return 1,"ok"

def init_logentry(logentry_id,device,ip,port,command,params,periodicity,persistence):
    """convert a logentry from xml file to a variable in the logger system"""

    #if device is not undef then get ip/port from cmod
    if device!="undef":
        retcode,res=submod_execcmd("get_ip@cmod",device)
        if retcode==0:
            return 0,"cant find ip in cmod <- %s"%(res)
        ip=res
        retcode,res=submod_execcmd("get_type@cmod",device)
        if retcode==0:
            return 0,"cant find type in cmod <- %s"%(res)
        nport=submod_getport(res.upper()+"_PORT")
    else:
        try:
            nport=int(port)
        except:
            nport=submod_getport(port)
    if params=="void":
        aparams=[]
    else:
        aparams=params.split(",")
    addvar_logger("logger",logentry_id,device,ip,nport,periodicity,persistence,command,*aparams)
    return 1,"ok"
    
def set_values_logger(logger_id,json_data):
    """Logs a JSON-encoded array of dictionnaries, *json_data*.
    Each of the dictionnaries must contain a time key and each will correspond to a log entry. Each of the other keys in the dictionnary will be used if they match a registered logger variable.
    Example: json_data = [{"time": 1.2, "temp":23.5, "voltage":9.9},{"time": 1.8, "temp":23.5}]"""
    global socket_off,log_pool

    # allow to receive directly a dictionnary if function used internally
    if isinstance(json_data,str):
        data=json.loads(json_data)
    elif not isinstance(json_data,list):
        return 0,"bad format of json_data. must be JSON array of dictionnaries"
    else:
        data=json_data

    for entry in data:
        data_off=[]
        retcode,res=submod_new_block(env["block_counter"])
        if retcode==0:
            return 0,"unable to create a new block: %s"%(res)
        else:
            blockid=res
            env["block_counter"]+=1
            
        retcode,res=submod_new_event(blockid)
        if retcode==0:
            return 0,"unable to create a new event: %s"%(res)
        else:
            evtid=res

        #set the time field
        submod_set_field_event(evtid,"time",entry["time"])
        submod_set_field_event(evtid,"realtime",entry["realtime"])

        for vs in env["varstr"].split(","):
            v=log_pool.get(vs)
            k=v["name"]
            if k in entry:
                if entry[k]!="undef":
                    v["last_value"]=entry[k]
                datum=entry[k]
            elif v["persistence"] in ["1","on","true","yes"]:
                datum=v["last_value"]
            else:
                datum="undef"
            if datum=="undef":
                #print("setting field %s with u"%(k))
                retcode,res=submod_set_field_event(evtid,k,"u")
                data_off.append(env["undef_pattern"])
            else:
                #print("setting field %s with data %s"%(k,datum))
                retcode,res=submod_set_field_event(evtid,k,datum)
                if retcode==0:
                    print("warning : cant set field: %s"%(res))
                data_off.append(datum)

        # finalize block
        retcode,res=submod_finalize_block(blockid)
        if retcode==0:
            return 0,"unable to finalize block : %s"%res

        # send offline data to acq_chain
        data_off_str="%s %s %s\n"%(entry["time"],entry["realtime"]," ".join(data_off))
        try:
            socket_off.sendall(data_off_str)
        except:
            try:
                socket_off=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
                socket_off.connect(("localhost",54321))
                socket_off.sendall(data_off_str)
            except:
                print("warning: error sending data to acq chain")

    return 1,"ok"

def set_value_logger(logger_id,time,var_name,var_value):
    "Log with *time* variable *var_name*, with value *var_value*"
    realtime="%f"%(float(time)+env["init_time"])
    return set_values_logger(logger_id,[{"time":time,"realtime":realtime,var_name:var_value}])

def log_var(data,rttag,ttag,v,async,async_p):
    if v["device"]!="undef":
        params=[v["device"]]
    else:
        params=[]
    if v["params"]!="void":
        params+=v["params"].split(",")
    retcode,res=bindpyrame.sendcmd(v["host"],int(v["port"]),v["command"],*params)
    #print("command %s(%s) -> %d,%s"%(v["command"],v["params"],retcode,res))
    if retcode==0:
        val="undef"
        print("warning: cant read value for %s <- %s"%(v["name"],res))
    else:
        val=res
    # detect JSON result
    if val[0] in ["[","{"]:
        # it is JSON
        json_val=json.loads(val)
        if isinstance(json_val,list):
            #list of dictionnaries
            # convert dict from time,value to time,var_name
            for jv in json_val:
                if "time" not in jv:
                    print("warning: time absent from JSON value %s"%(jv))
                    continue
                if "value" not in jv:
                    print("warning: value absent from JSON value %s"%(jv))
                    continue
                dtime=float(jv["time"])+ttag
                rtime=float(jv["time"])+rttag
                data.append({"time":"%f"%(dtime),"realtime":"%f"%(rtime),v["name"]:jv["value"]})
        else:
            print("warning: invalid JSON response: not a list")
    else:
        # only one value, so add it to the dictionnary
        # that already has time=ttag
        for d in data:
            if d["time"]=="%f"%(ttag):
                #now=time.time()-env["init_time"]
                #elapsed=now-ttag
                #print("%s started at %.3f ended at %.3f, elapsed %.3f"%(v["name"],ttag,now,elapsed))
                d[v["name"]]=val
                break
    async_p["async"]-=1
    v["active"]=0
    if async and async_p["async"]==0:
        #print("calling set_values_loger with ",data)
        set_values_logger("logger",data)

def log_vars(variables,async):
    global log_pool
    print("log_vars: %s"%(",".join(variables)))
    rttag=time.time()
    ttag=rttag-env["init_time"]
    async_c=0
    for _,v in log_pool.get_all():
        if (variables==() or (v["name"] in variables)) and v["command"]!="undef":
            async_c+=1
    # use dictionnary to keep pointer to counter
    async_p={"async":async_c}
    data=[{"time":"%f"%(ttag),"realtime":"%f"%(rttag)}]
    for _,v in log_pool.get_all():
        if (variables==() or (v["name"] in variables)) and v["command"]!="undef":
            t=threading.Thread(target=log_var,args=(data,rttag,ttag,v,async,async_p))
            t.start()
    if async==0:
        while async_p["async"]!=0:
            time.sleep(0.01)
        #print("logger_thread:  data",data)
        return set_values_logger("logger",data)

def logger_thread(queue):
    while True:
        variables=queue.get(True,None)
        log_vars(variables,1)

def log_logger(logger_id,a_sync,*variables):
    "Log variables whose name is an argument. If no argument is provided, log all variables."
    global log_queue

    #check initialization
    if env["initialized"]==0:
        return 0,"not initialized"

    #start acquisition if necessary
    if env["chain_started"]==0:
        retcode,res=submod_execcmd("start@acq","datalog")
        if retcode==0:
            return 0,"cant start acquisition <- %s"%(res)
        env["chain_started"]=1
        #open data socket with acquisition chain
        socket_off=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
        socket_off.connect(("localhost",54321))
    

    if sbool(a_sync):
        try:
            log_queue.put(variables,False)
        except Exception as e:
            print("warning: logger queue full. discarding request: %s"%(e))
            pass
    else:
        return log_vars(variables,0)
    return 1,"ok"

def scheduler_logger():
    global log_pool

    if log_pool==None:
        return 1,"ok"

    if env["stop_flag"]==1:
        env["stop_flag"]=0
        return 1,"ok"
    var_names=[]
    now=time.time()-env["init_time"]
    #print("scheduler_logger")

    # log vars with next_time close to now
    #print("looking for vars past or close to now %f"%(now))
    for _,v in log_pool.get_all():
        nt=v["next_time"]
        #print("  next_time[%s]=%s"%(v["name"],v["next_time"]))
        if nt<=now+0.0005 and v["periodicity"]!=-1.0:
            if env["paused"]==0:
                if v["active"]==0:
                    var_names.append(v["name"])
                    v["active"]=1
                else:
                    print("warning: variable %s is too slow"%(v["name"]))
            #print("updating next_time for %s"%(v["name"]))
            nt+=v["periodicity"]
            v["next_time"]=nt

    #logging if necessary
    #print("logging %s"%(", ".join(var_names)))
    if var_names!=[]:
        log_logger("logger","true",*var_names)


    # find minimum next_time after updating
    #print("determining next_time:")
    min_next_time=None
    for _,v in log_pool.get_all():
        #print("  next_time[%s]=%s"%(v["name"],v["next_time"]))
        if v["periodicity"]!=-1.0 and (min_next_time==None or v["next_time"]<min_next_time):
            min_next_time=v["next_time"]


    #submod_wakemeup if necessary
    if min_next_time==None:
        # no periodic variables! we're done
        return 1,"ok"
    time_to_next=min_next_time-(time.time()-env["init_time"])
    print("resubmitting scheduler_logger in %f s"%(time_to_next))
    if time_to_next<0:
        time_to_next=0
    retcode,res=submod_wakemeup(int(time_to_next*1e6),"scheduler_logger","none")
    if retcode==0:
        return 0,"error resubmitting task <- %s"%(res)
    return 1,"ok"

def start_acq_logger(logger_id,acq_id,acq_mp,acq_path,acq_name):
    "Start the logger in automatic mode for variables with periodicity!=-1. To log the latter, either call the function log_logger or set_value_logger."
    global log_pool,socket_off

    #check initialization
    if env["initialized"]==0:
        return 0,"not initialized"

    env["stop_flag"]=0
    env["acq_id"]=acq_id
    env["acq_mp"]=acq_mp
    env["acq_path"]=acq_path
    env["acq_name"]=acq_name
    
    for _,v in log_pool.get_all():
        v["next_time"]=0.0

    env["init_time"]=time.time()

    #flush acquisition if necessary
    if env["chain_started"]==1:
        if socket_off!=-1:
            socket_off.close()
        retcode,res=submod_execcmd("stop@acq","datalog")
        if retcode==0:
            return 0,"cant stop acquisition <- %s"%(res)

        flush_prefix="%x"%(random.getrandbits(16))
        retcode,res=submod_execcmd("flush_files@acq","datalog",flush_prefix,"file:///","/tmp/","crap","undef","undef","undef")
        if retcode==0:
            return 0,"cant flush <- %s"%(res)


    #start acquisition
    retcode,res=submod_execcmd("start@acq","datalog")
    if retcode==0:
        return 0,"cant start acquisition <- %s"%(res)
    env["chain_started"]=1
    #open data socket with acquisition chain
    socket_off=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
    socket_off.connect(("localhost",54321))

    return scheduler_logger()

def stop_acq_logger(logger_id):

    #check initialization
    if env["initialized"]==0:
        return 0,"not initialized"
    
    env["stop_flag"]=1

    #stop acquisition and flush acquisition files
    if socket_off!=-1:
        socket_off.close()
    retcode,res=submod_execcmd("stop@acq","datalog")
    if retcode==0:
        return 0,"cant stop acquisition <- %s"%(res)
    env["chain_started"]==0

    flush_prefix="%x"%(random.getrandbits(16))
    retcode,res=submod_execcmd("flush_files@acq","datalog",flush_prefix,env["acq_mp"],env["acq_path"],env["acq_name"],env["acq_id"],env["storage_ip"],env["storage_id"])
    if retcode==0:
        return 0,"cant flush <- %s"%(res)

    return 1,"ok"

def pause_logger(logger_id):
    env["paused"]=1
    return 1,"ok"

def resume_logger(logger_id):
    env["paused"]=0
    return 1,"ok"

def get_init_time_logger(logger_id):
    #check initialization
    if env["initialized"]==0:
        return 0,"not initialized"
   
    return 1,str(env["init_time"])

def flush_dispatcher_logger(logger_id):
    return submod_flush_dispatcher()

def get_last_value_logger(logger_id,var_name):
    try:
        v=log_pool.get(var_name)
    except:
        return 0,"no variable with name %s"%(var_name)
    return 1,v["last_value"]
