#!/bin/sh
# -*- coding: utf-8 -*-
# 
# Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
# This file is part of Pyrame.
. /usr/local/bin/unit_test_lib.sh
. /opt/pyrame/ports.sh

clean_all

echo
echo
echo "DATA LOGGER TESTS"
echo
autolauncher.py -n > server.trace 2>&1 &
wait_until_awake localhost $LOGGER_PORT
wait_until_awake localhost $ACQ_PORT
wait_until_awake localhost $VARMOD_PORT
wait_until_awake localhost $MOUNTD_PORT
exec_n_test ../varmod/setvar.sh varmod default x 2
exec_n_test ../varmod/setvar.sh varmod default y 3

# persistence
mkdir -p /tmp/logger_unit_test
exec_n_test ./init.sh /tmp/logger_unit_test undef "*"
exec_n_test ./get_init_time.sh
INIT_TIME=${RETSTR%.*}
NOW=`expr $(date +%s) - $INIT_TIME`
if test $NOW -gt 5; then
    echo "error getting init_time"
    exit 1
fi
exec_n_test_fail ./get_last_value.sh x
exec_n_test ./addvar.sh x undef localhost $VARMOD_PORT 1 true getvar_varmod varmod default x
exec_n_test ./addvar.sh y undef localhost $VARMOD_PORT -1 false getvar_varmod varmod default y
checkretstr undef ./get_last_value.sh x
exec_n_test ./log.sh true
exec_n_test ./log.sh true y
checkretstr 2 ./get_last_value.sh x
NOW=`expr $(date +%s) - $INIT_TIME`
exec_n_test ./set_value.sh $NOW "y" 12
sleep 1s
NOW=`expr $(date +%s) - $INIT_TIME`
exec_n_test ./set_value.sh $NOW "x" 24
checkretstr 24 ./get_last_value.sh x
checkretstr "0,1,2,3" ./get_block_list.sh -1
checkretstr "2,3" ./get_data.sh 0
checkretstr "2,3" ./get_data.sh 1
checkretstr "2,12" ./get_data.sh 2
checkretstr "24,u" ./get_data.sh 3
exec_n_test ./startacq.sh "undef" "file:///" "tmp" "utlogger"
echo "waiting for 10s..."
sleep 10s
exec_n_test ./stopacq.sh
sleep 1s
check_file_size 311 /tmp/utlogger_datalog_0.raw
exec_n_test ./deinit.sh

# peridicity 1,2
exec_n_test ./init.sh  /tmp/logger_unit_test
exec_n_test ./addvar.sh x undef localhost $VARMOD_PORT 1 false getvar_varmod varmod default x
exec_n_test ./addvar.sh y undef localhost $VARMOD_PORT 2 false getvar_varmod varmod default y
exec_n_test ./log.sh true
checkretstr "0" ./get_block_list.sh -1
checkretstr "2,3" ./get_data.sh 0 
exec_n_test ./startacq.sh "undef" "file:///" "tmp" "utlogger"
echo "waiting for 10s..."
sleep 10s
exec_n_test ./stopacq.sh
sleep 1s
check_file_size 311 /tmp/utlogger_datalog_0.raw
exec_n_test ./deinit.sh

# periodicity -1
exec_n_test_fail ./addvar.sh x undef localhost $VARMOD_PORT 1 false getvar_varmod varmod default x
exec_n_test_fail ./log.sh true
exec_n_test ./init.sh  /tmp/logger_unit_test
exec_n_test ./addvar.sh x undef localhost $VARMOD_PORT -1 false getvar_varmod varmod default x
exec_n_test ./addvar.sh y undef localhost $VARMOD_PORT -1 false getvar_varmod varmod default y
exec_n_test ./log.sh true
checkretstr "0" ./get_block_list.sh -1
checkretstr "2,3" ./get_data.sh 0 
exec_n_test ./startacq.sh "undef" "file:///" "tmp" "utlogger"
echo "waiting for 10s..."
sleep 10s
exec_n_test ./stopacq.sh
sleep 1s
check_file_size 0 /tmp/utlogger_datalog_0.raw
exec_n_test ./deinit.sh

clean_all
echo "all test passed"
