#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# 
# Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
# This file is part of Pyrame.
# 
# Pyrame is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyrame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyrame.  If not, see <http://www.gnu.org/licenses/>

import pools,conf_strings

# KI_707B #########################################################

class ki_707b(object):
    def __init__(self):
        self.ki_707b_pool=pools.pool()

    class ki_707b_Exception(Exception):
        pass

    def init(self,ki_707b_id,conf_string):
        try:
            conf=conf_strings.parse(conf_string)
        except Exception as e:
            return 0,str(e)
        if conf.name!="ki_707b":
            return 0,"Invalid module name %s in conf_string instead of ki_707b"%(conf.name)
        if not conf.has("bus"):
            return 0,"Error: some of the required parameters (bus) in conf_string are not present"
        try:
            conf_bus=conf_strings.parse(conf.params["bus"])
        except Exception as e:
            return 0,str(e)
        if conf_bus.name=="serial":
            if not conf_bus.has("baudrate"):
                conf_bus.params["baudrate"]="9600"
            conf_bus.params["parity"]="O"
            conf_bus.params["bytesize"]="7"
        if conf_bus.name=="tcp" and not conf_bus.has("port"):
            conf_bus.params["port"]="5025"
        bus_id="bus_%s"%(ki_707b_id)
        retcode,res=submod_execcmd("init@"+conf_bus.name,bus_id,conf_strings.unparse(conf_bus))
        if retcode==0:
            return 0,"Error initializing link <- %s"%(res)
        self.ki_707b_pool.new(ki_707b_id,{"bus":conf_bus.name,"bus_id":bus_id})
        return 1,"ok"

    def deinit(self,ki_707b_id):
        try:
            ki_707b=self.ki_707b_pool.get(ki_707b_id)
        except Exception as e:
            return 1,str(e)
        retcode,res=submod_execcmd("deinit@"+ki_707b["bus"],ki_707b["bus_id"])
        if retcode==0:
            return 0,"Error deinitializing link <- %s"%(res)
        self.ki_707b_pool.remove(ki_707b_id)
        return 1,"ok"

    def config(self,ki_707b_id):
        try:
            ki_707b=self.ki_707b_pool.get(ki_707b_id)
        except Exception as e:
            return 0,str(e)
        if "configured" in ki_707b:
            return 1,"already configured"
        retcode,res=submod_execcmd("config@"+ki_707b["bus"],ki_707b["bus_id"])
        if retcode==0:
            return 0,"Error configuring link <- %s"%(res)
        ki_707b["configured"]=True
        return 1,"ok"

    def inval(self,ki_707b_id):
        try:
            ki_707b=self.ki_707b_pool.get(ki_707b_id)
        except Exception as e:
            return 0,str(e)
        if not "configured" in ki_707b:
            return 1,"not configured"
        # Invalidate bus
        retcode,res=submod_execcmd("inval@"+ki_707b["bus"],ki_707b["bus_id"])
        if retcode==0:
            return 0,"Error invalidating link <- %s"%(res)
        # Remove parameters set during config
        del ki_707b["configured"]
        return 1,"ok"

    def actuate(self,ki_707b_id,state,slot,bank,row,column):
        try:
            ki_707b=self.ki_707b_pool.get(ki_707b_id)
        except Exception as e:
            return 0,str(e)
        if not "configured" in ki_707b:
            return 0,"not configured"
        cmd="channel."+state
#        # slot is one char decimal integer
#        try:
#            s=int(slot)
#        except:
#            return 0,"invalid slot. must be decimal integer"
#        if s not in range(1,7):
#            return 0,"invalid slot. must be from 1 to 6"
#        # if no bank, leave empty
        if bank=="none":
            bank=""
#        try:
#            b=int(bank)
#        except:
#            b=""
#        # check that row in a-z or in 1 to 8 ranges
#        if row.lower() not in map(chr,range(97,123)) and row not in map(str,range(1,9)):
#            return 0,"invalid row. must be a to z or 1 to 8"
        # column always must have 2 chars
        column="%02d"%(int(column))
        c="%x%s"%(int(column[0:-1]),column[-1:])
        if len(c)!=2:
            return 0,"invalid column. must be a decimal integer < 159"
        # build cmd
        cmd+="('%s%s%s%s')"%(slot,bank,row,c)
        retcode,res=submod_execcmd("write@"+ki_707b["bus"],ki_707b["bus_id"],cmd+r"\r\n")
        if retcode==0:
            return 0,"error sending command <- %s"%(res)
        return 1,"ok"

    def free_command(self,ki_707b_id,cmd):
        try:
            ki_707b=self.ki_707b_pool.get(ki_707b_id)
        except Exception as e:
            return 0,str(e)
        if not "configured" in ki_707b:
            return 0,"not configured"
        retcode,res=submod_execcmd("write@"+ki_707b["bus"],ki_707b["bus_id"],cmd+r"\r\n")
        if retcode==0:
            return 0,"error sending command <- %s"%(res)
        return 1,"ok"

# CREATE KI_707B POOL #############################################

me=ki_707b()

# COMMANDS #######################################################

def init_ki_707b(ki_707b_id,conf_string):
    """Initialize KI_707B or KI_708B switch matrix.
    
    conf_string must contain the parameters:

    - bus: a conf_string for cmd_serial or cmd_tcp"""
    return me.init(ki_707b_id,conf_string)
    
def deinit_ki_707b(ki_707b_id):
    "Deinitialize *ki_707b_id* switch matrix."
    return me.deinit(ki_707b_id)

def config_ki_707b(ki_707b_id):
    "Configure *ki_707b_id* switch matrix."
    return me.config(ki_707b_id)

def inval_ki_707b(ki_707b_id):
    "Invalidate *ki_707b_id* switch matrix."
    return me.inval(ki_707b_id)

def open_ki_707b(ki_707b_id,slot,bank,row,column):
    "Close switch from *ki_707b_id* switch matrix. *slot* and *column* are decimal integers. *bank* is either 'none' or a decimal integer depending on the card. *row* is either a decimal integer 1-8 or a letter a-z."
    return me.actuate(ki_707b_id,"open",slot,bank,row,column)

def close_ki_707b(ki_707b_id,slot,bank,row,column):
    "Close switch from *ki_707b_id* switch matrix. *slot* and *column* are decimal integers. *bank* is either 'none' or a decimal integer depending on the card. *row* is either a decimal integer 1-8 or a letter a-z."
    return me.actuate(ki_707b_id,"close",slot,bank,row,column)

def free_command_ki_707b(ki_707b_id,cmd):
    "Send free command"
    return me.free_command(ki_707b_id,cmd)

