#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# 
# Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
# This file is part of Pyrame.
# 
# Pyrame is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyrame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyrame.  If not, see <http://www.gnu.org/licenses/>

import time
import pools,conf_strings

man_motion_pool=pools.pool()

# COMMANDS #######################################################

def init_man_motion(man_motion_id,conf_string="undef"):
    """Initialize MAN_MOTION motion controller.
    
    *conf_string* is ignored"""
    man_motion_pool.new(man_motion_id,{"pos":0})
    return 1,"ok"

def deinit_man_motion(man_motion_id):
    "Deinitialize and deregister man_motion motion controller from the pool."
    try:
        man_motion=man_motion_pool.get(man_motion_id)
    except Exception as e:
        return 1,str(e)
    man_motion_pool.remove(man_motion_id)
    return 1,"ok"

def config_man_motion(man_motion_id,pos_max,pos_min):
    "Configure *man_motion_id* axis. *pos_max* and *pos_min* define the limits of excursion."
    try:
        man_motion=man_motion_pool.get(man_motion_id)
    except Exception as e:
        return 0,str(e)
    pos_min=float(pos_min)
    pos_max=float(pos_max)
    if pos_max<=pos_min:
        return 0,"max must be higher than min"
    man_motion["max"]=pos_max
    man_motion["min"]=pos_min
    return 1,"ok"

def inval_man_motion(man_motion_id):
    "Invalidate *man_motion_id* configuration"
    try:
        man_motion=man_motion_pool.get(man_motion_id)
    except Exception as e:
        return 0,str(e)
    del man_motion["max"]
    del man_motion["min"]
    return 1,"ok"

def home_man_motion(man_motion_id,direction,velocity):
    "Reset axis *man_motion_id* by homing on *direction* at *velocity*."
    try:
        man_motion=man_motion_pool.get(man_motion_id)
    except Exception as e:
        return 0,str(e)
    if "max" not in man_motion:
        return 0,"not configured"
    man_motion["pos"]=0
    return 1,"ok"

def go_min_man_motion(man_motion_id,velocity,acceleration):
    "Go to the minimum position as defined during the initalization of the axis"
    try:
        man_motion=man_motion_pool.get(man_motion_id)
    except Exception as e:
        return 0,str(e)
    if "max" not in man_motion:
        return 0,"not configured"
    man_motion["pos"]=man_motion["min"]
    return 1,"ok"

def go_max_man_motion(man_motion_id,velocity,acceleration):
    "Go to the maximum position as defined during the initalization of the axis"
    try:
        man_motion=man_motion_pool.get(man_motion_id)
    except Exception as e:
        return 0,str(e)
    if "max" not in man_motion:
        return 0,"not configured"
    man_motion["pos"]=man_motion["max"]
    return 1,"ok"

def move_man_motion(man_motion_id,displacement,velocity,acceleration):
    "Move by displacement units with the specified maximum velocity"
    try:
        man_motion=man_motion_pool.get(man_motion_id)
    except exception as e:
        return 0,str(e)
    if "max" not in man_motion:
        return 0,"not configured"
    man_motion["pos"]+=displacement
    return 1,"ok"

def goto_man_motion(man_motion_id,pos,velocity,acceleration):
    try:
        man_motion=man_motion_pool.get(man_motion_id)
    except exception as e:
        return 0,str(e)
    if "max" not in man_motion:
        return 0,"not configured"
    man_motion["pos"]=float(pos)
    return 1,"ok"

def get_pos_man_motion(man_motion_id):
    "Get position of the specified axis"
    try:
        man_motion=man_motion_pool.get(man_motion_id)
    except Exception as e:
        return 0,str(e)
    if "max" not in man_motion:
        return 0,"not configured"
    return 1,str(man_motion["pos"])
