#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# 
# Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
# This file is part of Pyrame.
# 
# Pyrame is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyrame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyrame.  If not, see <http://www.gnu.org/licenses/>

try:
    from __main__ import submod
except:
    pass

def get_type(x):
    if type(x)==str:
        return "0"
    if type(x)==int:
        return "1"
    if type(x)==float:
        return "2"
    return "-1"

class pool_item(dict):

    def __init__(self,name,args):
        self.name=name
        self.already_del=False
        #print("pool_item init",name,args)
        retcode,res=submod.callwrap("newns",name)
        if retcode==0:
            raise Exception("unable to newns %s: %s"%(name,res))
        self.update(args)

    def __del__(self):
        #print("del pool_item / delns")
        if not self.already_del:
            self.already_del=True
            retcode,res=submod.callwrap("delns",self.name)
            if retcode==0:
                raise Exception("unable to delns: %s"%(res))

    def __setitem__(self,key,value):
        vtype=get_type(value)
        if vtype==-1:
            raise Exception("unsupported type: %s"%(type(value)))
        retcode,res=submod.callwrap("setvar",self.name,key,str(value),vtype)
        if retcode==0:
            raise Exception("unable to setvar: %s"%(res))
        return

    def __getitem__(self,key):
        retcode,res=submod.callwrap("getvar",self.name,key)
        if retcode==0:
            raise Exception("unable to getvar: %s"%(res))
        v=res
        retcode,res=submod.callwrap("gettype",self.name,key)
        vtype=res
        #print("reading %s : res=%s type=%s value=%s"%(key,res,vtype,v))
        if vtype=="0":
            return v
        if vtype=="1":
            return int(v)
        if vtype=="2":
            return float(v)
        raise Exception("unknown type %s"%(vtype))
 
    def __delitem__(self,id):
        #print("delitem")
        retcode,res=submod.callwrap("delvar",self.name,id)
        if retcode==0:
            raise Exception("unable to delvar: %s"%(res))
        return 

    def __contains__(self,key):
        retcode,res=submod.callwrap("getvar",self.name,key)
        if retcode==0:
            return False
        return True
  
    def get(self,key,default=None):
        return self[key] if key in self else default

    def update(self,moredict):
        #print(moredict)
        if type(moredict)!=dict:
            raise Exception("require dict of moredict : unsupported argument type %s"%(type(moredict)))
        # try so that if submod is not available (gen_api) we ignore it
        try:
            for k in moredict:
                value=moredict[k]
                #print("k=%s value_type=%s value=%s"%(k,type(value),str(value)))
                retcode,res=submod.callwrap("setvar",self.name,k,str(value),get_type(value))
                if retcode==0:
                    raise Exception("unable to setvar: %s"%(res))
        except:
            pass

    def clear(self):
        for k in self.keys():
            del self[k]

    def setdefault(self,key,value=None):
        if key not in self:
            self[key]=value
        return self[key]

class pool:
    def __init__(self,name="undef"):
        self.name=name
        self.elements={}
        # try so that if submod is not available (gen_api) we ignore it
        try:
            _,avail_ns=submod.callwrap("listns")
            if avail_ns!="":
                nsl=avail_ns.split(",")
                for ns in nsl:
                    pi=pool_item(ns,{})
                    self.elements[ns]=pi
        except:
            pass

    def new(self,id,params={}):
        id=str(id)
        if id in self.elements:
            raise Exception("id %s already existant in the pool"%(id))
        pi=pool_item(id,params)
        self.elements[id]=pi
        return pi

    def get(self,id):
        id=str(id)
        # Look for id in pool
        if id not in self.elements:
            raise Exception("id %s has not been found in the pool" % (id))
        return self.elements[id]

    def get_all(self):
        return self.elements.iteritems()

    def remove(self,id):
        id=str(id)
        if id not in self.elements:
            raise Exception("id %s has not been found in the pool" % (id))
        todel=self.elements[id]
        self.elements.pop(id)
        todel.__del__()

    def length(self):
        return len(self.elements)

    def find_by_field(self,key,value):
        for k in self.elements.keys():
            if self.elements[k][key]==value:
                return k,self.elements[k][key]
        return -1,None

    def display(self):
        for k in self.elements.keys():
            print("id=%s"%(k))
            print(self.elements[k])
