#!/bin/bash
# -*- coding: utf-8 -*-
# 
# Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
# This file is part of Pyrame.
. /usr/local/bin/unit_test_lib.sh
. /opt/pyrame/ports.sh

clean_all

echo
echo
echo "CA_SY527 TESTS"
echo
stdbuf -oL -eL cmdmod /opt/pyrame/cmd_ps.xml > ps.trace 2>&1 &
stdbuf -oL -eL cmdmod /opt/pyrame/cmd_ca_sy527.xml > ca_sy527.trace 2>&1 &
stdbuf -oL -eL cmdmod /opt/pyrame/cmd_serial.xml > serial.trace 2>&1 &
stdbuf -oL -eL cmdmod /opt/pyrame/cmd_tcp.xml > tcp.trace 2>&1 &
wait_until_awake localhost $PS_PORT
wait_until_awake localhost $CA_SY527_PORT
wait_until_awake localhost $SERIAL_PORT
wait_until_awake localhost $TCP_PORT

exec_n_test ../cmd_ps/init.sh "ca_sy527(bus=serial(vendor=0557,product=2008))"
exec_n_test ../cmd_ps/config.sh 0
for i in 0 10 15; do
  ref=`expr 10 + $i`
  echo "checking channel 0/$i with voltage $ref :"
  exec_n_test ../cmd_ps/set_current_limit.sh 0 100e-6 0/$i
  exec_n_test ../cmd_ps/set_voltage.sh 0 $ref 0/$i
  exec_n_test ../cmd_ps/power_on.sh 0 0/$i
  sleep 2s
  exec_n_test ../cmd_ps/get_voltage.sh 0 0/$i
  dv=`echo "scale=2; ($ref - $RETSTR)/$ref" | bc`
  if test `echo "${dv#-} > 0.2" | bc` -eq 1; then
    echo "read voltage $RETSTR differs too much from set value $ref"
    exit 1
  fi
  exec_n_test ../cmd_ps/power_off.sh 0 0/$i
  echo "checking channel 2/$i with voltage $ref :"
  exec_n_test ../cmd_ps/set_current_limit.sh 0 100e-6 2/$i
  exec_n_test ../cmd_ps/set_voltage.sh 0 $ref 2/$i
  exec_n_test ../cmd_ps/power_on.sh 0 2/$i
  sleep 2s
  exec_n_test ../cmd_ps/get_voltage.sh 0 2/$i
  dv=`echo "scale=2; ($ref - $RETSTR)/$ref" | bc`
  if test `echo "${dv#-} > 0.2" | bc` -eq 1; then
    echo "read voltage $RETSTR differs too much from set value $ref"
    exit 1
  fi
  exec_n_test ../cmd_ps/power_off.sh 0 2/$i
done
for i in 0 10 15; do
  ref=1${i}e-6
  echo "checking channel 0/$i with current $ref :"
  exec_n_test ../cmd_ps/set_voltage_limit.sh 0 100 0/$i
  exec_n_test ../cmd_ps/set_current.sh 0 $ref 0/$i
  checkretstr 0.0 ../cmd_ps/get_current.sh 0 0/$i
  echo "checking channel 2/$i with current $ref :"
  exec_n_test ../cmd_ps/set_voltage_limit.sh 0 100 2/$i
  exec_n_test ../cmd_ps/set_current.sh 0 $ref 2/$i
  checkretstr 0.0 ../cmd_ps/get_current.sh 0 2/$i
done
exec_n_test ../cmd_ps/inval.sh 0
exec_n_test ../cmd_ps/deinit.sh 0
exec_n_test ../cmd_ps/exit.sh
exec_n_test ./exit.sh

clean_all
echo "all tests passed"
