#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# 
# Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
# This file is part of Pyrame.
# 
# Pyrame is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyrame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyrame.  If not, see <http://www.gnu.org/licenses/>

import sys,os,time
import bindpyrame

######################################################

if len(sys.argv)<2:
    print ("usage: %s conf_string_ps"%(sys.argv[0]))
    sys.exit(1)

conf_string_ps=sys.argv[1]

# IdV parameters (base units of the SI)
v_start=0 # v_start must be < than v_end
v_end=32
v_step=1
stabilization_time=0.5
current_limit=100e-3
channel="1"

######################################################
# disable stdout buffering
sys.stdout=os.fdopen(sys.stdout.fileno(),"w",0)
######################################################

# Helper function
def sendcmd(*params):
    retcode,res=bindpyrame.sendcmd("localhost",*params)
    if res.endswith("compliance"):
        return "compliance"
    if retcode==0:
        print(res)
        sys.exit(1)
    return res

# Get ports
ps_port=bindpyrame.get_port("PS_PORT")

# Initialization of module PS
ps_id=sendcmd(ps_port,"init_ps",conf_string_ps)

# Configure modules
sendcmd(ps_port,"config_ps",ps_id,"careful")

# Set current limit on PS
sendcmd(ps_port,"set_current_limit_ps",ps_id,str(current_limit),channel)

# Set starting voltage on PS
sendcmd(ps_port,"set_voltage_ps",ps_id,str(v_start),channel)

# Power on PS
sendcmd(ps_port,"power_on_ps",ps_id,channel)

time.sleep(2*stabilization_time)

# Main loop
print("# voltage(V)   current(A)")
for v in range(int(v_start/v_step),int(v_end/v_step)+1):
    v=v*v_step
    sendcmd(ps_port,"set_voltage_ps",ps_id,str(v),channel)
    time.sleep(stabilization_time)
    res=sendcmd(ps_port,"get_voltage_ps",ps_id,channel)
    if res=="compliance":
        v="-"
        supp=res
    else:
        v="%.6e"%(float(res))
        supp=""
    res=sendcmd(ps_port,"get_current_ps",ps_id,channel)
    i="%.6e"%(float(res))
    print("%s %s %s"%(v,i,supp))

# Power off PS
sendcmd(ps_port,"power_off_ps",ps_id,channel)

# Deinitialize modules
sendcmd(ps_port,"deinit_ps",ps_id)

