#!/bin/bash
# -*- coding: utf-8 -*-
# 
# Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
# This file is part of Pyrame.
. /usr/local/bin/unit_test_lib.sh
. /opt/pyrame/ports.sh

IP=10.220.0.39

clean_all
sleep 1s
stdbuf -oL -eL cmdmod /opt/pyrame/cmd_ps.xml > ps.trace 2>&1 &
stdbuf -oL -eL cmdmod /opt/pyrame/cmd_ha_hmp4030.xml > ha_hmp4030.trace 2>&1 &
stdbuf -oL -eL cmdmod /opt/pyrame/cmd_tcp.xml > tcp.trace 2>&1 &
wait_until_awake localhost $PS_PORT
wait_until_awake localhost $HA_HMP4030_PORT
wait_until_awake localhost $TCP_PORT

exec_n_test ../cmd_ps/init.sh "ha_hmp4030(bus=tcp(host=$IP,port=5025))"
for mode in fast debug; do
  echo "MODE $mode"
  exec_n_test ../cmd_ps/config.sh 0 $mode
  for i in `seq 10`; do
    exec_n_test ../cmd_ps/set_current_limit.sh 0 1 1
  done
  exec_n_test ../cmd_ps/free_query.sh 0 "SYST:ERR?"
  if test "a$RETSTR" != "a0, \"No error\""; then
    echo $RETSTR
    exit 1
  fi
  for i in `seq 10`; do
    for c in `seq 1 3`; do
      exec_n_test ../cmd_ps/set_current_limit.sh 0 1 $c
    done
  done
  exec_n_test ../cmd_ps/free_query.sh 0 "SYST:ERR?"
  if test "a$RETSTR" != "a0, \"No error\""; then
    echo $RETSTR
    exit 1
  fi
  for i in `seq 10`; do
    for c in `seq 1 3`; do
      exec_n_test ../cmd_ps/set_voltage.sh 0 10 $c
    done
  done
  exec_n_test ../cmd_ps/free_query.sh 0 "SYST:ERR?"
  if test "a$RETSTR" != "a0, \"No error\""; then
    echo $RETSTR
    exit 1
  fi
  for i in `seq 10`; do
    for c in `seq 1 3`; do
      exec_n_test ../cmd_ps/power_on.sh 0 $c
      exec_n_test ../cmd_ps/power_off.sh 0 $c
    done
  done
  exec_n_test ../cmd_ps/free_query.sh 0 "SYST:ERR?"
  if test "a$RETSTR" != "a0, \"No error\""; then
    echo $RETSTR
    exit 1
  fi
  exec_n_test ../cmd_ps/inval.sh 0
done
exec_n_test ../cmd_ps/deinit.sh 0

./unit_test_eth.py "ha_hmp4030(bus=tcp(host=$IP,port=5025))" > test.raw
gnuplot -e "filename='test.raw';slope=1/10e3;error=0.1" unit_test.gnuplot > gnuplot.trace 2>&1 
check_empty_file gnuplot.trace


clean_all
echo "all tests passed"
