#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# 
# Copyright 2012-2015 Frédéric Magniette, Miguel Rubio-Roy
# This file is part of Pyrame.
# 
# Pyrame is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyrame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyrame.  If not, see <http://www.gnu.org/licenses/>

import sys,time
import bindpyrame

######################################################

# Connectivity parameters to power supply (ps)
conf_string_ps="isegshq(bus=serial(device=/dev/ttyS0))"
#isegshq channels
channels = ["1","2"]

# IdV parameters (base units of the SI)
v1=-380
v2=-540
#step (V/s)
ramp=20
stabilization_time=1
current_limit=600e-6


######################################################

#defult ps port
port = 9500

# Helper function
def sendcmd(*params):
    retcode,res=bindpyrame.sendcmd("localhost",port,*params)
    if retcode==0:
        print(res)
        sys.exit(1)
    return res

# Get ports from Pyrame table
table=bindpyrame.init_ports_table("/opt/pyrame/ports.txt")
port=bindpyrame.get_port("PS_PORT",table)

# Initialization of module PS
ps_id=sendcmd("init_ps",conf_string_ps)

# Configure modules
sendcmd("config_ps",ps_id,"careful")

# Set current limit on PS
for ch in channels:
    sendcmd("set_current_limit_ps",ps_id,str(current_limit),ch)

# Power on PS
for ch in channels:
    sendcmd("power_on_ps",ps_id,ch)

# ?? optionla ???
#sendcmd("set_rise_delay_ps",ps_id,str("1"),channels[0])
#sendcmd("set_rise_delay_ps",ps_id,str("5"),channels[1])

sendcmd("set_voltage_ps",ps_id,str(v1),channels[0],str(ramp))
sendcmd("set_voltage_ps",ps_id,str(v2),channels[1],str(ramp))

v = ["",""]
i = ["",""]
# Main loop
print("# voltage(V)   current(A) # voltage(V)   current(A)")
for j in range(0,30):
    time.sleep(stabilization_time)
    k=0
    for ch in channels:
        res=sendcmd("get_current_ps",ps_id,ch)
        i[k]=res
        res=sendcmd("get_voltage_ps",ps_id,ch)
        v[k]=res
        k+=1
    res=sendcmd("get_error_queue_ps",ps_id)
    print("%i- %s : %s %s - %s : %s %s"%(j,channels[0],v[0],i[0],channels[1],v[1],i[1]))
    print res
    #    i=float(res)
    #print("%s : %.6e %.6e"%(ch,v,i))

print("powering off")

sendcmd("set_voltage_ps",ps_id,str(0),channels[0],str(ramp))
sendcmd("set_voltage_ps",ps_id,str(0),channels[1],str(ramp))

for j in range(0,30):
    time.sleep(stabilization_time)
    k=0
    for ch in channels:
        res=sendcmd("get_current_ps",ps_id,ch)
        i[k]=res
        res=sendcmd("get_voltage_ps",ps_id,ch)
        v[k]=res
        k+=1
    res=sendcmd("get_error_queue_ps",ps_id)
    print("%i- %s : %s %s - %s : %s %s"%(j,channels[0],v[0],i[0],channels[1],v[1],i[1]))
    print res

# Power off PS
for ch in channels:
    sendcmd("power_off_ps",ps_id,ch)

# Deinitialize modules
sendcmd("deinit_ps",ps_id)

