#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# 
# Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
# This file is part of Pyrame.
# 
# Pyrame is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyrame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyrame.  If not, see <http://www.gnu.org/licenses/>

import pools,conf_strings

# PF_TC110 #########################################################

class pf_tc110(object):
    def __init__(self):
        self.pf_tc110_pool=pools.pool()

    class pf_tc110_Exception(Exception):
        pass

    def init(self,pf_tc110_id,conf_string):
        try:
            conf=conf_strings.parse(conf_string)
        except Exception as e:
            return 0,str(e)
        if conf.name!="pf_tc110":
            return 0,"Invalid module name %s in conf_string instead of pf_tc110"%(conf.name)
        if not conf.has("rs485_addr"):
            return 0,"Error: the conf_string must include a rs485_addr parameter"
        if len(conf.params["rs485_addr"])>3:
            print("warning: trimming rs485 address to %s"%(conf.params["rs485_addr"][0:3]))
        if not conf.has("bus"):
            return 0,"Error: some of the required parameters (bus) in conf_string are not present"
        try:
            conf_bus=conf_strings.parse(conf.params["bus"])
        except Exception as e:
            return 0,str(e)
        if conf_bus.name=="serial":
            if not conf_bus.has("baudrate"):
                conf_bus.params["baudrate"]="9600"
        bus_id="bus_%s"%(pf_tc110_id)
        retcode,res=submod_execcmd("init@"+conf_bus.name,bus_id,conf_strings.unparse(conf_bus))
        if retcode==0:
            return 0,"Error initializing link <- %s" % (res)
        self.pf_tc110_pool.new(pf_tc110_id,{"addr":"%03d"%(int(conf.params["rs485_addr"][0:3])),"bus":conf_bus.name,"bus_id":bus_id})
        return 1,"ok"

    def deinit(self,pf_tc110_id):
        try:
            pf_tc110=self.pf_tc110_pool.get(pf_tc110_id)
        except Exception as e:
            return 1,str(e)
        retcode,res=submod_execcmd("deinit@"+pf_tc110["bus"],pf_tc110["bus_id"])
        if retcode==0:
            return 0,"Error deinitializing link <- %s" % (res)
        self.pf_tc110_pool.remove(pf_tc110_id)
        return 1,"ok"

    def config(self,pf_tc110_id):
        try:
            pf_tc110=self.pf_tc110_pool.get(pf_tc110_id)
        except Exception as e:
            return 0,str(e)
        if "configured" in pf_tc110:
            return 1,"already configured"
        retcode,res=submod_execcmd("config@"+pf_tc110["bus"],pf_tc110["bus_id"])
        if retcode==0:
            return 0,"Error configuring link <- %s" % (res)
        pf_tc110["configured"]=True
        return 1,"ok"

    def inval(self,pf_tc110_id):
        try:
            pf_tc110=self.pf_tc110_pool.get(pf_tc110_id)
        except Exception as e:
            return 0,str(e)
        if not "configured" in pf_tc110:
            return 1,"not configured"
        # Invalidate bus
        retcode,res=submod_execcmd("inval@"+pf_tc110["bus"],pf_tc110["bus_id"])
        if retcode==0:
            return 0,"Error invalidating link <- %s" % (res)
        # Remove parameters set during config
        del pf_tc110["configured"]
        return 1,"ok"

    def checksum(self,s):
        cs=0
        for c in s:
            cs+=ord(c)
        cs%=256
        return "%03d"%(cs)

    def get_set_param(self,pf_tc110_id,param,data="=?"):
        try:
            pf_tc110=self.pf_tc110_pool.get(pf_tc110_id)
        except Exception as e:
            return 0,str(e)
        if not "configured" in pf_tc110:
            return 0,"not configured"
        if len(param)>3:
            print("warning: trimming param to %s"%(param[0:3]))
        if data=="=?":
            action="00"
        else:
            action="10"
        data_length="%02d"%len(data)
        cmd=pf_tc110["addr"]+action+param[0:3]+data_length+data
        cmd+=self.checksum(cmd)
        retcode,res=submod_execcmd("wrnrd_until@"+pf_tc110["bus"],pf_tc110["bus_id"],cmd+r"\r",r"\r")
        if retcode==0:
            return 0,res
        if self.checksum(res[:10+int(res[8:10])])!=res[-3:]:
            return 0,"response with invalid checksum"
        return 1,res[10:-3]

    def set_rpm_pc(self,pf_tc110_id,rpm_pc):
        retcode,res=self.get_set_param(pf_tc110_id,"026","001")
        if retcode==0:
            return 0,res
        #retcode,res=self.get_set_param(pf_tc110_id,"707",rpm_pc)
        retcode,res=self.get_set_param(pf_tc110_id,"707",'{:0>4}'.format(rpm_pc)+"00")
        ###retcode,res=self.get_set_param(pf_tc110_id,"707","00abcd") #ab.cd%
        if retcode==0:
            return 0,res
        return 1,"ok"

    def power(self,pf_tc110_id,power,force="0"):
        if force=="1":
            # change power independently of speed
            if power:
                retcode,res=self.get_set_param(pf_tc110_id,"010","111111")
            else:
                retcode,res=self.get_set_param(pf_tc110_id,"010","000000")
            if retcode==0:
                return 0,res
            # ensure speed setting mode is off
            retcode,res=self.get_set_param(pf_tc110_id,"026","0")
            if retcode==0:
                return 0,res
            return 1,"ok"
        retcode,res=self.get_set_param(pf_tc110_id,"010")
        if retcode==0:
            return 0,res
        on=int(res)
        retcode,res=self.get_set_param(pf_tc110_id,"398")
        if retcode==0:
            return 0,res
        speed=int(res)
        if power and not on:
            if speed==0:
                retcode,res=self.get_set_param(pf_tc110_id,"010","111111")
                if retcode==0:
                    return 0,res
            else:
                return 0,"wait until speed is zero"
        elif not power and on:
            if speed>89000:
                retcode,res=self.get_set_param(pf_tc110_id,"010","000000")
                if retcode==0:
                    return 0,res
            else:
                return 0,"wait until drive is at full speed"
        # ensure speed setting mode is off
        retcode,res=self.get_set_param(pf_tc110_id,"026","0")
        if retcode==0:
            return 0,res
        return 1,"ok"

# CREATE PF_TC110 POOL #############################################

me=pf_tc110()

# COMMANDS #######################################################

def init_pf_tc110(pf_tc110_id,conf_string):
    """Initialize PF_TC110 vacuum controller.
    
    conf_string must contain the parameters:

    - chan: a integer from 1 to 4 for channels A1, A2, B1, B2.
    - bus: a conf_string for cmd_serial or cmd_tcp"""
    return me.init(pf_tc110_id,conf_string)
    
def deinit_pf_tc110(pf_tc110_id):
    "Deinitialize *pf_tc110_id* vacuum controller."
    return me.deinit(pf_tc110_id)

def config_pf_tc110(pf_tc110_id):
    "Configure *pf_tc110_id* vacuum controller."
    return me.config(pf_tc110_id)

def inval_pf_tc110(pf_tc110_id):
    "Invalidate *pf_tc110_id* vacuum controller."
    return me.inval(pf_tc110_id)

def power_on_pf_tc110(pf_tc110_id,force="0"):
    "Power on *pf_tc110_id* vacuum controller. Use *force* (1/0) to ignore speed limits"
    return me.power(pf_tc110_id,True,force)

def power_off_pf_tc110(pf_tc110_id,force="0"):
    "Power off *pf_tc110_id* vacuum controller. Use *force* (1/0) to ignore speed limits"
    return me.power(pf_tc110_id,False,force)

def get_param_pf_tc110(pf_tc110_id,param):
    "Get parameter from *pf_tc110_id* vacuum controller."
    return me.get_set_param(pf_tc110_id,param)

def set_param_pf_tc110(pf_tc110_id,param,data):
    "Get parameter from *pf_tc110_id* vacuum controller."
    return me.get_set_param(pf_tc110_id,param,data)

def get_rpm_pf_tc110(pf_tc110_id):
    "Get actual rotation speed (rpm) from *pf_tc110_id* vacuum controller."
    return me.get_set_param(pf_tc110_id,"398")

def set_rpm_pc_pf_tc110(pf_tc110_id,rpm_pc):
    "Set rotation speed percentage of nominal speed *rpm_pc* of *pf_tc110_id* vacuum controller."
    return me.set_rpm_pc(pf_tc110_id,rpm_pc)

def get_current_pf_tc110(pf_tc110_id):
    "Get drive current from *pf_tc110_id* vacuum controller."
    return me.get_set_param(pf_tc110_id,"310")

def get_motor_temp_pf_tc110(pf_tc110_id):
    "Get motor temperature from *pf_tc110_id* vacuum controller."
    return me.get_set_param(pf_tc110_id,"346")

def get_bearing_temp_pf_tc110(pf_tc110_id):
    "Get bearing temperature from *pf_tc110_id* vacuum controller."
    return me.get_set_param(pf_tc110_id,"342")

def get_power_temp_pf_tc110(pf_tc110_id):
    "Get drive power from *pf_tc110_id* vacuum controller."
    return me.get_set_param(pf_tc110_id,"316")

def get_operating_hours_pf_tc110(pf_tc110_id):
    "Get pump operating hours from *pf_tc110_id* vacuum controller."
    return me.get_set_param(pf_tc110_id,"311")

