#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# 
# Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
# This file is part of Pyrame.
# 
# Pyrame is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyrame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyrame.  If not, see <http://www.gnu.org/licenses/>

import sys,os,time,threading,Queue,datetime,traceback,subprocess,inspect,base64
import bindpyrame,getapi

script=None

#if __name__=="__main__":
#    execfile("/opt/pyrame/common_rcs.py")

######################################################################

def check_func(module,func_name):
    return (func_name in module.__dict__ and callable(module.__dict__[func_name]))

def set_param(name,value):
    "Set value of script parameter *name* to *value*"
    global script
    if script==None:
        return 0,"load a script first"
    if name not in script["module"].scriptenv["params"]:
        script["module"].scriptenv["params"][name]={"name":name,"default":"undef"}
    script["module"].scriptenv["params"][name]["value"]=value
    return 1,"ok"

def set_param_rc_script(name,value):
    retcode,res=set_param(name,value)
    if retcode==0:
        return 0,"rc_script: %s"%(res)
    return 1,res

######################################################################

def get_params_rc_script():
    global script
    if script==None or script["module"].scriptenv["params"]==None:
        return 0,"rc_script: load a script first"
    run_func=script["module_name"]+"_run"
    api=inspect.getargspec(script["module"].__dict__[run_func]).args
    res=[]
    for arg in api:
        if arg in script["module"].scriptenv["params"]:
            param=map(base64.b64encode,map(str,[arg,
                                     script["module"].scriptenv["params"][arg]["desc"],
                                     script["module"].scriptenv["params"][arg]["default"]]))
        else:
            param=map(base64.b64encode,map(str,[arg,arg,"undef"]))
        res.append(":".join(param))
    return 1,",".join(res)

######################################################################

def import_py_file(script,filename):
    # Return imported python module from *filename* without changing the path
    original_path=list(sys.path)
    directory,module_name=os.path.split(filename)
    module_name=os.path.splitext(module_name)[0]
    sys.path.insert(0,directory)
    try:
        module=__import__(module_name)
        # this is "import if necessary", so we reload systematically to be sure
        # we use the current file version and not a previously imported one
        module=reload(module)
    finally:
        sys.path[:]=original_path # restore
    with open(filename) as f:
        code=f.read()
    script["module_name"]=module_name
    script["module"]=module
    script["module_code"]=code

def script_alive():
    return script!=None and \
           "thread" in script and \
           script["thread"]!=None and \
           script["thread"].is_alive()

######################################################################

def load_rc_script(filename):
    "Load run script"
    if script_alive():
        return 0,"rc_script: a script is running. stop it first"
    global script
    script={}
    try:
        # import script
        script["filename"]=filename
        import_py_file(script,filename)
        global rc_stop_exception
        rc_stop_exception=script["module"].rc_stop_exception
        global pyrame_exception
        pyrame_exception=script["module"].pyrame_exception

        script["module"].scriptenv={}
        script["module"].scriptenv["params"]={}
        script["module"].scriptenv["module_code"]=script["module_code"]

        # create allow_run flag
        script["module"].scriptenv["allow_run"]=threading.Event()
        script["module"].scriptenv["stop_run"]=threading.Event()
    
        # create empty properties
        script["module"].scriptenv["verbose"]=True
        script["exceptions"]=Queue.Queue()
        script["last_progress"]=0
        script["module"].scriptenv["clean_stop"]=False
        script["thread"]=None
        # check existance of run function
        run_func=script["module_name"]+"_run"
        if not check_func(script["module"],run_func):
            raise Exception("%s function not found"%(run_func))
        # run init function of script
        init_func=script["module_name"]+"_init"
        if check_func(script["module"],init_func):
            script["module"].__dict__[init_func]()
    except Exception as e:
        script=None
        print traceback.format_exc()
        return 0,"rc_script: error loading script: %s"%(e)
    return 1,filename

######################################################################

def get_loaded_rc_script():
    "Get the path to the currently loaded script"
    global script
    if script==None:
        return 1,"None"
    return 1,script["filename"]

######################################################################

def get_last_error_rc_script():
    "Get last error catched from a running script"
    global script
    if script==None:
        return 0,"rc_script: load a script first"
    if script["exceptions"].empty():
        return 1,""
    while not script["exceptions"].empty():
        e=script["exceptions"].get()
    print "\n%s\n"%(e[1])
    return 1,str(e[0])

######################################################################

def runner():
    # Runner turns in a separate thread
    # and catches possible exceptions to give them to get_last_error
    global script
    try:
        run_func=script["module_name"]+"_run"
        # fill up call_dict with available params from set_param
        call_dict={}
        api=inspect.getargspec(script["module"].__dict__[run_func]).args
        for arg in api:
            if arg in script["module"].scriptenv["params"]:
                call_dict[arg]=str(script["module"].scriptenv["params"][arg]["value"])
        # call run function
        script["module"].__dict__[run_func](**call_dict)
        print("script successfully ended")
    except rc_stop_exception:
	print("script stopped by user request")
        return
    except Exception as e:
        script["exceptions"].put([e,traceback.format_exc()])
        print(traceback.format_exc())
    finally:
        # reset params to default values
        for p in script["module"].scriptenv["params"]:
            script["module"].scriptenv["params"][p]["value"]=script["module"].scriptenv["params"][p]["default"]

######################################################################

def start_rc_script():
    "Launch run() function of script in a thread. This function is asynchronous and returns as soon as the function is launched."
    global script
    if script_alive():
        return 0,"rc_script: a script is running. stop it first"
    if script==None:
        return 0,"rc_script: load a script first"
    try:
        while not script["exceptions"].empty():
            script["exceptions"].get()
        script["last_progress"]=0
        script["thread"]=threading.Thread(target=runner)
        # set flag to allow running
        script["module"].scriptenv["allow_run"].set()
        script["module"].scriptenv["stop_run"].clear()
        script["module"].scriptenv["clean_stop"]=False
        script["thread"].start()
    except Exception as e:
        return 0,"rc_script: error starting script: %s"%(e)
    return 1,"ok"

######################################################################

def stop_rc_script(timeout="10"):
    "Signal running script to stop. If after *timeout* seconds the script continues running, terminate the process."
    global script
    if script==None:
        return 0,"rc_script: load a script first"
    # before going further convert (and thus check) timeout
    timeout=float(timeout)
    if script_alive():
        # clear flag allowing to run script
        script["module"].scriptenv["allow_run"].clear()
        script["module"].scriptenv["stop_run"].set()
        # wait up to timeout for script to acknowledge stop by resetting the flag
        print("waiting for stop acknowledge...")
        if not script["module"].scriptenv["allow_run"].wait(timeout):
            print("timeout waiting for stop acknowledge")
        else:
            # allow some time between stop acknowledge and actual death
            # (exception bubbling and/or actions after an int_sleep)
            #print("wait some time: %f"%(timeout))
            waited=0
            while script["thread"].is_alive() and waited<=timeout:
                #print("sleep for %f cumul:%f"%(timeout/10,waited))
                time.sleep(timeout/10)
                waited+=timeout/10
        if script["thread"].is_alive():
            return 0,"rc_script: unable to stop script"
    script["thread"]=None
    script["module"].scriptenv["clean_stop"]=True
    return 1,"ok"

######################################################################

def get_progress_rc_script():
    "Get the progress, or degree of completion, of a script. Scripts are expected to use regularly the set_progress function, with a number from 0 to 100 to provide their degree of completion."
    global script
    if script==None:
        return 0,"rc_script: load a script first"
    # check if get_progress is implemented
    gp_func=script["module_name"]+"_get_progress"
    if not check_func(script["module"],gp_func):
        # not implemented
        if script["module"].scriptenv["clean_stop"]:
            return 1,"stop"
        elif not script_alive():
            return 1,"not running"
        else:
            return 1,"running"
    script["last_progress"]=script["module"].__dict__[gp_func]()
    if not script_alive() and script["last_progress"]>=100:
        # arrived to 100% and ended
        return 1,"end"
    if not script_alive():
        # stopped before arriving to 100%
        return 1,"stop"
    return 1,"%d"%(script["last_progress"])
