#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# 
# Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
# This file is part of Pyrame.
# 
# Pyrame is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyrame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyrame.  If not, see <http://www.gnu.org/licenses/>

import sys,signal,inspect,threading,os,traceback
import __main__ as script_main

def check_func(module,func_name):
    return (func_name in module.__dict__ and callable(module.__dict__[func_name]))

if __name__=="__main__" and "script" not in script_main.__dict__ and ("start_script" not in script_main.__dict__ or start_script):
    signal.signal(signal.SIGINT,sigint_handler)
    script={}
    module_name=os.path.split(__file__)[1]
    module_name=os.path.splitext(module_name)[0]
    script["module"]=script_main
    script["module"].scriptenv={}
    script["module"].scriptenv["params"]={}
    with open(__file__) as f:
        script["module"].scriptenv["module_code"]=f.read()
    script["module"].scriptenv["stop_run"]=threading.Event()
    script["module"].scriptenv["allow_run"]=threading.Event()

    # set flag to allow running
    script["module"].scriptenv["allow_run"].set()
    script["module"].scriptenv["stop_run"].clear()
    script["module"].scriptenv["verbose"]=False

    # check existance of run function
    run_func=module_name+"_run"
    if not check_func(script["module"],run_func):
        print("%s function not found"%(run_func))
        sys.exit(1)

    # init if function exists
    init_func=module_name+"_init"
    if check_func(script["module"],init_func):
        script["module"].__dict__[init_func]()

    # call run with available params from argv
    api=inspect.getargspec(script["module"].__dict__[run_func]).args
    if len(script["module"].scriptenv["params"])!=0 and len(api)!=len(script["module"].scriptenv["params"]):
        print("warning: inconsistency: %s has %d args but %s declares %d"%(run_func,len(api),init_func,len(script["module"].scriptenv["params"])))
    # show usage if --help or if we dont have enough default values
    if (len(sys.argv)>1 and sys.argv[1]=="--help") or \
       ((len(sys.argv)-1)!=len(api) and len(api)!=len(script["module"].scriptenv["params"])):
        print("usage: %s %s"%(sys.argv[0]," ".join(api))) 
        for a in api:
            if a in script["module"].scriptenv["params"]:
                print("    %s: %s (default: \"%s\")"%(a,script["module"].scriptenv["params"][a]["desc"],str(script["module"].scriptenv["params"][a]["default"])))
        sys.exit(1)

    # if missing arguments, use defaults
    if len(api)>len(sys.argv)-1:
        call_params=sys.argv[1:]
        i=1
        for a in api:
            if i>len(sys.argv)-1:
                if a in script["module"].scriptenv["params"]:
                    call_params.append(script["module"].scriptenv["params"][a]["default"])
            i+=1
    else:
        call_params=sys.argv[1:]
    call_params=map(str,call_params)

    print("running %s(%s)..."%(run_func,", ".join(map(lambda x: "\""+x+"\"",call_params))))
    i=0
    for a in api:
        if a in script["module"].scriptenv["params"]:
            script["module"].scriptenv["params"][a]["value"]=call_params[i]
        else:
            script["module"].scriptenv["params"][a]={"name":a,"value":call_params[i],"default":"undef"}
        i+=1
    try:
        script["module"].__dict__[run_func](*call_params)
    except rc_stop_exception:
	print("script stopped by user request")
        pass
        sys.exit(0)
    except pyrame_exception as e:
        print(str(e))
        sys.exit(1)
    except Exception as e:
        print(traceback.format_exc())
        sys.exit(1)
    print("script successfully ended")
