#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# 
# Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
# This file is part of Pyrame.
# 
# Pyrame is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyrame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyrame.  If not, see <http://www.gnu.org/licenses/>

import sys
import bindpyrame

# Parse arguments
if len(sys.argv)<2:
    print("usage: %s acq [rundb_host]"%(sys.argv[0]))
    print("       acq: id of acquisition")
    print("       rundb_host: hostname or IP of runDB (default: localhost)")
    sys.exit(1)

acq=sys.argv[1]

if len(sys.argv)==3:
    host=sys.argv[2]
else:
    host="localhost"

# Get port
try:
    port = bindpyrame.get_port("RUNDB_PORT")
except Exception as e:
    print e
    sys.exit(1)

# Helper function
def call(function,*params):
    retcode,res=bindpyrame.sendcmd(host,port,function,*params)
    if retcode==0:
        print(res)
        sys.exit(1)
    return res

# Get parameters of acq
acq_id=call("check_acq_id_rundb",acq)
params=call("get_params_acq_rundb",acq_id)
if params=="":
    print("acquisition doesn't contain any parameter")
    sys.exit(0)
params=params.split(",")
params.sort()
pad=max(map(len,params))
print("acq_id: %s"%(acq_id))
print("-"*80)
print("%%-%ds | %%s"%(pad)%("param","value"))
print("-"*80)
for param in params:
    value=call("get_param_acq_rundb",acq_id,param)
    print("%%-%ds | %%s"%(pad)%(param,value))

sys.exit(0)
