#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# 
# Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
# This file is part of Pyrame.
# 
# Pyrame is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyrame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyrame.  If not, see <http://www.gnu.org/licenses/>

import sys
import bindpyrame

# Parse arguments
if len(sys.argv)==2 and sys.argv[1]=="--help":
    print("usage: %s [include_acqs [root_run_group [depth [rundb_host]]]]"%(sys.argv[0]))
    print("  include_acqs:   include acqs in tree (default: false)")
    print("  root_run_group: first run group from which to show tree (default: root)")
    print("  depth:          depth of nested runs to show (default: undef)")
    print("  rundb_host:     hostname or IP of runDB (default: localhost)")
    sys.exit(1)

def sbool(value):
    return value.lower() in ["true","1","on"]

if len(sys.argv)>1:
    include_acqs=sbool(sys.argv[1])
else:
    include_acqs=False

if len(sys.argv)>2:
    root_run_g=sys.argv[2]
else:
    root_run_g="root"

if len(sys.argv)>3 and sys.argv[3]!="undef":
    max_depth=int(sys.argv[3])
    if max_depth<1:
        print("invalid depth value")
        sys.exit(1)
else:
    max_depth=-1

if len(sys.argv)>4:
    host=sys.argv[4]
else:
    host="localhost"

# Get port
try:
    port = bindpyrame.get_port("RUNDB_PORT")
except Exception as e:
    print e
    sys.exit(1)

# Helper function
def call(function,*params):
    retcode,res=bindpyrame.sendcmd(host,port,function,*params)
    if retcode==0:
        print(res)
        sys.exit(1)
    return res

# Print function for acqs
def print_acqs_in_run(run_id,current_depth):
    acqs=call("get_acqs_in_run_rundb",run_id,"false")
    if acqs!="":
        acq_names=acqs.split(",")
        tree="|    "*(current_depth-1)
        acq_names.sort()
        for acq_name in acq_names:
            print("%s|--> acq: %s"%(tree,acq_name))

# Print function for runs
def print_runs_in_run_group(run_g_id,current_depth,max_depth):
    runs=call("get_runs_in_run_group_rundb",run_g_id)
    if runs!="":
        runs=runs.split(",")
        tree="|    "*(current_depth-1)
        run_names=[]
        for run_id in runs:
            name=call("get_run_name_rundb",run_id)
            run_names.append([name,run_id])
        run_names.sort()
        for run in run_names:
            name=run[0]
            run_id=run[1]
            print("%s|--> run: %s"%(tree,name))
            if current_depth!=max_depth:
                if include_acqs:
                    print_acqs_in_run(run_id,current_depth+1)
            else:
                print("%s|--> reached max depth"%(tree))


# Recursive print function for run groups
def print_run_groups_in_parent(run_g_id,current_depth,max_depth):
    run_groups=call("get_run_groups_in_run_group_rundb",run_g_id,"false")
    if run_groups!="":
        run_groups=run_groups.split(",")
        tree="|    "*(current_depth-1)
        run_g_names=[]
        for run_g_id in run_groups:
            name=call("get_run_group_name_rundb",run_g_id)
            run_g_names.append([name,run_g_id])
        run_g_names.sort()
        for run_g in run_g_names:
            name=run_g[0]
            run_g_id=run_g[1]
            print("%s|--> run_g: %s"%(tree,name))
            if current_depth!=max_depth:
                print_run_groups_in_parent(run_g_id,current_depth+1,max_depth)
                print_runs_in_run_group(run_g_id,current_depth+1,max_depth)
            else:
                print("%s|--> reached max depth"%(tree))

# Print tree
print("tree of run group %s:"%(root_run_g))
print_run_groups_in_parent(root_run_g,1,max_depth)
print_runs_in_run_group(root_run_g,1,max_depth)

sys.exit(0)
