#!/usr/bin/env python2

import sys,subprocess,time
import couchdb

def execcmd(command,err_msg):
    result=subprocess.Popen(command,stdout=subprocess.PIPE,stderr=subprocess.STDOUT,shell=True)
    res,_=result.communicate()
    if result.returncode!=0:
        print(res)
        print(err_msg)
        sys.exit(1)
    return res

# get CouchDB db-dir
db_dir=execcmd("couch-config --db-dir","error getting couchdb db-dir")
db_dir=db_dir.strip()

cdbs=couchdb.Server()

old_acq_db=cdbs["acqs"]
old_acq_db.compact()
old_run_db=cdbs["runs"]
old_run_db.compact()

# create new databases
try:
    del cdbs["new_acqs"]
    del cdbs["new_runs"]
    del cdbs["new_run_groups"]
except:
    pass
new_acq_db=cdbs.create("new_acqs")
new_acq_db["_design/basic"]={
    "views":{
        "acq_name":{
            "map":"function(doc){emit(doc.name,doc.run_id);}"
        },
        "acq_run":{
            "map":"function(doc){emit(doc.run_id,doc.name);}"
        }
    }
}
new_run_db=cdbs.create("new_runs")
new_run_db["_design/basic"]={
    "views":{
        "run_name":{
            "map":"function(doc){emit(doc.name,doc.run_g_id);}"
        },
        "run_run_g":{
            "map":"function(doc){emit(doc.run_g_id,doc.name);}"
        }
    }
}
new_run_g_db=cdbs.create("new_run_groups")
new_run_g_db["_design/basic"]={
    "views":{
        "run_g_name":{
            "map":"function(doc){emit(doc.name,doc.run_g_id);}"
        },
        "run_g_run_g":{
            "map":"function(doc){emit(doc.run_g_id,doc.name);}"
        }
    }
}

# get root_path
doc=old_run_db["0"]
root_path=doc["path"]
doc["lineage"]=root_path
old_run_db["0"]=doc
new_run_g_db["root"]={"name":"root","run_g_id":None,"path":root_path}

def add_lineage_to_acqs(parent_id,path):
    acqs_completed=[]
    acqs=old_acq_db.view("_design/basic/_view/acq_runs",key=parent_id)
    if len(acqs)!=0:
        for a in acqs:
            #print("adding lineage to acq: %s %s"%(a.id,a.value))
            doc=old_acq_db[a.id]
            doc["lineage"]=path+"/"+a.value
            acqs_completed.append((a.id,doc["lineage"]))
            old_acq_db[a.id]=doc
    return acqs_completed
            
def add_lineage_to_runs(parent_id,path):
    runs_completed=[]
    acqs_completed=[]
    runs=old_run_db.view("_design/basic/_view/run_parents",key=parent_id)
    if len(runs)!=0:
        for r in runs:
            #print("adding lineage to run: %s %s"%(r.id,r.value))
            doc=old_run_db[r.id]
            doc["lineage"]=path+"/"+r.value
            old_run_db[r.id]=doc
            runs_completed.append((r.id,doc["lineage"]))
            c_acqs1,c_runs=add_lineage_to_runs(r.id,doc["lineage"])
            c_acqs2=add_lineage_to_acqs(r.id,doc["lineage"])
            runs_completed+=c_runs
            acqs_completed+=c_acqs1+c_acqs2
    return acqs_completed,runs_completed

def check_for_acqs(parent_id):
    runs_w_acqs=[]
    runs_wo_acqs=[]
    runs=old_run_db.view("_design/basic/_view/run_parents",key=parent_id)
    if len(runs)!=0:
        for r in runs:
            acqs=old_acq_db.view("_design/basic/_view/acq_runs",key=r.id)
            if len(acqs)!=0:
                #print("acqs in run %s"%(old_run_db[r.id]["lineage"]))
                runs_w_acqs.append(r.id)
            else:
                #print("no acqs in run %s"%(old_run_db[r.id]["lineage"]))
                runs_wo_acqs.append(r.id)
                w_acqs,wo_acqs=check_for_acqs(r.id)
                runs_w_acqs+=w_acqs
                runs_wo_acqs+=wo_acqs
    return runs_w_acqs,runs_wo_acqs

# add lineage to acqs and runs. This will be the future id
print("building new ids...")
acqs_completed,runs_completed=add_lineage_to_runs("0",root_path)
print("%d acqs in root run won't be migrated"%(len(add_lineage_to_acqs("0",root_path))))
if len(acqs_completed)!=len(set(acqs_completed)):
    print("error: duplicates detected in acqs")
    sys.exit(1)
if len(runs_completed)!=len(set(runs_completed)):
    print("error: duplicated detected in runs")
    sys.exit(1)
print("added lineage to %d acqs and %d runs"%(len(acqs_completed),len(runs_completed)))

# find runs with acqs.
# These are the only runs that will stay runs (rest -> run_groups)
print("finding runs with acqs...")
runs_w_acqs,runs_wo_acqs=check_for_acqs("0")
print("%d runs with acqs, %d runs without acqs"%(len(runs_w_acqs),len(runs_wo_acqs)))

print("creating run groups from runs without acqs...")
for r in runs_wo_acqs:
    doc=old_run_db[r]
    run_g_id=old_run_db[doc["parent_id"]]["lineage"]
    if run_g_id==root_path:
        run_g_id="root"
    doc["run_g_id"]=run_g_id
    del doc["parent_id"]
    del doc["lineage"]
    new_run_g_db[old_run_db[r]["lineage"]]=doc

print("creating runs from runs with acqs...")
for r in runs_w_acqs:
    doc=old_run_db[r]
    run_g_id=old_run_db[doc["parent_id"]]["lineage"]
    if run_g_id==root_path:
        run_g_id="root"
    doc["run_g_id"]=run_g_id
    del doc["parent_id"]
    del doc["lineage"]
    new_run_db[old_run_db[r]["lineage"]]=doc

print("creating acqs from acqs...")
for a in old_acq_db:
    if a.startswith("_"):
        continue
    doc=old_acq_db[a]
    if doc["run_id"]=="0":
        continue
    del doc["lineage"]
    run_id=doc["run_id"]
    while True:
        if run_id in runs_w_acqs:
            break
        run_id=old_run_db[run_id]["parent_id"]
        #print("acq %s is composite."%(old_acq_db[a]["lineage"]))
        #print("looking back %s"%(old_run_db[run_id]["lineage"]))
    doc["run_id"]=old_run_db[run_id]["lineage"]
    doc["name"]=old_acq_db[a]["lineage"][len(doc["run_id"])+1:]
    new_acq_db[old_acq_db[a]["lineage"]]=doc

# wait for sync
print("syncing changes")
time.sleep(1.5)

# stop CouchDB
print("stopping CouchDB")
execcmd("/etc/init.d/couchdb stop","error stopping couchdb")

# move old db
print("moving old db out")
command="cd %s ;"%(db_dir) \
       +"mv acqs.couch old_acqs.couch ;" \
       +"mv runs.couch old_runs.couch"
execcmd(command,"error moving old db out")

# move new db in
print("moving new db in")
command="cd %s ; mv new_acqs.couch acqs.couch ; mv new_runs.couch runs.couch ; mv new_run_groups.couch run_groups.couch"%(db_dir)
execcmd(command,"error moving new db in")

# start CouchDB
print("starting CouchDB")
execcmd("/etc/init.d/couchdb start","error starting couchdb")

