#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# 
# Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
# This file is part of Pyrame.
# 
# Pyrame is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Pyrame is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public License
# along with Pyrame.  If not, see <http://www.gnu.org/licenses/>

import base64

smtest_roctype=""
state="UNDEF"
rocs=[]

def sbool(value):
    return value.lower() in ["true","1","on"]

def get_seq_transition_smtest(smtest_id,transition_name,goto):
    goto=sbool(goto)
    transitions_fstate={
            "init":      "READY",
            "config":    "CONFIG",
            "validate":  "CONFIG",
            "start_acq": "ACQING",
            "stop_acq":  "CONFIG",
            "inval":     "READY",
            "deinit":    "UNDEF"
            }
    # Get future state
    fstate=transitions_fstate.get(transition_name,"")

    seq="unable"
    # UNDEF
    if state=="UNDEF"    and fstate=="READY":
        seq="init"
    if state=="UNDEF"    and fstate=="CONFIG" and goto:
        seq="init,config"
    if state=="UNDEF"    and fstate=="ACQING" and goto:
        seq="init,config,start_acq"

    # READY
    if state=="READY"    and fstate=="UNDEF":
        seq="deinit"
    if state=="READY"    and fstate=="CONFIG":
        seq="config"
    if state=="READY"    and fstate=="ACQING" and goto:
        seq="config,start_acq"

    # CONFIG
    if state=="CONFIG"   and fstate=="UNDEF" and goto:
        seq="inval,deinit"
    if state=="CONFIG"   and fstate=="READY":
        seq="inval"
    if state=="CONFIG"   and fstate=="ACQING":
        seq="start_acq"

    # RECONF
    if state=="RECONF"   and fstate=="UNDEF" and goto:
        seq="inval,deinit"
    if state=="RECONF"   and fstate=="READY":
        seq="inval"
    if state=="RECONF"   and fstate=="ACQING":
        seq="validate,start_acq"

    # ACQING
    if state=="ACQING"   and fstate=="UNDEF" and goto:
        seq="stop_acq,inval,deinit"
    if state=="ACQING"   and fstate=="READY" and goto:
        seq="stop_acq,inval"
    if state=="ACQING"   and fstate=="CONFIG":
        seq="stop_acq"

    if seq=="unable":
         return 0,"smtest: invalid transition"
    return 1,seq

def get_state_smtest(smtest_id):
    return 1,state

def init_smtest(smtest_id,roctype):
    if state!="UNDEF":
        return 0,"smtest: unable to initialize in state %s"%(state)
    global smtest_roctype
    smtest_roctype=roctype
    return 1,"ok"

def init_fin_smtest(smtest_id):
    retcode,res=submod_execcmd("get_name_sublist@cmod",smtest_roctype,smtest_id)
    if retcode==0:
        return 0,"smtest: error while getting rocs <- %s"%(res)
    rocs=res.split(",")
    for roc in rocs:
        chipid = int(roc.split("_")[-1])-1
        print("assigning chipid = 0x{0:02x} / {0:d} / 0b{0:08b} to roc {1}".format(chipid,roc))
        retcode,res=submod_execcmd("set_chipid_param@"+smtest_roctype,roc,str(chipid))
        if retcode==0:
            return 0,"smtest: error setting chipid for roc <- %s"%(res)
    global state
    state="READY"
    return 1,"ok"

def deinit_smtest(smtest_id):
    if state!="READY":
        return 0,"smtest: unable to deinitialize in state %s"%(state)
    return 1,"ok"

def deinit_fin_smtest(smtest_id):
    global state
    state="UNDEF"
    return 1,"ok"

def config_smtest(smtest_id):
    if state!="READY":
        return 0,"smtest: unable to configure in state %s"%(state)
    return 1,"ok"

def config_fin_smtest(smtest_id):
    global state
    # force validate after
    state="RECONF"
    return 1,"ok"

def validate_fin_smtest(smtest_id):
    global state
    if state!="RECONF":
        return 0,"smtest: unable to validate in state %s"%(state)
    retcode,res=submod_execcmd("get_name_sublist@cmod",smtest_roctype,smtest_id)
    if retcode==0:
        return 0,"smtest: error while getting rocs <- %s"%(res)
    global rocs
    if res!="":
        rocs=res.split(",")
    else:
        return 0,"smtest: no %s found in cmod"%(smtest_roctype)
    state="CONFIG"
    return 1,"ok"

def inval_smtest(smtest_id):
    if state not in ["CONFIG","RECONF"]:
        return 0,"smtest: unable to invalidate in state %s"%(state)
    return 1,"ok"

def inval_fin_smtest(smtest_id):
    global state
    global rocs
    rocs=[]
    state="READY"
    return 1,"ok"

def start_acq_smtest(smtest_id,complete_prefix,roctype):
    if state!="CONFIG":
        return 0,"smtest: unable to start_acq in state %s"%(state)
    sc=""
    for roc in rocs:
        retcode,res=submod_execcmd("dump_sc@%s"%(roctype),roc)
        if retcode==0:
            return 0,"smtest: error getting slow control of %s <- %s"%(roc,res)
        sc+=res
    with open("%s.txt"%(complete_prefix),"w") as f:
        f.write(sc)
    return 1,"ok"

def start_acq_fin_smtest(smtest_id):
    global state
    state="ACQING"
    return 1,"ok"

def stop_acq_smtest(smtest_id):
    if state!="ACQING":
        return 0,"smtest: unable to stop_acq in state %s"%(state)
    return 1,"ok"

def stop_acq_fin_smtest(smtest_id):
    global state
    state="CONFIG"
    return 1,"ok"

def get_configb64_smtest(smtest_id):
    if state not in ["CONFIG","RECONF"]:
        return 0,"get_config is callable only in state CONFIG or RECONF (currently: %s)"%(state)
    retcode,res=submod_execcmd("gener_configb64@cmod")
    if retcode==0:
        return 0,"cant get config <- %s"%(res)
    return 1,res
