/*
Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
This file is part of Pyrame.

Pyrame is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Pyrame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
*/

#include<pyrame.h>

#define GEN_PORT 35000
#define CONS_PORT 35001

int main(int argc,char**argv) {
  struct cmd_result *pres;
  int bs;
  char bs_str[48];
  int nb_blocks;
  char nb_blocks_str[48];
  int period;
  char period_str[48];
  char *last_rblocks=NULL;
  char *last_speed=NULL;
  int gen_socket=open_socket("localhost",GEN_PORT);
  int cons_socket=open_socket("localhost",CONS_PORT);
  int i;
  int bs_start,bs_end,bs_incr;
  int nb_bl_start,nb_bl_end,nb_bl_incr;
  int period_start,period_end,period_incr;

  if (argc<10) {
    printf("usage: %s block_size_start block_size_end block_size_incr nb_bl_start nb_bl_end nb_bl_incr period_start period_end period_incr\n",argv[0]);
    return 1;
  }
  bs_start=atoi(argv[1]);
  bs_end=atoi(argv[2]);
  bs_incr=atoi(argv[3]);
  if (bs_incr<=0) {
    printf("warning: setting bs_incr to 1\n");
    bs_incr=1;
  }
  nb_bl_start=atoi(argv[4]);
  nb_bl_end=atoi(argv[5]);
  nb_bl_incr=atoi(argv[6]);
  if (nb_bl_incr<=0) {
    printf("warning: setting nb_bl_incr to 1\n");
    nb_bl_incr=1;
  }
  period_start=atoi(argv[7]);
  period_end=atoi(argv[8]);
  period_incr=atoi(argv[9]);
  if (period_incr>=0) {
    printf("warning: setting period_incr to -1\n");
    period_incr=-1;
  }
  
  setbuf(stdout, NULL);

  printf("#block_size nb_blocks period recv_blocks speed\n");
  for (bs=bs_start;bs<=bs_end;bs+=bs_incr) {
    for (nb_blocks=nb_bl_start;nb_blocks<=nb_bl_end;nb_blocks+=nb_bl_incr) {
      for (period=period_start;period>=period_end;period+=period_incr) {
        sprintf(bs_str,"%d",bs);
        sprintf(nb_blocks_str,"%d",nb_blocks);
        sprintf(period_str,"%d",period);
        pres=execcmd(cons_socket,"start_cons",nb_blocks_str,"end");
        if (pres->retcode==0) {
          printf("cant start consumer <- %s\n",pres->str);
          return 1;
        }
        free_cmd_result(pres);
        pres=execcmd(gen_socket,"start_gen",bs_str,nb_blocks_str,period_str,"end");
        if (pres->retcode==0) {
          printf("cant start generator <- %s\n",pres->str);
          return 1;
        }
        free_cmd_result(pres);
        if (last_rblocks!=NULL) {
          free(last_rblocks);
        }
        if (last_speed!=NULL) {
          free(last_speed);
        }
        last_rblocks=NULL;
        last_speed=NULL;
        for (i=0;i<2;i++) {
          // assume 200us to generate block
          // assume 10us per additional byte
          // depth of gen dispatcher = 50
          // reduce to half if checking second time
          usleep((period+200+bs)*(nb_blocks+50)/(i*2+1));
          pres=execcmd(cons_socket,"get_rblocks_cons","end");
          if (pres->retcode==0) {
            printf("cant get received blocks from consumer <- %s\n",pres->str);
            return 1;
          }
          if (!strcmp(pres->str,nb_blocks_str)) {
            //consumer consumed all generated blocks
            last_rblocks=pres->str;
            free(pres);
            break;
          } else if (last_rblocks!=NULL && strcmp(pres->str,last_rblocks)) {
            //still consuming
            printf("insufficient sleep!\n");
          } else {
            //maybe still consuming
            last_rblocks=pres->str;
          }
          free(pres);
        }
        pres=execcmd(gen_socket,"get_last_speed_gen","end");
        if (pres->retcode==0) {
          printf("cant get last speed of generator <- %s\n",pres->str);
          return 1;
        }
        free(pres);
        last_speed=pres->str;
        printf("%d %d %d %s %s\n",bs,nb_blocks,period,last_rblocks,last_speed);
        pres=execcmd(gen_socket,"flush_gen","end");
        if (pres->retcode==0) {
          printf("cant flush generator <- %s\n",pres->str);
          return 1;
        }
        free_cmd_result(pres);
      }
    }
  }
  return 0;
}
