/*
Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
This file is part of Pyrame.

Pyrame is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Pyrame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
*/

#include<pyrame.h>

#define GEN_PORT 35000
#define CONS_PORT 35001

int main(int argc,char**argv) {
  struct cmd_result *pres;
  int bs;
  char bs_str[48];
  int nb_blocks;
  char nb_blocks_str[48];
  int period;
  char period_str[48];
  char *last_rblocks=NULL;
  char *last_speed=NULL;
  int gen_socket=open_socket("localhost",GEN_PORT);
  int cons_socket=open_socket("localhost",CONS_PORT);
  int i;
  int bs_start,bs_end,bs_incr;
  double cons_time,cons_byte_speed,cons_block_speed,gen_byte_speed,gen_block_speed;

  if (argc<5) {
    printf("usage: %s block_size_start block_size_end block_size_incr nb_blocks\n",argv[0]);
    return 1;
  }
  bs_start=atoi(argv[1]);
  bs_end=atoi(argv[2]);
  bs_incr=atoi(argv[3]);
  if (bs_incr<=0) {
    printf("warning: setting bs_incr to 1\n");
    bs_incr=1;
  }
  nb_blocks=atoi(argv[4]);
  period=0;
  
  setbuf(stdout, NULL);

  printf("#block_size cons_block_speed cons_byte_speed gen_block_speed gen_byte_speed\n");
  for (bs=bs_start;bs<=bs_end;bs+=bs_incr) {
    sprintf(bs_str,"%d",bs);
    sprintf(nb_blocks_str,"%d",nb_blocks);
    sprintf(period_str,"%d",period);
    pres=execcmd(gen_socket,"start_sync_gen",bs_str,nb_blocks_str,period_str,"end");
    if (pres->retcode==0) {
      printf("cant start generator <- %s\n",pres->str);
      return 1;
    }
    free_cmd_result(pres);
    
    pres=execcmd(cons_socket,"start_sync_cons",nb_blocks_str,"end");
    if (pres->retcode==0) {
      printf("cant start consumer <- %s\n",pres->str);
      return 1;
    }
    cons_time=atof(pres->str);
    free_cmd_result(pres);
    cons_block_speed=nb_blocks/cons_time;
    cons_byte_speed=nb_blocks*bs/cons_time;

    pres=execcmd(gen_socket,"get_last_speed_gen","end");
    if (pres->retcode==0) {
      printf("cant get last speed of generator <- %s\n",pres->str);
      return 1;
    }
    gen_block_speed=atof(pres->str);
    free_cmd_result(pres);
    gen_byte_speed=bs*gen_block_speed;

    printf("%d %f %f %f %f\n",bs,cons_block_speed,cons_byte_speed,gen_block_speed,gen_byte_speed);
    pres=execcmd(gen_socket,"flush_gen","end");
    if (pres->retcode==0) {
      printf("cant flush generator <- %s\n",pres->str);
      return 1;
    }
    free_cmd_result(pres);
  }
  return 0;
}
