/*
 * Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
 * This file is part of Pyrame.
 *
 * Pyrame is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Pyrame is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
 * */

#include <pyrame.h>

struct av_w_err {
  float average;
  float error;
};
struct av_w_err *average_w_error(float* values,int nb_values) {
  float stdev;
  int i;
  struct av_w_err *result=malloc(sizeof(struct av_w_err));
  result->average=0;
  for (i=0;i<nb_values;i++) {
    result->average+=values[i];
  }
  result->average/=nb_values;
  if (nb_values>1) {
    stdev=0;
    for (i=0;i<nb_values;i++) {
      stdev+=pow(values[i]-result->average,2);
    }
    stdev=sqrt(stdev/(nb_values-1));
    result->error=stdev;
  } else {
    result->error=0;
  }
  return result;
}

int main(int argc,char**argv) {
  struct timeval t1;
  struct timeval t2;
  int nb1;
  int nb2;
  short quiet=0;
  int i,j;
  int testport=get_port("TEST_PORT");
  int sock;
  int etime;
  float *etimes;
  int size_start,size_end,size_incr,s;
  struct av_w_err *average;
  struct cmd_result *result;
  char *arg;

  if (argc<6) {
    printf("usage: %s nb_measures transactions_per_measure size_start size_end size_incr [--quiet]\n",argv[0]);
    return 1;
  }

  nb1=atoi(argv[1]);
  nb2=atoi(argv[2]);
  size_start=atoi(argv[3]);
  size_end=atoi(argv[4]);
  size_incr=atoi(argv[5]);
  if (argc>=7 && !strcmp(argv[6],"--quiet")) {
    quiet=1;
  }

  etimes=malloc(nb1*sizeof(float));
  arg=malloc(size_end);

  if (!quiet) {
    setvbuf(stdout, NULL, _IONBF, 0);
    fprintf(stdout,"with single socket open\n");
  }
  printf("#size us/op error\n");
  sock=open_socket("localhost",testport);
  for (s=size_start;s<=size_end;s+=size_incr) {
    for (i=0;i<s;i++) {
      arg[i]='1';
    }
    arg[i]=0;
    for (i=0;i<nb1;i++) {
      gettimeofday(&t1,NULL);
      for (j=0;j<nb2;j++) {
        result=execcmd(sock,"onearg_test",arg,"end");
        if (result->retcode<=0) {
          fprintf(stdout,"result->res=%s\n",result->str);
        }
        free_cmd_result(result);
      }
      gettimeofday(&t2,NULL);
      etime=(t2.tv_sec-t1.tv_sec)*1000000+(t2.tv_usec-t1.tv_usec);
      etimes[i]=(float)etime/nb2;
      //printf("measure %d: total_elapsed_time=%f per_command=%f\n",i,(float)etime,etimes[i]);
    }
    average=average_w_error(etimes,nb1);
    // systematic error of gettimeofday is 0.5us, so we add it
    average->error=sqrt(pow(average->error,2)+pow(0.5,2));
    fprintf(stdout,"%d %f %f\n",s,average->average,average->error);
    free(average);
  }
  close(sock);

  /*
  if (!quiet) {
    fprintf(stdout,"with one socket open per operation\n");
    fprintf(stderr,"progress\n");
    fprintf(stderr,"____________________________________________________\n");
    fprintf(stderr,"|");
  }
  for (i=0;i<nb1;i++) {
    gettimeofday(&t1,NULL);
    for (j=0;j<nb2;j++) {
      s=open_socket("localhost",testport);
      result=execcmd(s,"void_test","end");
      if (result->retcode==0) {
        fprintf(stdout,"result->res=%s\n",result->str);
      }
      free(result);
      close(s);
    }
    gettimeofday(&t2,NULL);
    etime=(t2.tv_sec-t1.tv_sec)*1000000+(t2.tv_usec-t1.tv_usec);
    etimes[i]=(float)etime/nb2;
    if (!quiet && i%(nb1/50)==0) {
      fprintf(stderr,"·");
    }
  }
  if (!quiet) {
    fprintf(stderr,"|\n");
  }
  average=average_w_error(etimes,nb1);
  // systematic error of gettimeofday is 0.5us, so we add it
  average->error=sqrt(pow(average->error,2)+pow(0.5,2));
  if (!quiet) {
    fprintf(stdout,"performance: %f ± %f us/op\n",average->average,average->error);
  } else {
    fprintf(stdout,"%f %f\n",average->average,average->error);
  }
  free(average);
  */
}
