/*
Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
This file is part of Pyrame.

Pyrame is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Pyrame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
*/

#include <pthread.h>
#include <pyrame.h>

struct tcpc_ws {
  int sock;
  int timeout;
  pthread_t reader_t;
  pthread_t writer_t;
};

void *forwarder(char *name,int in,int out,int bufsize) {
  int count;
  char buf[bufsize];
  while (1) {
    count=read(in,buf,sizeof(buf));
    if (count<0) {
      printf("error in %s:\n",name);
      perror("read");
      break;
    }
    if (count==0) {
      break;
    }
    buf[count]=0;
    write(out,buf,count);
  }
  return NULL;
}

// Read from stdin and write to sock
void *tcp_writer(void *workspace) {
  struct tcpc_ws *ws=(struct tcpc_ws*)workspace;
  forwarder("tcp_writer",STDIN_FILENO,ws->sock,1024);
  if (ws->timeout>0) {
    printf("waiting %d ms for more network data...\n",ws->timeout*1000);
    usleep(ws->timeout*1000);
  }
  pthread_kill(ws->reader_t,SIGINT);
  return NULL;
}

// Read from sock and print to stdout
void *tcp_reader(void *workspace) {
  struct tcpc_ws *ws=(struct tcpc_ws*)workspace;
  forwarder("tcp_reader",ws->sock,STDOUT_FILENO,4096);
  pthread_kill(ws->writer_t,SIGINT);
  return NULL;
}

int main(int argc,char **argv) {
  struct tcpc_ws *ws=malloc(sizeof(struct tcpc_ws));
  memset(ws,0,sizeof(struct tcpc_ws));

  if (argc<3) {
      printf("usage: %s host port [timeout(ms)]\n",argv[0]);
      printf("       timeout: defaults to 1 second\n");
      exit(1);
  }

  ws->sock=open_socket(argv[1],atoi(argv[2]));
  if (ws->sock==-1) {
    printf("error opening connection to %s:%d\n",argv[1],atoi(argv[2]));
    return 1;
  }
  if (argc!=4) {
    ws->timeout=1000;
  } else {
    ws->timeout=atoi(argv[3]);
  }

  if (pthread_create(&ws->reader_t,NULL,tcp_reader,ws)<0) {
    perror("pthread_create reader");
    return 1;
  }
  if (pthread_create(&ws->writer_t,NULL,tcp_writer,ws)<0) {
    perror("pthread_create writer");
    return 1;
  }
  pthread_join(ws->writer_t,NULL);
  pthread_join(ws->reader_t,NULL);

  return 0;
}
