#!/bin/sh
# -*- coding: utf-8 -*-
# 
# Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
# This file is part of Pyrame.
#takes a command executes it and check its return value

exec_n_test () {
    echo -n "checking success of $@ ... "
    "$@" > ent.trace 2>&1 
    if test $? != "0"; then
	echo "fail with trace:"
	cat ent.trace
	exit 1
    else
        RETSTR=`cat ent.trace | cut -f "3-" -d "=" -`
	echo "success"
    fi
}

exec_n_test_retcode () {
    EXP_RETCODE=$1
    shift
    echo -n "checking retcode=$EXP_RETCODE of $@ ... "
    "$@" > ent.trace 2>&1 
    RETCODE=`cat ent.trace | cut -f "2-" -d "=" - | cut -f "1" -d " " -`
    if test "$RETCODE" != "$EXP_RETCODE"; then
	echo "fail with trace:"
	cat ent.trace
	exit 1
    else
        RETSTR=`cat ent.trace | cut -f "3-" -d "=" -`
	echo "success"
    fi
}

exec_subscript () {
    "$@"
    if test $? != "0"
    then
	echo "fail"
        exit 1
    else
        echo "success"
    fi
}

exec_sub_unit_test () {
    echo "checking success of sub unit test $@ :"
    "$@"
    if test $? != "0"; then
        echo "failure of sub unit test $@"
        exit 1
    else
        echo "success of sub unit test $@"
    fi
}

checkretstr () {
    EXPECTED="$1"
    shift
    echo -n "checking retstr \"$EXPECTED\" against result of $@ ... "
    "$@" > ent.trace 2>&1
    RETSTR=`cat ent.trace | cut -f "3-" -d "=" -`
    if test "$RETSTR" != "$EXPECTED"; then
        echo "fail with trace:"
        cat ent.trace
        rm ent.trace
        exit 1
    else
        echo "success"
	rm -f ent.trace
    fi
}

checkretstrpattern () {
    PATTERN="$1"
    shift
    echo -n "checking pattern \"$PATTERN\" in retstr of $@ ... "
    "$@" > ent.trace 2>&1
    grep $PATTERN ent.trace > /dev/null 2>&1
    if test $? -ne 0; then
        echo "fail pattern not found in trace:"
        cat ent.trace
        rm ent.trace
        exit 1
    else
        echo "success"
	rm -f ent.trace
    fi
}

exec_n_test_fail () {
    echo -n "checking failure of $@ ... "
    "$@" > ent.trace 2>&1 
    if test $? != "0"; then
        echo "success"
        rm -f ent.trace
    else
        echo "not failed"
        cat ent.trace
        rm -f ent.trace 
        exit 1
    fi
}

sort_comma_list () {
  echo `echo "$@" | tr "," "\n" | sort | tr "\n" ","`
}

checkretlist () {
    # This is like checkretstr but it sorts both the expected and returned
    # comma-separated lists before checking
    EXPECTED_LIST=`sort_comma_list "$1"`
    shift
    exec_n_test "$@"
    echo -n "checking list returned in last command ... "
    RET_LIST=`sort_comma_list $RETSTR`
    if test "a$EXPECTED_LIST" != "a$RET_LIST"; then
        echo "fail with trace:"
        cat ent.trace
        rm ent.trace
        exit 1
    else
        echo "success"
	rm -f ent.trace
    fi
}

check_equal_files () {
    echo -n "checking equality of files $1 $2 ... "
    diff -q $1 $2 > /dev/null 2>&1 
    if test $? -eq 0; then
        echo "success"
    else
        echo "fail: files are different"
        exit 1
    fi
}

check_diff_files () {
    echo -n "checking diffs of files $1 $2 ... "
    diff -q $1 $2  > /dev/null 2>&1 
    if test $? -eq 0; then
        echo "fail: files are equal"
        exit 1
    else
        echo "success"
    fi
}

check_exist_file () {
    echo -n "checking existence of $1 ... "
    if test -f $1; then
        echo "success"
    else
        echo "fail: the file does not exist: $1"
        exit 1
    fi
}

check_empty_file () {
    echo -n "checking emptyness of $1 ... "
    nbc=`cat $1 | wc -c`
    if test ${nbc} -eq 0; then
        echo "success"
    else
        echo "fail: the file has size: ${nbc}"
        exit 1
    fi
}

check_file_size () {
    echo -n "checking file size of $2 ... "
    size=`wc -c "$2" | awk '{ print $1 }'`
    if test $size -eq $1; then
        echo "success"
    else
        echo "fail: sizes differ waiting for $1 file is $size"
        exit 1
    fi
}

check_not_empty_file () {
    echo -n "checking non emptyness of $1 ... "
    nbc=`cat $1 | wc -c`
    if test ${nbc} != "0"; then
        echo "success"
    else
        echo "fail: the file is empty or not exists"
        exit 1
    fi
}

check_equal_values () {
    echo -n "checking equality of $1 $2 ... "
    if test "$1" = "$2"; then
        echo "success"
    else
        echo "fail: values are different"
        exit 1
    fi
}  

check_pattern () {
    echo -n "checking if pattern $1 is found in argument ... "
    PATTERN="$1"
    shift
    echo "$@" | grep "$PATTERN" > /dev/null 2>&1
    if test $? != "0"; then
	echo "fail: pattern not found"
	exit 1
    else
	echo "success"
    fi
}

check_pattern_in_file () {
    echo -n "checking if pattern $1 is found in $2 ... "
    grep "$1" "$2" > /dev/null 2>&1
    if test $? != "0"; then
	echo "fail: pattern not found"
	exit 1
    else
	echo "success"
    fi
}

clean_all () {
    kill_pyrame.sh > /dev/null 2>&1 
    rm -f *.trace running*.*raw test*.*raw > /dev/null 2>&1 
    sleep 1s
}

wait_until_awake () {
    i=0
    waittime=1
    chksock.py "$1" "$2"
    last=$?
    while test $i -lt 5; do
        if test $last -eq 1; then
            echo "waiting ${waittime}s for $1:$2 to open ... "
            sleep ${waittime}s
            waittime=$(($waittime*2))
        else
            return
        fi
        chksock.py $1 $2
        last=$?
        i=$(($i+1))
    done
    exit 1
}

get_param_json () {
  param=$1
  shift
  echo "$@" | python -c "import json,sys;obj=json.load(sys.stdin);print obj[\"$param\"]"
}

