#!/bin/sh
# -*- coding: utf-8 -*-
# 
# Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
# This file is part of Pyrame.
. /usr/local/bin/unit_test_lib.sh
. /opt/pyrame/ports.sh

clean_all
echo 
echo 
echo "VARMOD TESTS"
echo
cmdmod /opt/pyrame/cmd_varmod.xml > server.trace 2>&1 &
wait_until_awake localhost $VARMOD_PORT

exec_n_test ./setvar.sh "id" default x 2
checkretstr 2 ./getvar.sh "id" default x

echo "init workspace"
exec_n_test ./newns.sh "id" testws
exec_n_test_fail ./newns.sh "id" testws

echo "simple get/set/del"
exec_n_test ./setvar.sh "id" testws x 2
checkretstr 2 ./getvar.sh "id" testws x
exec_n_test ./rmvar.sh "id" testws x
exec_n_test_fail ./getvar.sh "id" testws x

echo "unknown workspace"
exec_n_test_fail ./setvar.sh "id" toto x 2
exec_n_test_fail ./getvar.sh "id" toto x
exec_n_test_fail ./rmvar.sh "id" toto x
exec_n_test_fail ./intopvar.sh "id" toto x + 2
exec_n_test_fail ./flopvar.sh "id" toto x + 2.0
exec_n_test_fail ./stropvar.sh "id" toto x c "hello"
exec_n_test_fail ./get_token.sh "id" toto tok
exec_n_test_fail ./release_token.sh "id" toto tok

echo "integer operations"
exec_n_test ./setvar.sh "id" testws i 2
exec_n_test ./intopvar.sh "id" testws i + 2
checkretstr 4 ./getvar.sh "id" testws i
exec_n_test ./intopvar.sh "id" testws i - 2
checkretstr 2 ./getvar.sh "id" testws i
exec_n_test ./intopvar.sh "id" testws i x 2
checkretstr 4 ./getvar.sh "id" testws i
exec_n_test ./intopvar.sh "id" testws i / 2
checkretstr 2 ./getvar.sh "id" testws i
echo "autocreation of int variables"
exec_n_test ./rmvar.sh "id" testws i
exec_n_test ./intopvar.sh "id" testws i + 2
checkretstr 2 ./getvar.sh "id" testws i
echo "forbidden/unknown int operations"
exec_n_test_fail ./intopvar.sh "id" testws i / 0
exec_n_test_fail ./intopvar.sh "id" testws i Z 0
exec_n_test ./setvar.sh "id" testws s "toto"
exec_n_test_fail ./intopvar.sh "id" testws s + 2
exec_n_test_fail ./intopvar.sh "id" testws i + "toto"
echo "intop cleanup"
exec_n_test ./rmvar.sh "id" testws i
exec_n_test ./rmvar.sh "id" testws s

echo "float operations"
exec_n_test ./setvar.sh "id" testws f 2.5
exec_n_test ./flopvar.sh "id" testws f + 2.6
checkretstr 5.1 ./getvar.sh "id" testws f
exec_n_test ./flopvar.sh "id" testws f - 2.8
checkretstr 2.3 ./getvar.sh "id" testws f
exec_n_test ./flopvar.sh "id" testws f x 2
checkretstr 4.6 ./getvar.sh "id" testws f
exec_n_test ./flopvar.sh "id" testws f / 2.3
checkretstr 2.0 ./getvar.sh "id" testws f
echo "autocreation of float variables"
exec_n_test ./rmvar.sh "id" testws f
exec_n_test ./flopvar.sh "id" testws f + 2.0
checkretstr 2.0 ./getvar.sh "id" testws f
echo "forbidden/unknown float operations"
exec_n_test_fail ./flopvar.sh "id" testws f / 0.0
exec_n_test ./rmvar.sh "id" testws f
exec_n_test_fail ./flopvar.sh "id" testws f Z 0.0

echo "string operations"
exec_n_test ./setvar.sh "id" testws s toto
checkretstr toto ./getvar.sh "id" testws s
exec_n_test ./stropvar.sh "id" testws s c titi 
checkretstr tototiti ./getvar.sh "id" testws s
exec_n_test ./rmvar.sh "id" testws s
echo "forbidden/unknown string operations"
exec_n_test_fail ./stropvar.sh "id" testws s Z ""

echo "token operations"
exec_n_test ./get_token.sh "id" testws tok
exec_n_test_fail ./get_token.sh "id" testws tok
exec_n_test ./release_token.sh "id" testws tok
exec_n_test_fail ./release_token.sh "id" testws tok
exec_n_test ./release_token.sh "id" testws tok2
exec_n_test ./get_token.sh "id" testws tok2
exec_n_test ./release_token.sh "id" testws tok2
exec_n_test ./del_token.sh "id" testws tok
exec_n_test ./del_token.sh "id" testws tok2

echo "data source operations"
checkretstr "${CD_BASE_PORT},${CD_READY_BASE_PORT}" ./register_ds.sh "id" testws test 127.0.0.1 undef undef "!a|b|c"
exec_n_test_fail ./register_ds.sh "id" testws test 127.0.0.1 undef undef
checkretstr "9000,23001" ./register_ds.sh "id" testws test2 127.0.0.1 9000 undef "!a|b|c" 
checkretstr "127.0.0.1,${CD_BASE_PORT},${CD_READY_BASE_PORT}" ./get_ds.sh "id" testws test
checkretstr "127.0.0.1,9000,23001" ./get_ds.sh "id" testws test2
checkretstr "a,b,c" ./get_ds_fields.sh "id" testws test2
checkretstr "test,test2" ./find_ds.sh "id" testws test
exec_n_test ./rm_ds.sh "id" testws test
exec_n_test_fail ./get_ds.sh "id" testws test
exec_n_test ./rm_ds.sh "id" testws test2

echo "environment reloading"
exec_n_test ./setvar.sh "id" testws x 2
cp /tmp/varmod.env /tmp/oldenv
clean_all
cmdmod /opt/pyrame/cmd_varmod.xml -e /tmp/oldenv > server.trace 2>&1 &
wait_until_awake localhost $VARMOD_PORT
checkretstr 2 ./getvar.sh "id" testws x
 

echo "delns"
exec_n_test ./delns.sh "id" testws
exec_n_test_fail ./delns.sh "id" toto

if test a$1 != "anonit"
then
    clean_all
fi

clean_all
echo "all tests passed"
