/*
Copyright 2012-2015 Frédéric Magniette, Miguel Rubio-Roy
This file is part of Pyrame.

Pyrame is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Pyrame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with Pyrame.  If not, see <http://www.gnu.org/licenses/>
*/

#include "../libpyrame/pyrame.h"

struct httpcmd {
  char *host;
  char *port;
  struct cmd *command;
};

char *pyrame_return(unsigned char retcode,char *retstr) {
  int i,j=0;
  int num_escapes=0;
  char *escaped_retstr;
  for (i=0;i<strlen(retstr);i++) {
    if (retstr[i]=='"') {
      num_escapes++;
    }
  }
  if (num_escapes!=0) {
    escaped_retstr=malloc(strlen(retstr)+num_escapes+1);
    for (i=0;i<strlen(retstr);i++) {
      if (retstr[i]=='"') {
        escaped_retstr[j]='\\';
        j++;
        escaped_retstr[j]='"';
      } else {
        escaped_retstr[j]=retstr[i];
      }
      j++;
    }
    escaped_retstr[j]=0;
  } else {
    escaped_retstr=retstr;
  }
  printf("{\"retcode\": %u, \"res\": \"%s\"}\n",retcode,escaped_retstr);
  exit(!retcode);
}

char *get_request() {
  char *request,*len_s,*junk;
  int len;
  len_s=getenv("CONTENT_LENGTH");
  if (len_s==NULL) {
    pyrame_return(0,"invalid HTML request. Only POST requests are allowed");
  }
  len=strtol(len_s,&junk,10);
  if (junk[0]!='\0' || len_s[0]=='\0') {
    pyrame_return(0,"invalid HTML request");
  }
  request=malloc(len+1);
  fgets(request,len+1,stdin);
  return request;
}

struct httpcmd *parse_request(char *request) {
  struct httpcmd *httpcommand;
  const char * const min_params[]={"host","port","func"};
  const int nb_min_params=3;
  char **params;
  char *param_name,param_name_searched[20];
  int i,j;
  int req_size=strlen(request);

  // Count number of parameters
  j=1;
  for (i=0;i<strlen(request);i++) {
    if (request[i]=='&') {
      j++;
    }
  }
  if (j<nb_min_params) {
    pyrame_return(0,"wrong request syntax: too few parameters");
  } 

  // Allocate structure
  httpcommand=malloc(sizeof(struct httpcmd));
  httpcommand->command=malloc(sizeof(struct cmd));
  httpcommand->command->nb_params=j-nb_min_params;
  httpcommand->command->params=malloc(sizeof(char*)*httpcommand->command->nb_params);

  // Fill in the mandatory parameters
  i=0;
  params=malloc(sizeof(char*)*nb_min_params);
  for (j=0;j<nb_min_params;j++) {
    param_name=request+i;
    while (request[i]!='=' && i<req_size) {
      i++;
    }
    request[i]=0;
    if (strcmp(param_name,min_params[j])) {
      pyrame_return(0,"wrong request syntax: appropriate param not found");
    }
    i++;
    params[j]=request+i;
    while (request[i]!='&' && i<req_size) {
      i++;
    }
    request[i]=0;
    i++;
  }
  httpcommand->host=params[0];
  httpcommand->port=params[1];
  httpcommand->command->name=params[2];

  // Fill in the optional parameters
  j=0;
  while (j<httpcommand->command->nb_params) {
    snprintf(param_name_searched,20,"param%d",j+1);
    param_name=request+i;
    while (request[i]!='=' && i<req_size) {
      i++;
    }
    request[i]=0;
    if (strcmp(param_name,param_name_searched)) {
      pyrame_return(0,"wrong request syntax: appropriate param not found");
    }
    i++;
    httpcommand->command->params[j]=request+i;
    while (request[i]!='&' && i<req_size) {
      i++;
    }
    request[i]=0;
    i++;
    j++;
  }

  return httpcommand;
}

void decode_params(struct cmd *cmd) {
  char *decoded_param,*junk;
  char encoded[3];
  int nb_encodes;
  int i,j,k;
  for (i=0;i<cmd->nb_params;i++) {
    //printf("cmd->params[%d]=%s\n",i,cmd->params[i]);
    nb_encodes=0;
    for (j=0;j<strlen(cmd->params[i]);j++) {
      if (cmd->params[i][j]=='%') {
        nb_encodes++;
      }
    }
    //printf("nb_encodes=%d\n",nb_encodes);
    decoded_param=malloc(strlen(cmd->params[i])-2*nb_encodes+1);
    encoded[2]=0;
    k=0;
    for (j=0;j<strlen(cmd->params[i]);j++) {
      //printf("%c",cmd->params[i][j]);
      if (cmd->params[i][j]=='%') {
        j++;
        //printf("%c",cmd->params[i][j]);
        encoded[0]=cmd->params[i][j];
        j++;
        //printf("%c",cmd->params[i][j]);
        encoded[1]=cmd->params[i][j];
        decoded_param[k]=strtol(encoded,&junk,16);
        if (junk[0]!='\0' || encoded[0]=='\0') {
          //printf("\nencoded=%s\n",encoded);
          pyrame_return(0,"invalid HTML encoding");
        }
      } else {
        decoded_param[k]=cmd->params[i][j];
      }
      k++;
    }
    //printf("\n");
    cmd->params[i]=decoded_param;
  }
}

int main() {
  char *request;
  struct httpcmd *httpcommand;
  struct cmd_result *result;
  int port;

  printf("Content-type: application/json\n\n");

  request=get_request();
  //printf("len:%ld\n",len);
  //pyrame_return(1,request);
  httpcommand=parse_request(request);
  decode_params(httpcommand->command);
/*
  fprintf(stderr,"host=%s\n",httpcommand->host);
  fprintf(stderr,"port=%s\n",httpcommand->port);
  fprintf(stderr,"command->name=%s\n",httpcommand->command->name);
  int i;
  for (i=0;i<httpcommand->command->nb_params;i++) {
    fprintf(stderr,"command->params%d=%s\n",i,httpcommand->command->params[i]);
  }
*/
  port=get_port(httpcommand->port);
  result=send_cmd(httpcommand->host,port,httpcommand->command);
  pyrame_return(result->retcode,result->str);
  return 0;
}
